import { address, createEncoder, getAddressEncoder, createDecoder, getAddressDecoder, combineCodec, getProgramDerivedAddress, getUtf8Encoder, getU8Encoder, getAddressFromPublicKey, generateKeyPair, getStructEncoder, getArrayEncoder, getStructDecoder, getArrayDecoder, getEnumEncoder, getEnumDecoder, transformEncoder, getOptionEncoder, getU64Encoder, getU32Encoder, getI64Encoder, getOptionDecoder, getU64Decoder, getU32Decoder, getI64Decoder, getDiscriminatedUnionEncoder, getTupleEncoder, getDiscriminatedUnionDecoder, getTupleDecoder, fixEncoderSize, getBytesEncoder, fixDecoderSize, getBytesDecoder, addEncoderSizePrefix, addDecoderSizePrefix, getUtf8Decoder, getBooleanEncoder, getBooleanDecoder, getU8Decoder, decodeAccount, assertAccountExists, fetchEncodedAccount, assertAccountsExist, fetchEncodedAccounts, containsBytes, none, getU16Encoder, getU16Decoder, AccountRole, isTransactionSigner as isTransactionSigner$1, upgradeRoleToSigner } from '@solana/web3.js';
import { resolveBuyerAta, resolveListAta, resolveMetadata, TokenStandard, resolveEditionFromTokenStandard, resolveBuyerTokenRecordFromTokenStandard, resolveListTokenRecordFromTokenStandard, resolveAuthorizationRulesProgramFromTokenStandard, resolveTokenMetadataProgramFromTokenStandard, resolveSysvarInstructionsFromTokenStandard, resolveWnsApprovePda, resolveWnsDistributionPda, resolveWnsExtraAccountMetasPda, resolveOwnerAta, resolveOwnerTokenRecordFromTokenStandard, resolveSellerAta, resolveSellerTokenRecordFromTokenStandard, resolveBidTokenRecordFromTokenStandard } from '@tensor-foundation/resolvers';

// src/generated/accounts/bidState.ts
var DEFAULT_ADDRESS = address("11111111111111111111111111111111");
var getNullableAddressEncoder = () => createEncoder({
  fixedSize: 32,
  write(value, bytes, offset) {
    if (value === null) {
      bytes.set(getAddressEncoder().encode(DEFAULT_ADDRESS), offset);
    } else {
      bytes.set(getAddressEncoder().encode(value), offset);
    }
    return offset + 32;
  }
});
var getNullableAddressDecoder = () => createDecoder({
  fixedSize: 32,
  read(bytes, offset) {
    if (getAddressDecoder().decode(bytes, offset) === DEFAULT_ADDRESS) {
      return [null, offset + 32];
    } else {
      return [getAddressDecoder().decode(bytes, offset), offset + 32];
    }
  }
});
var getNullableAddressCodec = () => combineCodec(getNullableAddressEncoder(), getNullableAddressDecoder());
async function findTreeAuthorityPda(seeds, config = {}) {
  const {
    programAddress = "BGUMAp9Gq7iTEuizy4pqaxsTyUCBK68MDfK752saRPUY"
  } = config;
  return await getProgramDerivedAddress({
    programAddress,
    seeds: [getAddressEncoder().encode(seeds.merkleTree)]
  });
}
async function findFeeVaultPda(seeds) {
  const bytes = getAddressEncoder().encode(seeds.address);
  const lastByte = bytes[bytes.length - 1];
  return await getProgramDerivedAddress({
    programAddress: address("TFEEgwDP6nn1s8mMX2tTNPPz8j2VomkphLUmyxKm17A"),
    seeds: [
      getUtf8Encoder().encode("fee_vault"),
      getU8Encoder().encode(lastByte)
    ]
  });
}
function expectSome(value) {
  if (value == null) {
    throw new Error("Expected a value but received null or undefined.");
  }
  return value;
}
function expectAddress(value) {
  if (!value) {
    throw new Error("Expected a Address.");
  }
  if (typeof value === "object" && "address" in value) {
    return value.address;
  }
  if (Array.isArray(value)) {
    return value[0];
  }
  return value;
}
function expectTransactionSigner(value) {
  if (!value || !isTransactionSigner(value)) {
    throw new Error("Expected a TransactionSigner.");
  }
  return value;
}
function getAccountMetaFactory(programAddress, optionalAccountStrategy) {
  return (account) => {
    if (!account.value) {
      return Object.freeze({
        address: programAddress,
        role: AccountRole.READONLY
      });
    }
    const writableRole = account.isWritable ? AccountRole.WRITABLE : AccountRole.READONLY;
    return Object.freeze({
      address: expectAddress(account.value),
      role: isTransactionSigner(account.value) ? upgradeRoleToSigner(writableRole) : writableRole,
      ...isTransactionSigner(account.value) ? { signer: account.value } : {}
    });
  };
}
function isTransactionSigner(value) {
  return !!value && typeof value === "object" && "address" in value && isTransactionSigner$1(value);
}

// src/hooked/resolvers.ts
var resolveBidIdOnCreate = async ({
  args
}) => {
  if (expectSome(args.target) === 0 /* AssetId */)
    return await Promise.resolve(expectAddress(args.targetId));
  return await getAddressFromPublicKey((await generateKeyPair()).publicKey);
};
var resolveFeeVaultPdaFromListState = async ({
  accounts
}) => {
  return {
    value: await findFeeVaultPda({
      address: expectAddress(accounts.listState?.value)
    })
  };
};
var resolveFeeVaultPdaFromBidState = async ({
  accounts
}) => {
  return {
    value: await findFeeVaultPda({
      address: expectAddress(accounts.bidState?.value)
    })
  };
};
var resolveTreeAuthorityPda = async ({
  accounts
}) => {
  return {
    value: await findTreeAuthorityPda(
      {
        merkleTree: expectAddress(accounts.merkleTree?.value)
      },
      { programAddress: expectAddress(accounts.bubblegumProgram?.value) }
    )
  };
};
var resolveCreatorPath = ({
  args
}) => {
  const creators = args.creators ?? [];
  return creators.map(([creator, share]) => ({
    address: creator,
    role: +(share > 0)
  }));
};
var resolveProofPath = ({
  args
}) => {
  const proof = args.proof ?? [];
  const canopyDepth = args.canopyDepth ?? 0;
  return proof.slice(0, proof.length - canopyDepth).map((proof2) => ({
    address: proof2,
    role: 0
  }));
};
var resolveRemainingSignerWithOwnerOrDelegate = ({
  accounts
}) => {
  if (!!accounts.owner.value && isTransactionSigner(accounts.owner.value)) {
    return {
      value: accounts.owner.value
    };
  }
  if (!!accounts.delegate.value && isTransactionSigner(accounts.delegate.value))
    return {
      value: accounts.delegate.value
    };
  throw new Error(
    "Either owner or delegate has to be provided as TransactionSigner."
  );
};
var resolveRemainingSignerWithSellerOrDelegate = ({
  accounts
}) => {
  if (!!accounts.seller.value && isTransactionSigner(accounts.seller.value)) {
    return {
      value: accounts.seller.value
    };
  }
  if (!!accounts.delegate.value && isTransactionSigner(accounts.delegate.value))
    return {
      value: accounts.delegate.value
    };
  throw new Error(
    "Either seller or delegate has to be provided as TransactionSigner."
  );
};
var resolveBidTa = async ({
  accounts
}) => {
  return {
    value: await findBidTaPda({
      mint: expectAddress(accounts.mint?.value)
    })
  };
};
async function findBidStatePda(seeds, config = {}) {
  const {
    programAddress = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp"
  } = config;
  return await getProgramDerivedAddress({
    programAddress,
    seeds: [
      getUtf8Encoder().encode("bid_state"),
      getAddressEncoder().encode(seeds.owner),
      getAddressEncoder().encode(seeds.bidId)
    ]
  });
}
async function findBidTaPda(seeds, config = {}) {
  const {
    programAddress = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp"
  } = config;
  return await getProgramDerivedAddress({
    programAddress,
    seeds: [
      getUtf8Encoder().encode("nft_escrow"),
      getAddressEncoder().encode(seeds.mint)
    ]
  });
}
async function findListStatePda(seeds, config = {}) {
  const {
    programAddress = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp"
  } = config;
  return await getProgramDerivedAddress({
    programAddress,
    seeds: [
      getUtf8Encoder().encode("list_state"),
      getAddressEncoder().encode(seeds.mint)
    ]
  });
}
function getAuthorizationDataLocalEncoder() {
  return getStructEncoder([
    ["payload", getArrayEncoder(getTaggedPayloadEncoder())]
  ]);
}
function getAuthorizationDataLocalDecoder() {
  return getStructDecoder([
    ["payload", getArrayDecoder(getTaggedPayloadDecoder())]
  ]);
}
function getAuthorizationDataLocalCodec() {
  return combineCodec(
    getAuthorizationDataLocalEncoder(),
    getAuthorizationDataLocalDecoder()
  );
}
var Field = /* @__PURE__ */ ((Field2) => {
  Field2[Field2["Name"] = 0] = "Name";
  return Field2;
})(Field || {});
function getFieldEncoder() {
  return getEnumEncoder(Field);
}
function getFieldDecoder() {
  return getEnumDecoder(Field);
}
function getFieldCodec() {
  return combineCodec(getFieldEncoder(), getFieldDecoder());
}
function getMakeEventEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["maker", getAddressEncoder()],
      ["bidId", getOptionEncoder(getAddressEncoder())],
      ["target", getTargetEncoder()],
      ["targetId", getAddressEncoder()],
      ["field", getOptionEncoder(getFieldEncoder())],
      ["fieldId", getOptionEncoder(getAddressEncoder())],
      ["amount", getU64Encoder()],
      ["quantity", getU32Encoder()],
      ["currency", getOptionEncoder(getAddressEncoder())],
      ["expiry", getI64Encoder()],
      ["privateTaker", getOptionEncoder(getAddressEncoder())],
      ["assetId", getOptionEncoder(getAddressEncoder())]
    ]),
    (value) => ({ ...value, quantity: value.quantity ?? 1 })
  );
}
function getMakeEventDecoder() {
  return getStructDecoder([
    ["maker", getAddressDecoder()],
    ["bidId", getOptionDecoder(getAddressDecoder())],
    ["target", getTargetDecoder()],
    ["targetId", getAddressDecoder()],
    ["field", getOptionDecoder(getFieldDecoder())],
    ["fieldId", getOptionDecoder(getAddressDecoder())],
    ["amount", getU64Decoder()],
    ["quantity", getU32Decoder()],
    ["currency", getOptionDecoder(getAddressDecoder())],
    ["expiry", getI64Decoder()],
    ["privateTaker", getOptionDecoder(getAddressDecoder())],
    ["assetId", getOptionDecoder(getAddressDecoder())]
  ]);
}
function getMakeEventCodec() {
  return combineCodec(getMakeEventEncoder(), getMakeEventDecoder());
}
function getPayloadTypeLocalEncoder() {
  return getDiscriminatedUnionEncoder([
    [
      "Pubkey",
      getStructEncoder([["fields", getTupleEncoder([getAddressEncoder()])]])
    ],
    [
      "Seeds",
      getStructEncoder([
        ["fields", getTupleEncoder([getSeedsVecLocalEncoder()])]
      ])
    ],
    [
      "MerkleProof",
      getStructEncoder([
        ["fields", getTupleEncoder([getProofInfoLocalEncoder()])]
      ])
    ],
    [
      "Number",
      getStructEncoder([["fields", getTupleEncoder([getU64Encoder()])]])
    ]
  ]);
}
function getPayloadTypeLocalDecoder() {
  return getDiscriminatedUnionDecoder([
    [
      "Pubkey",
      getStructDecoder([["fields", getTupleDecoder([getAddressDecoder()])]])
    ],
    [
      "Seeds",
      getStructDecoder([
        ["fields", getTupleDecoder([getSeedsVecLocalDecoder()])]
      ])
    ],
    [
      "MerkleProof",
      getStructDecoder([
        ["fields", getTupleDecoder([getProofInfoLocalDecoder()])]
      ])
    ],
    [
      "Number",
      getStructDecoder([["fields", getTupleDecoder([getU64Decoder()])]])
    ]
  ]);
}
function getPayloadTypeLocalCodec() {
  return combineCodec(
    getPayloadTypeLocalEncoder(),
    getPayloadTypeLocalDecoder()
  );
}
function payloadTypeLocal(kind, data) {
  return Array.isArray(data) ? { __kind: kind, fields: data } : { __kind: kind, ...data ?? {} };
}
function isPayloadTypeLocal(kind, value) {
  return value.__kind === kind;
}
function getProofInfoLocalEncoder() {
  return getStructEncoder([
    ["proof", getArrayEncoder(fixEncoderSize(getBytesEncoder(), 32))]
  ]);
}
function getProofInfoLocalDecoder() {
  return getStructDecoder([
    ["proof", getArrayDecoder(fixDecoderSize(getBytesDecoder(), 32))]
  ]);
}
function getProofInfoLocalCodec() {
  return combineCodec(getProofInfoLocalEncoder(), getProofInfoLocalDecoder());
}
function getSeedsVecLocalEncoder() {
  return getStructEncoder([
    [
      "seeds",
      getArrayEncoder(addEncoderSizePrefix(getBytesEncoder(), getU32Encoder()))
    ]
  ]);
}
function getSeedsVecLocalDecoder() {
  return getStructDecoder([
    [
      "seeds",
      getArrayDecoder(addDecoderSizePrefix(getBytesDecoder(), getU32Decoder()))
    ]
  ]);
}
function getSeedsVecLocalCodec() {
  return combineCodec(getSeedsVecLocalEncoder(), getSeedsVecLocalDecoder());
}
function getTaggedPayloadEncoder() {
  return getStructEncoder([
    ["name", addEncoderSizePrefix(getUtf8Encoder(), getU32Encoder())],
    ["payload", getPayloadTypeLocalEncoder()]
  ]);
}
function getTaggedPayloadDecoder() {
  return getStructDecoder([
    ["name", addDecoderSizePrefix(getUtf8Decoder(), getU32Decoder())],
    ["payload", getPayloadTypeLocalDecoder()]
  ]);
}
function getTaggedPayloadCodec() {
  return combineCodec(getTaggedPayloadEncoder(), getTaggedPayloadDecoder());
}
function getTakeEventEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["taker", getAddressEncoder()],
      ["bidId", getOptionEncoder(getAddressEncoder())],
      ["target", getTargetEncoder()],
      ["targetId", getAddressEncoder()],
      ["field", getOptionEncoder(getFieldEncoder())],
      ["fieldId", getOptionEncoder(getAddressEncoder())],
      ["amount", getU64Encoder()],
      ["quantity", getU32Encoder()],
      ["tcompFee", getU64Encoder()],
      ["takerBrokerFee", getU64Encoder()],
      ["makerBrokerFee", getU64Encoder()],
      ["creatorFee", getU64Encoder()],
      ["currency", getOptionEncoder(getAddressEncoder())],
      ["assetId", getOptionEncoder(getAddressEncoder())]
    ]),
    (value) => ({ ...value, quantity: value.quantity ?? 1 })
  );
}
function getTakeEventDecoder() {
  return getStructDecoder([
    ["taker", getAddressDecoder()],
    ["bidId", getOptionDecoder(getAddressDecoder())],
    ["target", getTargetDecoder()],
    ["targetId", getAddressDecoder()],
    ["field", getOptionDecoder(getFieldDecoder())],
    ["fieldId", getOptionDecoder(getAddressDecoder())],
    ["amount", getU64Decoder()],
    ["quantity", getU32Decoder()],
    ["tcompFee", getU64Decoder()],
    ["takerBrokerFee", getU64Decoder()],
    ["makerBrokerFee", getU64Decoder()],
    ["creatorFee", getU64Decoder()],
    ["currency", getOptionDecoder(getAddressDecoder())],
    ["assetId", getOptionDecoder(getAddressDecoder())]
  ]);
}
function getTakeEventCodec() {
  return combineCodec(getTakeEventEncoder(), getTakeEventDecoder());
}
var Target = /* @__PURE__ */ ((Target2) => {
  Target2[Target2["AssetId"] = 0] = "AssetId";
  Target2[Target2["Whitelist"] = 1] = "Whitelist";
  return Target2;
})(Target || {});
function getTargetEncoder() {
  return getEnumEncoder(Target);
}
function getTargetDecoder() {
  return getEnumDecoder(Target);
}
function getTargetCodec() {
  return combineCodec(getTargetEncoder(), getTargetDecoder());
}
function getTCollectionEncoder() {
  return getStructEncoder([
    ["verified", getBooleanEncoder()],
    ["key", getAddressEncoder()]
  ]);
}
function getTCollectionDecoder() {
  return getStructDecoder([
    ["verified", getBooleanDecoder()],
    ["key", getAddressDecoder()]
  ]);
}
function getTCollectionCodec() {
  return combineCodec(getTCollectionEncoder(), getTCollectionDecoder());
}
function getTcompEventEncoder() {
  return getDiscriminatedUnionEncoder([
    [
      "Maker",
      getStructEncoder([["fields", getTupleEncoder([getMakeEventEncoder()])]])
    ],
    [
      "Taker",
      getStructEncoder([["fields", getTupleEncoder([getTakeEventEncoder()])]])
    ]
  ]);
}
function getTcompEventDecoder() {
  return getDiscriminatedUnionDecoder([
    [
      "Maker",
      getStructDecoder([["fields", getTupleDecoder([getMakeEventDecoder()])]])
    ],
    [
      "Taker",
      getStructDecoder([["fields", getTupleDecoder([getTakeEventDecoder()])]])
    ]
  ]);
}
function getTcompEventCodec() {
  return combineCodec(getTcompEventEncoder(), getTcompEventDecoder());
}
function tcompEvent(kind, data) {
  return Array.isArray(data) ? { __kind: kind, fields: data } : { __kind: kind, ...data ?? {} };
}
function isTcompEvent(kind, value) {
  return value.__kind === kind;
}
var TTokenProgramVersion = /* @__PURE__ */ ((TTokenProgramVersion2) => {
  TTokenProgramVersion2[TTokenProgramVersion2["Original"] = 0] = "Original";
  TTokenProgramVersion2[TTokenProgramVersion2["Token2022"] = 1] = "Token2022";
  return TTokenProgramVersion2;
})(TTokenProgramVersion || {});
function getTTokenProgramVersionEncoder() {
  return getEnumEncoder(TTokenProgramVersion);
}
function getTTokenProgramVersionDecoder() {
  return getEnumDecoder(TTokenProgramVersion);
}
function getTTokenProgramVersionCodec() {
  return combineCodec(
    getTTokenProgramVersionEncoder(),
    getTTokenProgramVersionDecoder()
  );
}
var TTokenStandard = /* @__PURE__ */ ((TTokenStandard2) => {
  TTokenStandard2[TTokenStandard2["NonFungible"] = 0] = "NonFungible";
  TTokenStandard2[TTokenStandard2["FungibleAsset"] = 1] = "FungibleAsset";
  TTokenStandard2[TTokenStandard2["Fungible"] = 2] = "Fungible";
  TTokenStandard2[TTokenStandard2["NonFungibleEdition"] = 3] = "NonFungibleEdition";
  return TTokenStandard2;
})(TTokenStandard || {});
function getTTokenStandardEncoder() {
  return getEnumEncoder(TTokenStandard);
}
function getTTokenStandardDecoder() {
  return getEnumDecoder(TTokenStandard);
}
function getTTokenStandardCodec() {
  return combineCodec(getTTokenStandardEncoder(), getTTokenStandardDecoder());
}
var TUseMethod = /* @__PURE__ */ ((TUseMethod2) => {
  TUseMethod2[TUseMethod2["Burn"] = 0] = "Burn";
  TUseMethod2[TUseMethod2["Multiple"] = 1] = "Multiple";
  TUseMethod2[TUseMethod2["Single"] = 2] = "Single";
  return TUseMethod2;
})(TUseMethod || {});
function getTUseMethodEncoder() {
  return getEnumEncoder(TUseMethod);
}
function getTUseMethodDecoder() {
  return getEnumDecoder(TUseMethod);
}
function getTUseMethodCodec() {
  return combineCodec(getTUseMethodEncoder(), getTUseMethodDecoder());
}
function getTUsesEncoder() {
  return getStructEncoder([
    ["useMethod", getTUseMethodEncoder()],
    ["remaining", getU64Encoder()],
    ["total", getU64Encoder()]
  ]);
}
function getTUsesDecoder() {
  return getStructDecoder([
    ["useMethod", getTUseMethodDecoder()],
    ["remaining", getU64Decoder()],
    ["total", getU64Decoder()]
  ]);
}
function getTUsesCodec() {
  return combineCodec(getTUsesEncoder(), getTUsesDecoder());
}

// src/generated/accounts/bidState.ts
function getBidStateEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["version", getU8Encoder()],
      ["bump", getArrayEncoder(getU8Encoder(), { size: 1 })],
      ["owner", getAddressEncoder()],
      ["bidId", getAddressEncoder()],
      ["target", getTargetEncoder()],
      ["targetId", getAddressEncoder()],
      ["field", getOptionEncoder(getFieldEncoder())],
      ["fieldId", getOptionEncoder(getAddressEncoder())],
      ["quantity", getU32Encoder()],
      ["filledQuantity", getU32Encoder()],
      ["amount", getU64Encoder()],
      ["currency", getOptionEncoder(getAddressEncoder())],
      ["expiry", getI64Encoder()],
      ["privateTaker", getOptionEncoder(getAddressEncoder())],
      ["makerBroker", getOptionEncoder(getAddressEncoder())],
      ["margin", getOptionEncoder(getAddressEncoder())],
      ["updatedAt", getI64Encoder()],
      ["cosigner", getNullableAddressEncoder()],
      ["rentPayer", getNullableAddressEncoder()],
      ["reserved", getArrayEncoder(getU8Encoder(), { size: 8 })],
      ["reserved1", getArrayEncoder(getU8Encoder(), { size: 16 })],
      ["reserved2", fixEncoderSize(getBytesEncoder(), 32)]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([155, 197, 5, 97, 189, 60, 8, 183]),
      quantity: value.quantity ?? 1
    })
  );
}
function getBidStateDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["version", getU8Decoder()],
    ["bump", getArrayDecoder(getU8Decoder(), { size: 1 })],
    ["owner", getAddressDecoder()],
    ["bidId", getAddressDecoder()],
    ["target", getTargetDecoder()],
    ["targetId", getAddressDecoder()],
    ["field", getOptionDecoder(getFieldDecoder())],
    ["fieldId", getOptionDecoder(getAddressDecoder())],
    ["quantity", getU32Decoder()],
    ["filledQuantity", getU32Decoder()],
    ["amount", getU64Decoder()],
    ["currency", getOptionDecoder(getAddressDecoder())],
    ["expiry", getI64Decoder()],
    ["privateTaker", getOptionDecoder(getAddressDecoder())],
    ["makerBroker", getOptionDecoder(getAddressDecoder())],
    ["margin", getOptionDecoder(getAddressDecoder())],
    ["updatedAt", getI64Decoder()],
    ["cosigner", getNullableAddressDecoder()],
    ["rentPayer", getNullableAddressDecoder()],
    ["reserved", getArrayDecoder(getU8Decoder(), { size: 8 })],
    ["reserved1", getArrayDecoder(getU8Decoder(), { size: 16 })],
    ["reserved2", fixDecoderSize(getBytesDecoder(), 32)]
  ]);
}
function getBidStateCodec() {
  return combineCodec(getBidStateEncoder(), getBidStateDecoder());
}
function decodeBidState(encodedAccount) {
  return decodeAccount(
    encodedAccount,
    getBidStateDecoder()
  );
}
async function fetchBidState(rpc, address3, config) {
  const maybeAccount = await fetchMaybeBidState(rpc, address3, config);
  assertAccountExists(maybeAccount);
  return maybeAccount;
}
async function fetchMaybeBidState(rpc, address3, config) {
  const maybeAccount = await fetchEncodedAccount(rpc, address3, config);
  return decodeBidState(maybeAccount);
}
async function fetchAllBidState(rpc, addresses, config) {
  const maybeAccounts = await fetchAllMaybeBidState(rpc, addresses, config);
  assertAccountsExist(maybeAccounts);
  return maybeAccounts;
}
async function fetchAllMaybeBidState(rpc, addresses, config) {
  const maybeAccounts = await fetchEncodedAccounts(rpc, addresses, config);
  return maybeAccounts.map((maybeAccount) => decodeBidState(maybeAccount));
}
async function fetchBidStateFromSeeds(rpc, seeds, config = {}) {
  const maybeAccount = await fetchMaybeBidStateFromSeeds(rpc, seeds, config);
  assertAccountExists(maybeAccount);
  return maybeAccount;
}
async function fetchMaybeBidStateFromSeeds(rpc, seeds, config = {}) {
  const { programAddress, ...fetchConfig } = config;
  const [address3] = await findBidStatePda(seeds, { programAddress });
  return await fetchMaybeBidState(rpc, address3, fetchConfig);
}
function getBidTaEncoder() {
  return transformEncoder(
    getStructEncoder([["discriminator", fixEncoderSize(getBytesEncoder(), 8)]]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([186, 230, 110, 26, 235, 24, 211, 156])
    })
  );
}
function getBidTaDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)]
  ]);
}
function getBidTaCodec() {
  return combineCodec(getBidTaEncoder(), getBidTaDecoder());
}
function decodeBidTa(encodedAccount) {
  return decodeAccount(
    encodedAccount,
    getBidTaDecoder()
  );
}
async function fetchBidTa(rpc, address3, config) {
  const maybeAccount = await fetchMaybeBidTa(rpc, address3, config);
  assertAccountExists(maybeAccount);
  return maybeAccount;
}
async function fetchMaybeBidTa(rpc, address3, config) {
  const maybeAccount = await fetchEncodedAccount(rpc, address3, config);
  return decodeBidTa(maybeAccount);
}
async function fetchAllBidTa(rpc, addresses, config) {
  const maybeAccounts = await fetchAllMaybeBidTa(rpc, addresses, config);
  assertAccountsExist(maybeAccounts);
  return maybeAccounts;
}
async function fetchAllMaybeBidTa(rpc, addresses, config) {
  const maybeAccounts = await fetchEncodedAccounts(rpc, addresses, config);
  return maybeAccounts.map((maybeAccount) => decodeBidTa(maybeAccount));
}
function getBidTaSize() {
  return 8;
}
async function fetchBidTaFromSeeds(rpc, seeds, config = {}) {
  const maybeAccount = await fetchMaybeBidTaFromSeeds(rpc, seeds, config);
  assertAccountExists(maybeAccount);
  return maybeAccount;
}
async function fetchMaybeBidTaFromSeeds(rpc, seeds, config = {}) {
  const { programAddress, ...fetchConfig } = config;
  const [address3] = await findBidTaPda(seeds, { programAddress });
  return await fetchMaybeBidTa(rpc, address3, fetchConfig);
}
function getListStateEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["version", getU8Encoder()],
      ["bump", getArrayEncoder(getU8Encoder(), { size: 1 })],
      ["owner", getAddressEncoder()],
      ["assetId", getAddressEncoder()],
      ["amount", getU64Encoder()],
      ["currency", getOptionEncoder(getAddressEncoder())],
      ["expiry", getI64Encoder()],
      ["privateTaker", getOptionEncoder(getAddressEncoder())],
      ["makerBroker", getOptionEncoder(getAddressEncoder())],
      ["rentPayer", getNullableAddressEncoder()],
      ["cosigner", getNullableAddressEncoder()],
      ["reserved1", fixEncoderSize(getBytesEncoder(), 64)]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([78, 242, 89, 138, 161, 221, 176, 75])
    })
  );
}
function getListStateDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["version", getU8Decoder()],
    ["bump", getArrayDecoder(getU8Decoder(), { size: 1 })],
    ["owner", getAddressDecoder()],
    ["assetId", getAddressDecoder()],
    ["amount", getU64Decoder()],
    ["currency", getOptionDecoder(getAddressDecoder())],
    ["expiry", getI64Decoder()],
    ["privateTaker", getOptionDecoder(getAddressDecoder())],
    ["makerBroker", getOptionDecoder(getAddressDecoder())],
    ["rentPayer", getNullableAddressDecoder()],
    ["cosigner", getNullableAddressDecoder()],
    ["reserved1", fixDecoderSize(getBytesDecoder(), 64)]
  ]);
}
function getListStateCodec() {
  return combineCodec(getListStateEncoder(), getListStateDecoder());
}
function decodeListState(encodedAccount) {
  return decodeAccount(
    encodedAccount,
    getListStateDecoder()
  );
}
async function fetchListState(rpc, address3, config) {
  const maybeAccount = await fetchMaybeListState(rpc, address3, config);
  assertAccountExists(maybeAccount);
  return maybeAccount;
}
async function fetchMaybeListState(rpc, address3, config) {
  const maybeAccount = await fetchEncodedAccount(rpc, address3, config);
  return decodeListState(maybeAccount);
}
async function fetchAllListState(rpc, addresses, config) {
  const maybeAccounts = await fetchAllMaybeListState(rpc, addresses, config);
  assertAccountsExist(maybeAccounts);
  return maybeAccounts;
}
async function fetchAllMaybeListState(rpc, addresses, config) {
  const maybeAccounts = await fetchEncodedAccounts(rpc, addresses, config);
  return maybeAccounts.map((maybeAccount) => decodeListState(maybeAccount));
}
async function fetchListStateFromSeeds(rpc, seeds, config = {}) {
  const maybeAccount = await fetchMaybeListStateFromSeeds(rpc, seeds, config);
  assertAccountExists(maybeAccount);
  return maybeAccount;
}
async function fetchMaybeListStateFromSeeds(rpc, seeds, config = {}) {
  const { programAddress, ...fetchConfig } = config;
  const [address3] = await findListStatePda(seeds, { programAddress });
  return await fetchMaybeListState(rpc, address3, fetchConfig);
}

// src/generated/errors/tensorMarketplace.ts
var TENSOR_MARKETPLACE_ERROR__ARITHMETIC_ERROR = 6100;
var TENSOR_MARKETPLACE_ERROR__EXPIRY_TOO_LARGE = 6101;
var TENSOR_MARKETPLACE_ERROR__BAD_OWNER = 6102;
var TENSOR_MARKETPLACE_ERROR__BAD_LIST_STATE = 6103;
var TENSOR_MARKETPLACE_ERROR__BAD_ROYALTIES_PCT = 6104;
var TENSOR_MARKETPLACE_ERROR__PRICE_MISMATCH = 6105;
var TENSOR_MARKETPLACE_ERROR__CREATOR_MISMATCH = 6106;
var TENSOR_MARKETPLACE_ERROR__INSUFFICIENT_BALANCE = 6107;
var TENSOR_MARKETPLACE_ERROR__BID_EXPIRED = 6108;
var TENSOR_MARKETPLACE_ERROR__TAKER_NOT_ALLOWED = 6109;
var TENSOR_MARKETPLACE_ERROR__BAD_BID_FIELD = 6110;
var TENSOR_MARKETPLACE_ERROR__BID_NOT_YET_EXPIRED = 6111;
var TENSOR_MARKETPLACE_ERROR__BAD_MARGIN = 6112;
var TENSOR_MARKETPLACE_ERROR__WRONG_IX_FOR_BID_TARGET = 6113;
var TENSOR_MARKETPLACE_ERROR__WRONG_TARGET_ID = 6114;
var TENSOR_MARKETPLACE_ERROR__MISSING_FVC = 6115;
var TENSOR_MARKETPLACE_ERROR__MISSING_COLLECTION = 6116;
var TENSOR_MARKETPLACE_ERROR__CANNOT_MODIFY_TARGET = 6117;
var TENSOR_MARKETPLACE_ERROR__TARGET_ID_MUST_EQUAL_BID_ID = 6118;
var TENSOR_MARKETPLACE_ERROR__CURRENCY_NOT_YET_ENABLED = 6119;
var TENSOR_MARKETPLACE_ERROR__MAKER_BROKER_NOT_YET_ENABLED = 6120;
var TENSOR_MARKETPLACE_ERROR__OPTIONAL_ROYALTIES_NOT_YET_ENABLED = 6121;
var TENSOR_MARKETPLACE_ERROR__WRONG_STATE_VERSION = 6122;
var TENSOR_MARKETPLACE_ERROR__WRONG_BID_FIELD_ID = 6123;
var TENSOR_MARKETPLACE_ERROR__BROKER_MISMATCH = 6124;
var TENSOR_MARKETPLACE_ERROR__ASSET_ID_MISMATCH = 6125;
var TENSOR_MARKETPLACE_ERROR__LISTING_EXPIRED = 6126;
var TENSOR_MARKETPLACE_ERROR__LISTING_NOT_YET_EXPIRED = 6127;
var TENSOR_MARKETPLACE_ERROR__BAD_QUANTITY = 6128;
var TENSOR_MARKETPLACE_ERROR__BID_FULLY_FILLED = 6129;
var TENSOR_MARKETPLACE_ERROR__BAD_WHITELIST = 6130;
var TENSOR_MARKETPLACE_ERROR__FORBIDDEN_COLLECTION = 6131;
var TENSOR_MARKETPLACE_ERROR__BAD_COSIGNER = 6132;
var TENSOR_MARKETPLACE_ERROR__BAD_MINT_PROOF = 6133;
var TENSOR_MARKETPLACE_ERROR__CURRENCY_MISMATCH = 6134;
var TENSOR_MARKETPLACE_ERROR__BID_BALANCE_NOT_EMPTIED = 6135;
var TENSOR_MARKETPLACE_ERROR__BAD_RENT_DEST = 6136;
var TENSOR_MARKETPLACE_ERROR__CURRENCY_NOT_YET_WHITELISTED = 6137;
var TENSOR_MARKETPLACE_ERROR__MAKER_BROKER_NOT_YET_WHITELISTED = 6138;
var TENSOR_MARKETPLACE_ERROR__WRONG_TOKEN_RECORD_DERIVATION = 6139;
var TENSOR_MARKETPLACE_ERROR__INVALID_FEE_ACCOUNT = 6140;
var TENSOR_MARKETPLACE_ERROR__INSUFFICIENT_REMAINING_ACCOUNTS = 6141;
var TENSOR_MARKETPLACE_ERROR__MISSING_BROKER = 6142;
var TENSOR_MARKETPLACE_ERROR__MISSING_BROKER_TOKEN_ACCOUNT = 6143;
var TENSOR_MARKETPLACE_ERROR__INVALID_TOKEN_ACCOUNT = 6144;
var tensorMarketplaceErrorMessages;
if (process.env.NODE_ENV !== "production") {
  tensorMarketplaceErrorMessages = {
    [TENSOR_MARKETPLACE_ERROR__ARITHMETIC_ERROR]: `arithmetic error`,
    [TENSOR_MARKETPLACE_ERROR__ASSET_ID_MISMATCH]: `asset id mismatch`,
    [TENSOR_MARKETPLACE_ERROR__BAD_BID_FIELD]: `cannot pass bid field`,
    [TENSOR_MARKETPLACE_ERROR__BAD_COSIGNER]: `bad cosigner`,
    [TENSOR_MARKETPLACE_ERROR__BAD_LIST_STATE]: `bad list state`,
    [TENSOR_MARKETPLACE_ERROR__BAD_MARGIN]: `bad margin`,
    [TENSOR_MARKETPLACE_ERROR__BAD_MINT_PROOF]: `bad mint proof`,
    [TENSOR_MARKETPLACE_ERROR__BAD_OWNER]: `bad owner`,
    [TENSOR_MARKETPLACE_ERROR__BAD_QUANTITY]: `bad quantity passed in`,
    [TENSOR_MARKETPLACE_ERROR__BAD_RENT_DEST]: `Bad rent dest.`,
    [TENSOR_MARKETPLACE_ERROR__BAD_ROYALTIES_PCT]: `royalties pct must be between 0 and 100`,
    [TENSOR_MARKETPLACE_ERROR__BAD_WHITELIST]: `bad whitelist`,
    [TENSOR_MARKETPLACE_ERROR__BID_BALANCE_NOT_EMPTIED]: `The bid balance was not emptied`,
    [TENSOR_MARKETPLACE_ERROR__BID_EXPIRED]: `bid has expired`,
    [TENSOR_MARKETPLACE_ERROR__BID_FULLY_FILLED]: `bid fully filled`,
    [TENSOR_MARKETPLACE_ERROR__BID_NOT_YET_EXPIRED]: `bid not yet expired`,
    [TENSOR_MARKETPLACE_ERROR__BROKER_MISMATCH]: `broker mismatch`,
    [TENSOR_MARKETPLACE_ERROR__CANNOT_MODIFY_TARGET]: `cannot modify bid target, create a new bid`,
    [TENSOR_MARKETPLACE_ERROR__CREATOR_MISMATCH]: `creator mismatch`,
    [TENSOR_MARKETPLACE_ERROR__CURRENCY_MISMATCH]: `Currency mismatch`,
    [TENSOR_MARKETPLACE_ERROR__CURRENCY_NOT_YET_ENABLED]: `currency not yet enabled`,
    [TENSOR_MARKETPLACE_ERROR__CURRENCY_NOT_YET_WHITELISTED]: `currency not yet whitelisted`,
    [TENSOR_MARKETPLACE_ERROR__EXPIRY_TOO_LARGE]: `expiry too large`,
    [TENSOR_MARKETPLACE_ERROR__FORBIDDEN_COLLECTION]: `forbidden collection`,
    [TENSOR_MARKETPLACE_ERROR__INSUFFICIENT_BALANCE]: `insufficient balance`,
    [TENSOR_MARKETPLACE_ERROR__INSUFFICIENT_REMAINING_ACCOUNTS]: `insufficient remaining accounts`,
    [TENSOR_MARKETPLACE_ERROR__INVALID_FEE_ACCOUNT]: `invalid fee account`,
    [TENSOR_MARKETPLACE_ERROR__INVALID_TOKEN_ACCOUNT]: `invalidtoken account`,
    [TENSOR_MARKETPLACE_ERROR__LISTING_EXPIRED]: `listing has expired`,
    [TENSOR_MARKETPLACE_ERROR__LISTING_NOT_YET_EXPIRED]: `listing not yet expired`,
    [TENSOR_MARKETPLACE_ERROR__MAKER_BROKER_NOT_YET_ENABLED]: `maker broker not yet enabled`,
    [TENSOR_MARKETPLACE_ERROR__MAKER_BROKER_NOT_YET_WHITELISTED]: `maker broker not yet whitelisted`,
    [TENSOR_MARKETPLACE_ERROR__MISSING_BROKER]: `missing broker account`,
    [TENSOR_MARKETPLACE_ERROR__MISSING_BROKER_TOKEN_ACCOUNT]: `missing broker token account`,
    [TENSOR_MARKETPLACE_ERROR__MISSING_COLLECTION]: `metadata missing collection`,
    [TENSOR_MARKETPLACE_ERROR__MISSING_FVC]: `creator array missing first verified creator`,
    [TENSOR_MARKETPLACE_ERROR__OPTIONAL_ROYALTIES_NOT_YET_ENABLED]: `optional royalties not yet enabled`,
    [TENSOR_MARKETPLACE_ERROR__PRICE_MISMATCH]: `price mismatch`,
    [TENSOR_MARKETPLACE_ERROR__TAKER_NOT_ALLOWED]: `taker not allowed`,
    [TENSOR_MARKETPLACE_ERROR__TARGET_ID_MUST_EQUAL_BID_ID]: `target id and bid id must be the same for single bids`,
    [TENSOR_MARKETPLACE_ERROR__WRONG_BID_FIELD_ID]: `wrong field id`,
    [TENSOR_MARKETPLACE_ERROR__WRONG_IX_FOR_BID_TARGET]: `wrong ix for bid target called`,
    [TENSOR_MARKETPLACE_ERROR__WRONG_STATE_VERSION]: `wrong state version`,
    [TENSOR_MARKETPLACE_ERROR__WRONG_TARGET_ID]: `wrong target id`,
    [TENSOR_MARKETPLACE_ERROR__WRONG_TOKEN_RECORD_DERIVATION]: `token record derivation is wrong`
  };
}
function getTensorMarketplaceErrorMessage(code) {
  if (process.env.NODE_ENV !== "production") {
    return tensorMarketplaceErrorMessages[code];
  }
  return "Error message not available in production bundles.";
}
var TENSOR_MARKETPLACE_PROGRAM_ADDRESS = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
var TensorMarketplaceAccount = /* @__PURE__ */ ((TensorMarketplaceAccount2) => {
  TensorMarketplaceAccount2[TensorMarketplaceAccount2["ListState"] = 0] = "ListState";
  TensorMarketplaceAccount2[TensorMarketplaceAccount2["BidState"] = 1] = "BidState";
  TensorMarketplaceAccount2[TensorMarketplaceAccount2["BidTa"] = 2] = "BidTa";
  return TensorMarketplaceAccount2;
})(TensorMarketplaceAccount || {});
function identifyTensorMarketplaceAccount(account) {
  const data = account instanceof Uint8Array ? account : account.data;
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([78, 242, 89, 138, 161, 221, 176, 75])
    ),
    0
  )) {
    return 0 /* ListState */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([155, 197, 5, 97, 189, 60, 8, 183])
    ),
    0
  )) {
    return 1 /* BidState */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([186, 230, 110, 26, 235, 24, 211, 156])
    ),
    0
  )) {
    return 2 /* BidTa */;
  }
  throw new Error(
    "The provided account could not be identified as a tensorMarketplace account."
  );
}
var TensorMarketplaceInstruction = /* @__PURE__ */ ((TensorMarketplaceInstruction2) => {
  TensorMarketplaceInstruction2[TensorMarketplaceInstruction2["TcompNoop"] = 0] = "TcompNoop";
  TensorMarketplaceInstruction2[TensorMarketplaceInstruction2["Edit"] = 1] = "Edit";
  TensorMarketplaceInstruction2[TensorMarketplaceInstruction2["Bid"] = 2] = "Bid";
  TensorMarketplaceInstruction2[TensorMarketplaceInstruction2["CancelBid"] = 3] = "CancelBid";
  TensorMarketplaceInstruction2[TensorMarketplaceInstruction2["CloseExpiredBid"] = 4] = "CloseExpiredBid";
  TensorMarketplaceInstruction2[TensorMarketplaceInstruction2["BuyCompressed"] = 5] = "BuyCompressed";
  TensorMarketplaceInstruction2[TensorMarketplaceInstruction2["BuySpl"] = 6] = "BuySpl";
  TensorMarketplaceInstruction2[TensorMarketplaceInstruction2["CloseExpiredListingCompressed"] = 7] = "CloseExpiredListingCompressed";
  TensorMarketplaceInstruction2[TensorMarketplaceInstruction2["ListCompressed"] = 8] = "ListCompressed";
  TensorMarketplaceInstruction2[TensorMarketplaceInstruction2["DelistCompressed"] = 9] = "DelistCompressed";
  TensorMarketplaceInstruction2[TensorMarketplaceInstruction2["TakeBidCompressedMetaHash"] = 10] = "TakeBidCompressedMetaHash";
  TensorMarketplaceInstruction2[TensorMarketplaceInstruction2["TakeBidCompressedFullMeta"] = 11] = "TakeBidCompressedFullMeta";
  TensorMarketplaceInstruction2[TensorMarketplaceInstruction2["BuyLegacy"] = 12] = "BuyLegacy";
  TensorMarketplaceInstruction2[TensorMarketplaceInstruction2["BuyLegacySpl"] = 13] = "BuyLegacySpl";
  TensorMarketplaceInstruction2[TensorMarketplaceInstruction2["CloseExpiredListingLegacy"] = 14] = "CloseExpiredListingLegacy";
  TensorMarketplaceInstruction2[TensorMarketplaceInstruction2["DelistLegacy"] = 15] = "DelistLegacy";
  TensorMarketplaceInstruction2[TensorMarketplaceInstruction2["ListLegacy"] = 16] = "ListLegacy";
  TensorMarketplaceInstruction2[TensorMarketplaceInstruction2["TakeBidLegacy"] = 17] = "TakeBidLegacy";
  TensorMarketplaceInstruction2[TensorMarketplaceInstruction2["BuyT22"] = 18] = "BuyT22";
  TensorMarketplaceInstruction2[TensorMarketplaceInstruction2["BuyT22Spl"] = 19] = "BuyT22Spl";
  TensorMarketplaceInstruction2[TensorMarketplaceInstruction2["CloseExpiredListingT22"] = 20] = "CloseExpiredListingT22";
  TensorMarketplaceInstruction2[TensorMarketplaceInstruction2["DelistT22"] = 21] = "DelistT22";
  TensorMarketplaceInstruction2[TensorMarketplaceInstruction2["ListT22"] = 22] = "ListT22";
  TensorMarketplaceInstruction2[TensorMarketplaceInstruction2["TakeBidT22"] = 23] = "TakeBidT22";
  TensorMarketplaceInstruction2[TensorMarketplaceInstruction2["BuyWns"] = 24] = "BuyWns";
  TensorMarketplaceInstruction2[TensorMarketplaceInstruction2["BuyWnsSpl"] = 25] = "BuyWnsSpl";
  TensorMarketplaceInstruction2[TensorMarketplaceInstruction2["CloseExpiredListingWns"] = 26] = "CloseExpiredListingWns";
  TensorMarketplaceInstruction2[TensorMarketplaceInstruction2["DelistWns"] = 27] = "DelistWns";
  TensorMarketplaceInstruction2[TensorMarketplaceInstruction2["ListWns"] = 28] = "ListWns";
  TensorMarketplaceInstruction2[TensorMarketplaceInstruction2["TakeBidWns"] = 29] = "TakeBidWns";
  TensorMarketplaceInstruction2[TensorMarketplaceInstruction2["BuyCore"] = 30] = "BuyCore";
  TensorMarketplaceInstruction2[TensorMarketplaceInstruction2["BuyCoreSpl"] = 31] = "BuyCoreSpl";
  TensorMarketplaceInstruction2[TensorMarketplaceInstruction2["CloseExpiredListingCore"] = 32] = "CloseExpiredListingCore";
  TensorMarketplaceInstruction2[TensorMarketplaceInstruction2["DelistCore"] = 33] = "DelistCore";
  TensorMarketplaceInstruction2[TensorMarketplaceInstruction2["ListCore"] = 34] = "ListCore";
  TensorMarketplaceInstruction2[TensorMarketplaceInstruction2["TakeBidCore"] = 35] = "TakeBidCore";
  return TensorMarketplaceInstruction2;
})(TensorMarketplaceInstruction || {});
function identifyTensorMarketplaceInstruction(instruction) {
  const data = instruction instanceof Uint8Array ? instruction : instruction.data;
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([106, 162, 10, 226, 132, 68, 223, 21])
    ),
    0
  )) {
    return 0 /* TcompNoop */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([15, 183, 33, 86, 87, 28, 151, 145])
    ),
    0
  )) {
    return 1 /* Edit */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([199, 56, 85, 38, 146, 243, 37, 158])
    ),
    0
  )) {
    return 2 /* Bid */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([40, 243, 190, 217, 208, 253, 86, 206])
    ),
    0
  )) {
    return 3 /* CancelBid */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([83, 20, 105, 67, 248, 68, 104, 190])
    ),
    0
  )) {
    return 4 /* CloseExpiredBid */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([102, 6, 61, 18, 1, 218, 235, 234])
    ),
    0
  )) {
    return 5 /* BuyCompressed */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([65, 136, 254, 255, 59, 130, 234, 174])
    ),
    0
  )) {
    return 6 /* BuySpl */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([150, 70, 13, 135, 9, 204, 75, 4])
    ),
    0
  )) {
    return 7 /* CloseExpiredListingCompressed */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([54, 174, 193, 67, 17, 41, 132, 38])
    ),
    0
  )) {
    return 8 /* ListCompressed */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([55, 136, 205, 107, 107, 173, 4, 31])
    ),
    0
  )) {
    return 9 /* DelistCompressed */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([85, 227, 202, 70, 45, 215, 10, 193])
    ),
    0
  )) {
    return 10 /* TakeBidCompressedMetaHash */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([242, 194, 203, 225, 234, 53, 10, 96])
    ),
    0
  )) {
    return 11 /* TakeBidCompressedFullMeta */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([68, 127, 43, 8, 212, 31, 249, 114])
    ),
    0
  )) {
    return 12 /* BuyLegacy */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([134, 94, 125, 229, 24, 157, 194, 199])
    ),
    0
  )) {
    return 13 /* BuyLegacySpl */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([56, 16, 96, 188, 55, 68, 250, 58])
    ),
    0
  )) {
    return 14 /* CloseExpiredListingLegacy */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([88, 35, 231, 184, 110, 218, 149, 23])
    ),
    0
  )) {
    return 15 /* DelistLegacy */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([6, 110, 255, 18, 16, 36, 8, 30])
    ),
    0
  )) {
    return 16 /* ListLegacy */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([188, 35, 116, 108, 0, 233, 237, 201])
    ),
    0
  )) {
    return 17 /* TakeBidLegacy */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([81, 98, 227, 171, 201, 105, 180, 216])
    ),
    0
  )) {
    return 18 /* BuyT22 */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([102, 21, 163, 39, 94, 39, 122, 94])
    ),
    0
  )) {
    return 19 /* BuyT22Spl */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([69, 2, 190, 122, 144, 119, 122, 220])
    ),
    0
  )) {
    return 20 /* CloseExpiredListingT22 */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([216, 72, 73, 18, 204, 82, 123, 26])
    ),
    0
  )) {
    return 21 /* DelistT22 */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([9, 117, 93, 230, 221, 4, 199, 212])
    ),
    0
  )) {
    return 22 /* ListT22 */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([18, 250, 113, 242, 31, 244, 19, 150])
    ),
    0
  )) {
    return 23 /* TakeBidT22 */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([168, 43, 179, 217, 44, 59, 35, 244])
    ),
    0
  )) {
    return 24 /* BuyWns */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([113, 137, 57, 23, 186, 196, 217, 210])
    ),
    0
  )) {
    return 25 /* BuyWnsSpl */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([222, 31, 183, 134, 230, 207, 7, 132])
    ),
    0
  )) {
    return 26 /* CloseExpiredListingWns */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([172, 171, 57, 16, 74, 158, 32, 57])
    ),
    0
  )) {
    return 27 /* DelistWns */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([23, 202, 102, 138, 255, 190, 39, 196])
    ),
    0
  )) {
    return 28 /* ListWns */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([88, 5, 122, 88, 250, 139, 35, 216])
    ),
    0
  )) {
    return 29 /* TakeBidWns */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([169, 227, 87, 255, 76, 86, 255, 25])
    ),
    0
  )) {
    return 30 /* BuyCore */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([234, 28, 37, 122, 114, 239, 233, 208])
    ),
    0
  )) {
    return 31 /* BuyCoreSpl */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([89, 171, 78, 80, 74, 188, 63, 58])
    ),
    0
  )) {
    return 32 /* CloseExpiredListingCore */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([56, 24, 231, 2, 227, 19, 14, 68])
    ),
    0
  )) {
    return 33 /* DelistCore */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([173, 76, 167, 125, 118, 71, 1, 153])
    ),
    0
  )) {
    return 34 /* ListCore */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([250, 41, 248, 20, 61, 161, 27, 141])
    ),
    0
  )) {
    return 35 /* TakeBidCore */;
  }
  throw new Error(
    "The provided instruction could not be identified as a tensorMarketplace instruction."
  );
}

// src/generated/instructions/bid.ts
function getBidInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["bidId", getAddressEncoder()],
      ["target", getTargetEncoder()],
      ["targetId", getAddressEncoder()],
      ["field", getOptionEncoder(getFieldEncoder())],
      ["fieldId", getOptionEncoder(getAddressEncoder())],
      ["amount", getU64Encoder()],
      ["quantity", getU32Encoder()],
      ["expireInSec", getOptionEncoder(getU64Encoder())],
      ["currency", getOptionEncoder(getAddressEncoder())],
      ["privateTaker", getOptionEncoder(getAddressEncoder())],
      ["makerBroker", getOptionEncoder(getAddressEncoder())]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([199, 56, 85, 38, 146, 243, 37, 158]),
      field: value.field ?? none(),
      fieldId: value.fieldId ?? none(),
      quantity: value.quantity ?? 1,
      expireInSec: value.expireInSec ?? none(),
      currency: value.currency ?? none(),
      privateTaker: value.privateTaker ?? none(),
      makerBroker: value.makerBroker ?? none()
    })
  );
}
function getBidInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["bidId", getAddressDecoder()],
    ["target", getTargetDecoder()],
    ["targetId", getAddressDecoder()],
    ["field", getOptionDecoder(getFieldDecoder())],
    ["fieldId", getOptionDecoder(getAddressDecoder())],
    ["amount", getU64Decoder()],
    ["quantity", getU32Decoder()],
    ["expireInSec", getOptionDecoder(getU64Decoder())],
    ["currency", getOptionDecoder(getAddressDecoder())],
    ["privateTaker", getOptionDecoder(getAddressDecoder())],
    ["makerBroker", getOptionDecoder(getAddressDecoder())]
  ]);
}
function getBidInstructionDataCodec() {
  return combineCodec(
    getBidInstructionDataEncoder(),
    getBidInstructionDataDecoder()
  );
}
async function getBidInstructionAsync(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    bidState: { value: input.bidState ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: true },
    sharedEscrow: { value: input.sharedEscrow ?? null, isWritable: true },
    cosigner: { value: input.cosigner ?? null, isWritable: false },
    rentPayer: { value: input.rentPayer ?? null, isWritable: true }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!args.bidId) {
    args.bidId = await resolveBidIdOnCreate(resolverScope);
  }
  if (!accounts.bidState.value) {
    accounts.bidState.value = await findBidStatePda({
      bidId: expectSome(args.bidId),
      owner: expectAddress(accounts.owner.value)
    });
  }
  if (!accounts.sharedEscrow.value) {
    accounts.sharedEscrow.value = expectTransactionSigner(
      accounts.owner.value
    ).address;
  }
  if (!accounts.rentPayer.value) {
    accounts.rentPayer.value = expectSome(accounts.owner.value);
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.bidState),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.sharedEscrow),
      getAccountMeta(accounts.cosigner),
      getAccountMeta(accounts.rentPayer)
    ],
    programAddress,
    data: getBidInstructionDataEncoder().encode(args)
  };
  return instruction;
}
function getBidInstruction(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    bidState: { value: input.bidState ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: true },
    sharedEscrow: { value: input.sharedEscrow ?? null, isWritable: true },
    cosigner: { value: input.cosigner ?? null, isWritable: false },
    rentPayer: { value: input.rentPayer ?? null, isWritable: true }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.sharedEscrow.value) {
    accounts.sharedEscrow.value = expectTransactionSigner(
      accounts.owner.value
    ).address;
  }
  if (!accounts.rentPayer.value) {
    accounts.rentPayer.value = expectSome(accounts.owner.value);
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.bidState),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.sharedEscrow),
      getAccountMeta(accounts.cosigner),
      getAccountMeta(accounts.rentPayer)
    ],
    programAddress,
    data: getBidInstructionDataEncoder().encode(args)
  };
  return instruction;
}
function parseBidInstruction(instruction) {
  if (instruction.accounts.length < 7) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  const getNextOptionalAccount = () => {
    const accountMeta = getNextAccount();
    return accountMeta.address === TENSOR_MARKETPLACE_PROGRAM_ADDRESS ? void 0 : accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      systemProgram: getNextAccount(),
      marketplaceProgram: getNextAccount(),
      bidState: getNextAccount(),
      owner: getNextAccount(),
      sharedEscrow: getNextAccount(),
      cosigner: getNextOptionalAccount(),
      rentPayer: getNextAccount()
    },
    data: getBidInstructionDataDecoder().decode(instruction.data)
  };
}
function getBuyCompressedInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["nonce", getU64Encoder()],
      ["index", getU32Encoder()],
      ["root", fixEncoderSize(getBytesEncoder(), 32)],
      ["metaHash", fixEncoderSize(getBytesEncoder(), 32)],
      [
        "creatorShares",
        addEncoderSizePrefix(getBytesEncoder(), getU32Encoder())
      ],
      ["creatorVerified", getArrayEncoder(getBooleanEncoder())],
      ["sellerFeeBasisPoints", getU16Encoder()],
      ["maxAmount", getU64Encoder()],
      ["optionalRoyaltyPct", getOptionEncoder(getU16Encoder())]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([102, 6, 61, 18, 1, 218, 235, 234]),
      optionalRoyaltyPct: value.optionalRoyaltyPct ?? 100
    })
  );
}
function getBuyCompressedInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["nonce", getU64Decoder()],
    ["index", getU32Decoder()],
    ["root", fixDecoderSize(getBytesDecoder(), 32)],
    ["metaHash", fixDecoderSize(getBytesDecoder(), 32)],
    ["creatorShares", addDecoderSizePrefix(getBytesDecoder(), getU32Decoder())],
    ["creatorVerified", getArrayDecoder(getBooleanDecoder())],
    ["sellerFeeBasisPoints", getU16Decoder()],
    ["maxAmount", getU64Decoder()],
    ["optionalRoyaltyPct", getOptionDecoder(getU16Decoder())]
  ]);
}
function getBuyCompressedInstructionDataCodec() {
  return combineCodec(
    getBuyCompressedInstructionDataEncoder(),
    getBuyCompressedInstructionDataDecoder()
  );
}
async function getBuyCompressedInstructionAsync(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    feeVault: { value: input.feeVault ?? null, isWritable: true },
    treeAuthority: { value: input.treeAuthority ?? null, isWritable: false },
    merkleTree: { value: input.merkleTree ?? null, isWritable: true },
    logWrapper: { value: input.logWrapper ?? null, isWritable: false },
    compressionProgram: {
      value: input.compressionProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    bubblegumProgram: {
      value: input.bubblegumProgram ?? null,
      isWritable: false
    },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    listState: { value: input.listState ?? null, isWritable: true },
    buyer: { value: input.buyer ?? null, isWritable: false },
    payer: { value: input.payer ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: true },
    takerBroker: { value: input.takerBroker ?? null, isWritable: true },
    makerBroker: { value: input.makerBroker ?? null, isWritable: true },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true },
    cosigner: { value: input.cosigner ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.feeVault.value) {
    accounts.feeVault = {
      ...accounts.feeVault,
      ...await resolveFeeVaultPdaFromListState(resolverScope)
    };
  }
  if (!accounts.bubblegumProgram.value) {
    accounts.bubblegumProgram.value = "BGUMAp9Gq7iTEuizy4pqaxsTyUCBK68MDfK752saRPUY";
  }
  if (!accounts.treeAuthority.value) {
    accounts.treeAuthority = {
      ...accounts.treeAuthority,
      ...await resolveTreeAuthorityPda(resolverScope)
    };
  }
  if (!accounts.logWrapper.value) {
    accounts.logWrapper.value = "noopb9bkMVfRPU8AsbpTUg8AQkHtKwMYZiFUjNRtMmV";
  }
  if (!accounts.compressionProgram.value) {
    accounts.compressionProgram.value = "cmtDvXumGCrqC1Age74AVPhSRVXJMd8PJS91L8KbNCK";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.buyer.value) {
    accounts.buyer.value = expectTransactionSigner(
      accounts.payer.value
    ).address;
  }
  if (!accounts.rentDestination.value) {
    accounts.rentDestination.value = expectSome(accounts.owner.value);
  }
  if (!accounts.cosigner.value) {
    accounts.cosigner.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!args.nonce) {
    args.nonce = expectSome(args.index);
  }
  if (!args.creators) {
    args.creators = [];
  }
  if (!args.proof) {
    args.proof = [];
  }
  if (!args.canopyDepth) {
    args.canopyDepth = 0;
  }
  const remainingAccounts = [
    ...resolveCreatorPath(resolverScope),
    ...resolveProofPath(resolverScope)
  ];
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.feeVault),
      getAccountMeta(accounts.treeAuthority),
      getAccountMeta(accounts.merkleTree),
      getAccountMeta(accounts.logWrapper),
      getAccountMeta(accounts.compressionProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.bubblegumProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.buyer),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.takerBroker),
      getAccountMeta(accounts.makerBroker),
      getAccountMeta(accounts.rentDestination),
      getAccountMeta(accounts.cosigner),
      ...remainingAccounts
    ],
    programAddress,
    data: getBuyCompressedInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function getBuyCompressedInstruction(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    feeVault: { value: input.feeVault ?? null, isWritable: true },
    treeAuthority: { value: input.treeAuthority ?? null, isWritable: false },
    merkleTree: { value: input.merkleTree ?? null, isWritable: true },
    logWrapper: { value: input.logWrapper ?? null, isWritable: false },
    compressionProgram: {
      value: input.compressionProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    bubblegumProgram: {
      value: input.bubblegumProgram ?? null,
      isWritable: false
    },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    listState: { value: input.listState ?? null, isWritable: true },
    buyer: { value: input.buyer ?? null, isWritable: false },
    payer: { value: input.payer ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: true },
    takerBroker: { value: input.takerBroker ?? null, isWritable: true },
    makerBroker: { value: input.makerBroker ?? null, isWritable: true },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true },
    cosigner: { value: input.cosigner ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.bubblegumProgram.value) {
    accounts.bubblegumProgram.value = "BGUMAp9Gq7iTEuizy4pqaxsTyUCBK68MDfK752saRPUY";
  }
  if (!accounts.logWrapper.value) {
    accounts.logWrapper.value = "noopb9bkMVfRPU8AsbpTUg8AQkHtKwMYZiFUjNRtMmV";
  }
  if (!accounts.compressionProgram.value) {
    accounts.compressionProgram.value = "cmtDvXumGCrqC1Age74AVPhSRVXJMd8PJS91L8KbNCK";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.buyer.value) {
    accounts.buyer.value = expectTransactionSigner(
      accounts.payer.value
    ).address;
  }
  if (!accounts.rentDestination.value) {
    accounts.rentDestination.value = expectSome(accounts.owner.value);
  }
  if (!accounts.cosigner.value) {
    accounts.cosigner.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!args.nonce) {
    args.nonce = expectSome(args.index);
  }
  if (!args.creators) {
    args.creators = [];
  }
  if (!args.proof) {
    args.proof = [];
  }
  if (!args.canopyDepth) {
    args.canopyDepth = 0;
  }
  const remainingAccounts = [
    ...resolveCreatorPath(resolverScope),
    ...resolveProofPath(resolverScope)
  ];
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.feeVault),
      getAccountMeta(accounts.treeAuthority),
      getAccountMeta(accounts.merkleTree),
      getAccountMeta(accounts.logWrapper),
      getAccountMeta(accounts.compressionProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.bubblegumProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.buyer),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.takerBroker),
      getAccountMeta(accounts.makerBroker),
      getAccountMeta(accounts.rentDestination),
      getAccountMeta(accounts.cosigner),
      ...remainingAccounts
    ],
    programAddress,
    data: getBuyCompressedInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseBuyCompressedInstruction(instruction) {
  if (instruction.accounts.length < 16) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  const getNextOptionalAccount = () => {
    const accountMeta = getNextAccount();
    return accountMeta.address === TENSOR_MARKETPLACE_PROGRAM_ADDRESS ? void 0 : accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      feeVault: getNextAccount(),
      treeAuthority: getNextAccount(),
      merkleTree: getNextAccount(),
      logWrapper: getNextAccount(),
      compressionProgram: getNextAccount(),
      systemProgram: getNextAccount(),
      bubblegumProgram: getNextAccount(),
      marketplaceProgram: getNextAccount(),
      listState: getNextAccount(),
      buyer: getNextAccount(),
      payer: getNextAccount(),
      owner: getNextAccount(),
      takerBroker: getNextOptionalAccount(),
      makerBroker: getNextOptionalAccount(),
      rentDestination: getNextAccount(),
      cosigner: getNextOptionalAccount()
    },
    data: getBuyCompressedInstructionDataDecoder().decode(instruction.data)
  };
}
function getBuyCoreInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["maxAmount", getU64Encoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([169, 227, 87, 255, 76, 86, 255, 25])
    })
  );
}
function getBuyCoreInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["maxAmount", getU64Decoder()]
  ]);
}
function getBuyCoreInstructionDataCodec() {
  return combineCodec(
    getBuyCoreInstructionDataEncoder(),
    getBuyCoreInstructionDataDecoder()
  );
}
function getBuyCoreInstruction(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    feeVault: { value: input.feeVault ?? null, isWritable: true },
    listState: { value: input.listState ?? null, isWritable: true },
    asset: { value: input.asset ?? null, isWritable: true },
    collection: { value: input.collection ?? null, isWritable: false },
    buyer: { value: input.buyer ?? null, isWritable: false },
    payer: { value: input.payer ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: true },
    takerBroker: { value: input.takerBroker ?? null, isWritable: true },
    makerBroker: { value: input.makerBroker ?? null, isWritable: true },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true },
    mplCoreProgram: { value: input.mplCoreProgram ?? null, isWritable: false },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    cosigner: { value: input.cosigner ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.buyer.value) {
    accounts.buyer.value = expectTransactionSigner(
      accounts.payer.value
    ).address;
  }
  if (!accounts.rentDestination.value) {
    accounts.rentDestination.value = expectSome(accounts.owner.value);
  }
  if (!accounts.mplCoreProgram.value) {
    accounts.mplCoreProgram.value = "CoREENxT6tW1HoK8ypY1SxRMZTcVPm7R94rH4PZNhX7d";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.cosigner.value) {
    accounts.cosigner.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  const remainingAccounts = (args.creators ?? []).map(
    (address3) => ({ address: address3, role: AccountRole.WRITABLE })
  );
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.feeVault),
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.asset),
      getAccountMeta(accounts.collection),
      getAccountMeta(accounts.buyer),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.takerBroker),
      getAccountMeta(accounts.makerBroker),
      getAccountMeta(accounts.rentDestination),
      getAccountMeta(accounts.mplCoreProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.cosigner),
      ...remainingAccounts
    ],
    programAddress,
    data: getBuyCoreInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseBuyCoreInstruction(instruction) {
  if (instruction.accounts.length < 14) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  const getNextOptionalAccount = () => {
    const accountMeta = getNextAccount();
    return accountMeta.address === TENSOR_MARKETPLACE_PROGRAM_ADDRESS ? void 0 : accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      feeVault: getNextAccount(),
      listState: getNextAccount(),
      asset: getNextAccount(),
      collection: getNextOptionalAccount(),
      buyer: getNextAccount(),
      payer: getNextAccount(),
      owner: getNextAccount(),
      takerBroker: getNextOptionalAccount(),
      makerBroker: getNextOptionalAccount(),
      rentDestination: getNextAccount(),
      mplCoreProgram: getNextAccount(),
      marketplaceProgram: getNextAccount(),
      systemProgram: getNextAccount(),
      cosigner: getNextOptionalAccount()
    },
    data: getBuyCoreInstructionDataDecoder().decode(instruction.data)
  };
}
function getBuyCoreSplInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["maxAmount", getU64Encoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([234, 28, 37, 122, 114, 239, 233, 208])
    })
  );
}
function getBuyCoreSplInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["maxAmount", getU64Decoder()]
  ]);
}
function getBuyCoreSplInstructionDataCodec() {
  return combineCodec(
    getBuyCoreSplInstructionDataEncoder(),
    getBuyCoreSplInstructionDataDecoder()
  );
}
function getBuyCoreSplInstruction(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    feeVault: { value: input.feeVault ?? null, isWritable: true },
    feeVaultCurrencyTa: {
      value: input.feeVaultCurrencyTa ?? null,
      isWritable: true
    },
    buyer: { value: input.buyer ?? null, isWritable: false },
    listState: { value: input.listState ?? null, isWritable: true },
    asset: { value: input.asset ?? null, isWritable: true },
    collection: { value: input.collection ?? null, isWritable: false },
    currency: { value: input.currency ?? null, isWritable: false },
    owner: { value: input.owner ?? null, isWritable: true },
    ownerCurrencyTa: { value: input.ownerCurrencyTa ?? null, isWritable: true },
    payer: { value: input.payer ?? null, isWritable: true },
    payerCurrencyTa: { value: input.payerCurrencyTa ?? null, isWritable: true },
    takerBroker: { value: input.takerBroker ?? null, isWritable: true },
    takerBrokerTa: { value: input.takerBrokerTa ?? null, isWritable: true },
    makerBroker: { value: input.makerBroker ?? null, isWritable: true },
    makerBrokerTa: { value: input.makerBrokerTa ?? null, isWritable: true },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    mplCoreProgram: { value: input.mplCoreProgram ?? null, isWritable: false },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    cosigner: { value: input.cosigner ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.buyer.value) {
    accounts.buyer.value = expectTransactionSigner(
      accounts.payer.value
    ).address;
  }
  if (!accounts.rentDestination.value) {
    accounts.rentDestination.value = expectSome(accounts.owner.value);
  }
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.mplCoreProgram.value) {
    accounts.mplCoreProgram.value = "CoREENxT6tW1HoK8ypY1SxRMZTcVPm7R94rH4PZNhX7d";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const remainingAccounts = [
    ...(args.creators ?? []).map((address3) => ({
      address: address3,
      role: AccountRole.WRITABLE
    })),
    ...(args.creatorsAtas ?? []).map((address3) => ({
      address: address3,
      role: AccountRole.WRITABLE
    }))
  ];
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.feeVault),
      getAccountMeta(accounts.feeVaultCurrencyTa),
      getAccountMeta(accounts.buyer),
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.asset),
      getAccountMeta(accounts.collection),
      getAccountMeta(accounts.currency),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.ownerCurrencyTa),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.payerCurrencyTa),
      getAccountMeta(accounts.takerBroker),
      getAccountMeta(accounts.takerBrokerTa),
      getAccountMeta(accounts.makerBroker),
      getAccountMeta(accounts.makerBrokerTa),
      getAccountMeta(accounts.rentDestination),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.mplCoreProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.cosigner),
      ...remainingAccounts
    ],
    programAddress,
    data: getBuyCoreSplInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseBuyCoreSplInstruction(instruction) {
  if (instruction.accounts.length < 22) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  const getNextOptionalAccount = () => {
    const accountMeta = getNextAccount();
    return accountMeta.address === TENSOR_MARKETPLACE_PROGRAM_ADDRESS ? void 0 : accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      feeVault: getNextAccount(),
      feeVaultCurrencyTa: getNextAccount(),
      buyer: getNextAccount(),
      listState: getNextAccount(),
      asset: getNextAccount(),
      collection: getNextOptionalAccount(),
      currency: getNextAccount(),
      owner: getNextAccount(),
      ownerCurrencyTa: getNextAccount(),
      payer: getNextAccount(),
      payerCurrencyTa: getNextAccount(),
      takerBroker: getNextOptionalAccount(),
      takerBrokerTa: getNextOptionalAccount(),
      makerBroker: getNextOptionalAccount(),
      makerBrokerTa: getNextOptionalAccount(),
      rentDestination: getNextAccount(),
      tokenProgram: getNextAccount(),
      associatedTokenProgram: getNextAccount(),
      mplCoreProgram: getNextAccount(),
      marketplaceProgram: getNextAccount(),
      systemProgram: getNextAccount(),
      cosigner: getNextOptionalAccount()
    },
    data: getBuyCoreSplInstructionDataDecoder().decode(instruction.data)
  };
}
function getBuyLegacyInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["maxAmount", getU64Encoder()],
      ["optionalRoyaltyPct", getOptionEncoder(getU16Encoder())],
      [
        "authorizationData",
        getOptionEncoder(getAuthorizationDataLocalEncoder())
      ]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([68, 127, 43, 8, 212, 31, 249, 114]),
      optionalRoyaltyPct: value.optionalRoyaltyPct ?? none(),
      authorizationData: value.authorizationData ?? none()
    })
  );
}
function getBuyLegacyInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["maxAmount", getU64Decoder()],
    ["optionalRoyaltyPct", getOptionDecoder(getU16Decoder())],
    ["authorizationData", getOptionDecoder(getAuthorizationDataLocalDecoder())]
  ]);
}
function getBuyLegacyInstructionDataCodec() {
  return combineCodec(
    getBuyLegacyInstructionDataEncoder(),
    getBuyLegacyInstructionDataDecoder()
  );
}
async function getBuyLegacyInstructionAsync(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    feeVault: { value: input.feeVault ?? null, isWritable: true },
    buyer: { value: input.buyer ?? null, isWritable: false },
    buyerTa: { value: input.buyerTa ?? null, isWritable: true },
    listTa: { value: input.listTa ?? null, isWritable: true },
    listState: { value: input.listState ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    owner: { value: input.owner ?? null, isWritable: true },
    payer: { value: input.payer ?? null, isWritable: true },
    takerBroker: { value: input.takerBroker ?? null, isWritable: true },
    makerBroker: { value: input.makerBroker ?? null, isWritable: true },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    metadata: { value: input.metadata ?? null, isWritable: true },
    edition: { value: input.edition ?? null, isWritable: false },
    buyerTokenRecord: {
      value: input.buyerTokenRecord ?? null,
      isWritable: true
    },
    listTokenRecord: { value: input.listTokenRecord ?? null, isWritable: true },
    authorizationRules: {
      value: input.authorizationRules ?? null,
      isWritable: false
    },
    authorizationRulesProgram: {
      value: input.authorizationRulesProgram ?? null,
      isWritable: false
    },
    tokenMetadataProgram: {
      value: input.tokenMetadataProgram ?? null,
      isWritable: false
    },
    sysvarInstructions: {
      value: input.sysvarInstructions ?? null,
      isWritable: false
    },
    cosigner: { value: input.cosigner ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.listState.value) {
    accounts.listState.value = await findListStatePda({
      mint: expectAddress(accounts.mint.value)
    });
  }
  if (!accounts.feeVault.value) {
    accounts.feeVault = {
      ...accounts.feeVault,
      ...await resolveFeeVaultPdaFromListState(resolverScope)
    };
  }
  if (!accounts.buyer.value) {
    accounts.buyer.value = expectTransactionSigner(
      accounts.payer.value
    ).address;
  }
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.buyerTa.value) {
    accounts.buyerTa = {
      ...accounts.buyerTa,
      ...await resolveBuyerAta(resolverScope)
    };
  }
  if (!accounts.listTa.value) {
    accounts.listTa = {
      ...accounts.listTa,
      ...await resolveListAta(resolverScope)
    };
  }
  if (!accounts.rentDestination.value) {
    accounts.rentDestination.value = expectSome(accounts.owner.value);
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.metadata.value) {
    accounts.metadata = {
      ...accounts.metadata,
      ...await resolveMetadata(resolverScope)
    };
  }
  if (!args.tokenStandard) {
    args.tokenStandard = TokenStandard.ProgrammableNonFungible;
  }
  if (!accounts.edition.value) {
    accounts.edition = {
      ...accounts.edition,
      ...await resolveEditionFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.buyerTokenRecord.value) {
    accounts.buyerTokenRecord = {
      ...accounts.buyerTokenRecord,
      ...await resolveBuyerTokenRecordFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.listTokenRecord.value) {
    accounts.listTokenRecord = {
      ...accounts.listTokenRecord,
      ...await resolveListTokenRecordFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.authorizationRulesProgram.value) {
    accounts.authorizationRulesProgram = {
      ...accounts.authorizationRulesProgram,
      ...resolveAuthorizationRulesProgramFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.tokenMetadataProgram.value) {
    accounts.tokenMetadataProgram = {
      ...accounts.tokenMetadataProgram,
      ...resolveTokenMetadataProgramFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.sysvarInstructions.value) {
    accounts.sysvarInstructions = {
      ...accounts.sysvarInstructions,
      ...resolveSysvarInstructionsFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.cosigner.value) {
    accounts.cosigner.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  const remainingAccounts = (args.creators ?? []).map(
    (address3) => ({ address: address3, role: AccountRole.WRITABLE })
  );
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.feeVault),
      getAccountMeta(accounts.buyer),
      getAccountMeta(accounts.buyerTa),
      getAccountMeta(accounts.listTa),
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.takerBroker),
      getAccountMeta(accounts.makerBroker),
      getAccountMeta(accounts.rentDestination),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.metadata),
      getAccountMeta(accounts.edition),
      getAccountMeta(accounts.buyerTokenRecord),
      getAccountMeta(accounts.listTokenRecord),
      getAccountMeta(accounts.authorizationRules),
      getAccountMeta(accounts.authorizationRulesProgram),
      getAccountMeta(accounts.tokenMetadataProgram),
      getAccountMeta(accounts.sysvarInstructions),
      getAccountMeta(accounts.cosigner),
      ...remainingAccounts
    ],
    programAddress,
    data: getBuyLegacyInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function getBuyLegacyInstruction(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    feeVault: { value: input.feeVault ?? null, isWritable: true },
    buyer: { value: input.buyer ?? null, isWritable: false },
    buyerTa: { value: input.buyerTa ?? null, isWritable: true },
    listTa: { value: input.listTa ?? null, isWritable: true },
    listState: { value: input.listState ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    owner: { value: input.owner ?? null, isWritable: true },
    payer: { value: input.payer ?? null, isWritable: true },
    takerBroker: { value: input.takerBroker ?? null, isWritable: true },
    makerBroker: { value: input.makerBroker ?? null, isWritable: true },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    metadata: { value: input.metadata ?? null, isWritable: true },
    edition: { value: input.edition ?? null, isWritable: false },
    buyerTokenRecord: {
      value: input.buyerTokenRecord ?? null,
      isWritable: true
    },
    listTokenRecord: { value: input.listTokenRecord ?? null, isWritable: true },
    authorizationRules: {
      value: input.authorizationRules ?? null,
      isWritable: false
    },
    authorizationRulesProgram: {
      value: input.authorizationRulesProgram ?? null,
      isWritable: false
    },
    tokenMetadataProgram: {
      value: input.tokenMetadataProgram ?? null,
      isWritable: false
    },
    sysvarInstructions: {
      value: input.sysvarInstructions ?? null,
      isWritable: false
    },
    cosigner: { value: input.cosigner ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.buyer.value) {
    accounts.buyer.value = expectTransactionSigner(
      accounts.payer.value
    ).address;
  }
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.rentDestination.value) {
    accounts.rentDestination.value = expectSome(accounts.owner.value);
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!args.tokenStandard) {
    args.tokenStandard = TokenStandard.ProgrammableNonFungible;
  }
  if (!accounts.authorizationRulesProgram.value) {
    accounts.authorizationRulesProgram = {
      ...accounts.authorizationRulesProgram,
      ...resolveAuthorizationRulesProgramFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.tokenMetadataProgram.value) {
    accounts.tokenMetadataProgram = {
      ...accounts.tokenMetadataProgram,
      ...resolveTokenMetadataProgramFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.sysvarInstructions.value) {
    accounts.sysvarInstructions = {
      ...accounts.sysvarInstructions,
      ...resolveSysvarInstructionsFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.cosigner.value) {
    accounts.cosigner.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  const remainingAccounts = (args.creators ?? []).map(
    (address3) => ({ address: address3, role: AccountRole.WRITABLE })
  );
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.feeVault),
      getAccountMeta(accounts.buyer),
      getAccountMeta(accounts.buyerTa),
      getAccountMeta(accounts.listTa),
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.takerBroker),
      getAccountMeta(accounts.makerBroker),
      getAccountMeta(accounts.rentDestination),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.metadata),
      getAccountMeta(accounts.edition),
      getAccountMeta(accounts.buyerTokenRecord),
      getAccountMeta(accounts.listTokenRecord),
      getAccountMeta(accounts.authorizationRules),
      getAccountMeta(accounts.authorizationRulesProgram),
      getAccountMeta(accounts.tokenMetadataProgram),
      getAccountMeta(accounts.sysvarInstructions),
      getAccountMeta(accounts.cosigner),
      ...remainingAccounts
    ],
    programAddress,
    data: getBuyLegacyInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseBuyLegacyInstruction(instruction) {
  if (instruction.accounts.length < 24) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  const getNextOptionalAccount = () => {
    const accountMeta = getNextAccount();
    return accountMeta.address === TENSOR_MARKETPLACE_PROGRAM_ADDRESS ? void 0 : accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      feeVault: getNextAccount(),
      buyer: getNextAccount(),
      buyerTa: getNextAccount(),
      listTa: getNextAccount(),
      listState: getNextAccount(),
      mint: getNextAccount(),
      owner: getNextAccount(),
      payer: getNextAccount(),
      takerBroker: getNextOptionalAccount(),
      makerBroker: getNextOptionalAccount(),
      rentDestination: getNextAccount(),
      tokenProgram: getNextAccount(),
      associatedTokenProgram: getNextAccount(),
      marketplaceProgram: getNextAccount(),
      systemProgram: getNextAccount(),
      metadata: getNextAccount(),
      edition: getNextAccount(),
      buyerTokenRecord: getNextOptionalAccount(),
      listTokenRecord: getNextOptionalAccount(),
      authorizationRules: getNextOptionalAccount(),
      authorizationRulesProgram: getNextOptionalAccount(),
      tokenMetadataProgram: getNextOptionalAccount(),
      sysvarInstructions: getNextOptionalAccount(),
      cosigner: getNextOptionalAccount()
    },
    data: getBuyLegacyInstructionDataDecoder().decode(instruction.data)
  };
}
function getBuyLegacySplInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["maxAmount", getU64Encoder()],
      ["optionalRoyaltyPct", getOptionEncoder(getU16Encoder())],
      [
        "authorizationData",
        getOptionEncoder(getAuthorizationDataLocalEncoder())
      ]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([134, 94, 125, 229, 24, 157, 194, 199]),
      optionalRoyaltyPct: value.optionalRoyaltyPct ?? none(),
      authorizationData: value.authorizationData ?? none()
    })
  );
}
function getBuyLegacySplInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["maxAmount", getU64Decoder()],
    ["optionalRoyaltyPct", getOptionDecoder(getU16Decoder())],
    ["authorizationData", getOptionDecoder(getAuthorizationDataLocalDecoder())]
  ]);
}
function getBuyLegacySplInstructionDataCodec() {
  return combineCodec(
    getBuyLegacySplInstructionDataEncoder(),
    getBuyLegacySplInstructionDataDecoder()
  );
}
async function getBuyLegacySplInstructionAsync(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    feeVault: { value: input.feeVault ?? null, isWritable: true },
    feeVaultCurrencyTa: {
      value: input.feeVaultCurrencyTa ?? null,
      isWritable: true
    },
    buyer: { value: input.buyer ?? null, isWritable: false },
    buyerTa: { value: input.buyerTa ?? null, isWritable: true },
    listTa: { value: input.listTa ?? null, isWritable: true },
    listState: { value: input.listState ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    currency: { value: input.currency ?? null, isWritable: false },
    owner: { value: input.owner ?? null, isWritable: true },
    ownerCurrencyTa: { value: input.ownerCurrencyTa ?? null, isWritable: true },
    payer: { value: input.payer ?? null, isWritable: true },
    payerCurrencyTa: { value: input.payerCurrencyTa ?? null, isWritable: true },
    makerBroker: { value: input.makerBroker ?? null, isWritable: true },
    takerBroker: { value: input.takerBroker ?? null, isWritable: true },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    currencyTokenProgram: {
      value: input.currencyTokenProgram ?? null,
      isWritable: false
    },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    metadata: { value: input.metadata ?? null, isWritable: true },
    edition: { value: input.edition ?? null, isWritable: false },
    buyerTokenRecord: {
      value: input.buyerTokenRecord ?? null,
      isWritable: true
    },
    listTokenRecord: { value: input.listTokenRecord ?? null, isWritable: true },
    authorizationRules: {
      value: input.authorizationRules ?? null,
      isWritable: false
    },
    authorizationRulesProgram: {
      value: input.authorizationRulesProgram ?? null,
      isWritable: false
    },
    tokenMetadataProgram: {
      value: input.tokenMetadataProgram ?? null,
      isWritable: false
    },
    sysvarInstructions: {
      value: input.sysvarInstructions ?? null,
      isWritable: false
    },
    cosigner: { value: input.cosigner ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.listState.value) {
    accounts.listState.value = await findListStatePda({
      mint: expectAddress(accounts.mint.value)
    });
  }
  if (!accounts.feeVault.value) {
    accounts.feeVault = {
      ...accounts.feeVault,
      ...await resolveFeeVaultPdaFromListState(resolverScope)
    };
  }
  if (!accounts.buyer.value) {
    accounts.buyer.value = expectTransactionSigner(
      accounts.payer.value
    ).address;
  }
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.buyerTa.value) {
    accounts.buyerTa = {
      ...accounts.buyerTa,
      ...await resolveBuyerAta(resolverScope)
    };
  }
  if (!accounts.listTa.value) {
    accounts.listTa = {
      ...accounts.listTa,
      ...await resolveListAta(resolverScope)
    };
  }
  if (!accounts.rentDestination.value) {
    accounts.rentDestination.value = expectSome(accounts.owner.value);
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.metadata.value) {
    accounts.metadata = {
      ...accounts.metadata,
      ...await resolveMetadata(resolverScope)
    };
  }
  if (!args.tokenStandard) {
    args.tokenStandard = TokenStandard.ProgrammableNonFungible;
  }
  if (!accounts.edition.value) {
    accounts.edition = {
      ...accounts.edition,
      ...await resolveEditionFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.buyerTokenRecord.value) {
    accounts.buyerTokenRecord = {
      ...accounts.buyerTokenRecord,
      ...await resolveBuyerTokenRecordFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.listTokenRecord.value) {
    accounts.listTokenRecord = {
      ...accounts.listTokenRecord,
      ...await resolveListTokenRecordFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.authorizationRulesProgram.value) {
    accounts.authorizationRulesProgram = {
      ...accounts.authorizationRulesProgram,
      ...resolveAuthorizationRulesProgramFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.tokenMetadataProgram.value) {
    accounts.tokenMetadataProgram = {
      ...accounts.tokenMetadataProgram,
      ...resolveTokenMetadataProgramFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.sysvarInstructions.value) {
    accounts.sysvarInstructions = {
      ...accounts.sysvarInstructions,
      ...resolveSysvarInstructionsFromTokenStandard(resolverScope)
    };
  }
  const remainingAccounts = [
    ...(args.creators ?? []).map((address3) => ({
      address: address3,
      role: AccountRole.WRITABLE
    })),
    ...(args.creatorsTa ?? []).map((address3) => ({
      address: address3,
      role: AccountRole.WRITABLE
    }))
  ];
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.feeVault),
      getAccountMeta(accounts.feeVaultCurrencyTa),
      getAccountMeta(accounts.buyer),
      getAccountMeta(accounts.buyerTa),
      getAccountMeta(accounts.listTa),
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.currency),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.ownerCurrencyTa),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.payerCurrencyTa),
      getAccountMeta(accounts.makerBroker),
      getAccountMeta(accounts.takerBroker),
      getAccountMeta(accounts.rentDestination),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.currencyTokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.metadata),
      getAccountMeta(accounts.edition),
      getAccountMeta(accounts.buyerTokenRecord),
      getAccountMeta(accounts.listTokenRecord),
      getAccountMeta(accounts.authorizationRules),
      getAccountMeta(accounts.authorizationRulesProgram),
      getAccountMeta(accounts.tokenMetadataProgram),
      getAccountMeta(accounts.sysvarInstructions),
      getAccountMeta(accounts.cosigner),
      ...remainingAccounts
    ],
    programAddress,
    data: getBuyLegacySplInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function getBuyLegacySplInstruction(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    feeVault: { value: input.feeVault ?? null, isWritable: true },
    feeVaultCurrencyTa: {
      value: input.feeVaultCurrencyTa ?? null,
      isWritable: true
    },
    buyer: { value: input.buyer ?? null, isWritable: false },
    buyerTa: { value: input.buyerTa ?? null, isWritable: true },
    listTa: { value: input.listTa ?? null, isWritable: true },
    listState: { value: input.listState ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    currency: { value: input.currency ?? null, isWritable: false },
    owner: { value: input.owner ?? null, isWritable: true },
    ownerCurrencyTa: { value: input.ownerCurrencyTa ?? null, isWritable: true },
    payer: { value: input.payer ?? null, isWritable: true },
    payerCurrencyTa: { value: input.payerCurrencyTa ?? null, isWritable: true },
    makerBroker: { value: input.makerBroker ?? null, isWritable: true },
    takerBroker: { value: input.takerBroker ?? null, isWritable: true },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    currencyTokenProgram: {
      value: input.currencyTokenProgram ?? null,
      isWritable: false
    },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    metadata: { value: input.metadata ?? null, isWritable: true },
    edition: { value: input.edition ?? null, isWritable: false },
    buyerTokenRecord: {
      value: input.buyerTokenRecord ?? null,
      isWritable: true
    },
    listTokenRecord: { value: input.listTokenRecord ?? null, isWritable: true },
    authorizationRules: {
      value: input.authorizationRules ?? null,
      isWritable: false
    },
    authorizationRulesProgram: {
      value: input.authorizationRulesProgram ?? null,
      isWritable: false
    },
    tokenMetadataProgram: {
      value: input.tokenMetadataProgram ?? null,
      isWritable: false
    },
    sysvarInstructions: {
      value: input.sysvarInstructions ?? null,
      isWritable: false
    },
    cosigner: { value: input.cosigner ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.buyer.value) {
    accounts.buyer.value = expectTransactionSigner(
      accounts.payer.value
    ).address;
  }
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.rentDestination.value) {
    accounts.rentDestination.value = expectSome(accounts.owner.value);
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!args.tokenStandard) {
    args.tokenStandard = TokenStandard.ProgrammableNonFungible;
  }
  if (!accounts.authorizationRulesProgram.value) {
    accounts.authorizationRulesProgram = {
      ...accounts.authorizationRulesProgram,
      ...resolveAuthorizationRulesProgramFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.tokenMetadataProgram.value) {
    accounts.tokenMetadataProgram = {
      ...accounts.tokenMetadataProgram,
      ...resolveTokenMetadataProgramFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.sysvarInstructions.value) {
    accounts.sysvarInstructions = {
      ...accounts.sysvarInstructions,
      ...resolveSysvarInstructionsFromTokenStandard(resolverScope)
    };
  }
  const remainingAccounts = [
    ...(args.creators ?? []).map((address3) => ({
      address: address3,
      role: AccountRole.WRITABLE
    })),
    ...(args.creatorsTa ?? []).map((address3) => ({
      address: address3,
      role: AccountRole.WRITABLE
    }))
  ];
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.feeVault),
      getAccountMeta(accounts.feeVaultCurrencyTa),
      getAccountMeta(accounts.buyer),
      getAccountMeta(accounts.buyerTa),
      getAccountMeta(accounts.listTa),
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.currency),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.ownerCurrencyTa),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.payerCurrencyTa),
      getAccountMeta(accounts.makerBroker),
      getAccountMeta(accounts.takerBroker),
      getAccountMeta(accounts.rentDestination),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.currencyTokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.metadata),
      getAccountMeta(accounts.edition),
      getAccountMeta(accounts.buyerTokenRecord),
      getAccountMeta(accounts.listTokenRecord),
      getAccountMeta(accounts.authorizationRules),
      getAccountMeta(accounts.authorizationRulesProgram),
      getAccountMeta(accounts.tokenMetadataProgram),
      getAccountMeta(accounts.sysvarInstructions),
      getAccountMeta(accounts.cosigner),
      ...remainingAccounts
    ],
    programAddress,
    data: getBuyLegacySplInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseBuyLegacySplInstruction(instruction) {
  if (instruction.accounts.length < 29) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  const getNextOptionalAccount = () => {
    const accountMeta = getNextAccount();
    return accountMeta.address === TENSOR_MARKETPLACE_PROGRAM_ADDRESS ? void 0 : accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      feeVault: getNextAccount(),
      feeVaultCurrencyTa: getNextAccount(),
      buyer: getNextAccount(),
      buyerTa: getNextAccount(),
      listTa: getNextAccount(),
      listState: getNextAccount(),
      mint: getNextAccount(),
      currency: getNextAccount(),
      owner: getNextAccount(),
      ownerCurrencyTa: getNextAccount(),
      payer: getNextAccount(),
      payerCurrencyTa: getNextAccount(),
      makerBroker: getNextOptionalAccount(),
      takerBroker: getNextOptionalAccount(),
      rentDestination: getNextAccount(),
      tokenProgram: getNextAccount(),
      currencyTokenProgram: getNextAccount(),
      associatedTokenProgram: getNextAccount(),
      marketplaceProgram: getNextAccount(),
      systemProgram: getNextAccount(),
      metadata: getNextAccount(),
      edition: getNextAccount(),
      buyerTokenRecord: getNextOptionalAccount(),
      listTokenRecord: getNextOptionalAccount(),
      authorizationRules: getNextOptionalAccount(),
      authorizationRulesProgram: getNextOptionalAccount(),
      tokenMetadataProgram: getNextOptionalAccount(),
      sysvarInstructions: getNextOptionalAccount(),
      cosigner: getNextOptionalAccount()
    },
    data: getBuyLegacySplInstructionDataDecoder().decode(instruction.data)
  };
}
function getBuySplInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["nonce", getU64Encoder()],
      ["index", getU32Encoder()],
      ["root", fixEncoderSize(getBytesEncoder(), 32)],
      ["metaHash", fixEncoderSize(getBytesEncoder(), 32)],
      [
        "creatorShares",
        addEncoderSizePrefix(getBytesEncoder(), getU32Encoder())
      ],
      ["creatorVerified", getArrayEncoder(getBooleanEncoder())],
      ["sellerFeeBasisPoints", getU16Encoder()],
      ["maxAmount", getU64Encoder()],
      ["optionalRoyaltyPct", getOptionEncoder(getU16Encoder())]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([65, 136, 254, 255, 59, 130, 234, 174]),
      optionalRoyaltyPct: value.optionalRoyaltyPct ?? none()
    })
  );
}
function getBuySplInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["nonce", getU64Decoder()],
    ["index", getU32Decoder()],
    ["root", fixDecoderSize(getBytesDecoder(), 32)],
    ["metaHash", fixDecoderSize(getBytesDecoder(), 32)],
    ["creatorShares", addDecoderSizePrefix(getBytesDecoder(), getU32Decoder())],
    ["creatorVerified", getArrayDecoder(getBooleanDecoder())],
    ["sellerFeeBasisPoints", getU16Decoder()],
    ["maxAmount", getU64Decoder()],
    ["optionalRoyaltyPct", getOptionDecoder(getU16Decoder())]
  ]);
}
function getBuySplInstructionDataCodec() {
  return combineCodec(
    getBuySplInstructionDataEncoder(),
    getBuySplInstructionDataDecoder()
  );
}
async function getBuySplInstructionAsync(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    feeVault: { value: input.feeVault ?? null, isWritable: true },
    feeVaultTa: { value: input.feeVaultTa ?? null, isWritable: true },
    treeAuthority: { value: input.treeAuthority ?? null, isWritable: false },
    merkleTree: { value: input.merkleTree ?? null, isWritable: true },
    logWrapper: { value: input.logWrapper ?? null, isWritable: false },
    compressionProgram: {
      value: input.compressionProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    bubblegumProgram: {
      value: input.bubblegumProgram ?? null,
      isWritable: false
    },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    listState: { value: input.listState ?? null, isWritable: true },
    buyer: { value: input.buyer ?? null, isWritable: false },
    payer: { value: input.payer ?? null, isWritable: false },
    payerSource: { value: input.payerSource ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: false },
    ownerDestination: {
      value: input.ownerDestination ?? null,
      isWritable: true
    },
    currency: { value: input.currency ?? null, isWritable: false },
    takerBroker: { value: input.takerBroker ?? null, isWritable: true },
    takerBrokerCurrencyTa: {
      value: input.takerBrokerCurrencyTa ?? null,
      isWritable: true
    },
    makerBroker: { value: input.makerBroker ?? null, isWritable: true },
    makerBrokerCurrencyTa: {
      value: input.makerBrokerCurrencyTa ?? null,
      isWritable: true
    },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true },
    rentPayer: { value: input.rentPayer ?? null, isWritable: true },
    cosigner: { value: input.cosigner ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.bubblegumProgram.value) {
    accounts.bubblegumProgram.value = "BGUMAp9Gq7iTEuizy4pqaxsTyUCBK68MDfK752saRPUY";
  }
  if (!accounts.treeAuthority.value) {
    accounts.treeAuthority = {
      ...accounts.treeAuthority,
      ...await resolveTreeAuthorityPda(resolverScope)
    };
  }
  if (!accounts.logWrapper.value) {
    accounts.logWrapper.value = "noopb9bkMVfRPU8AsbpTUg8AQkHtKwMYZiFUjNRtMmV";
  }
  if (!accounts.compressionProgram.value) {
    accounts.compressionProgram.value = "cmtDvXumGCrqC1Age74AVPhSRVXJMd8PJS91L8KbNCK";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.cosigner.value) {
    accounts.cosigner.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.feeVault),
      getAccountMeta(accounts.feeVaultTa),
      getAccountMeta(accounts.treeAuthority),
      getAccountMeta(accounts.merkleTree),
      getAccountMeta(accounts.logWrapper),
      getAccountMeta(accounts.compressionProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.bubblegumProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.buyer),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.payerSource),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.ownerDestination),
      getAccountMeta(accounts.currency),
      getAccountMeta(accounts.takerBroker),
      getAccountMeta(accounts.takerBrokerCurrencyTa),
      getAccountMeta(accounts.makerBroker),
      getAccountMeta(accounts.makerBrokerCurrencyTa),
      getAccountMeta(accounts.rentDestination),
      getAccountMeta(accounts.rentPayer),
      getAccountMeta(accounts.cosigner)
    ],
    programAddress,
    data: getBuySplInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function getBuySplInstruction(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    feeVault: { value: input.feeVault ?? null, isWritable: true },
    feeVaultTa: { value: input.feeVaultTa ?? null, isWritable: true },
    treeAuthority: { value: input.treeAuthority ?? null, isWritable: false },
    merkleTree: { value: input.merkleTree ?? null, isWritable: true },
    logWrapper: { value: input.logWrapper ?? null, isWritable: false },
    compressionProgram: {
      value: input.compressionProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    bubblegumProgram: {
      value: input.bubblegumProgram ?? null,
      isWritable: false
    },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    listState: { value: input.listState ?? null, isWritable: true },
    buyer: { value: input.buyer ?? null, isWritable: false },
    payer: { value: input.payer ?? null, isWritable: false },
    payerSource: { value: input.payerSource ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: false },
    ownerDestination: {
      value: input.ownerDestination ?? null,
      isWritable: true
    },
    currency: { value: input.currency ?? null, isWritable: false },
    takerBroker: { value: input.takerBroker ?? null, isWritable: true },
    takerBrokerCurrencyTa: {
      value: input.takerBrokerCurrencyTa ?? null,
      isWritable: true
    },
    makerBroker: { value: input.makerBroker ?? null, isWritable: true },
    makerBrokerCurrencyTa: {
      value: input.makerBrokerCurrencyTa ?? null,
      isWritable: true
    },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true },
    rentPayer: { value: input.rentPayer ?? null, isWritable: true },
    cosigner: { value: input.cosigner ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.bubblegumProgram.value) {
    accounts.bubblegumProgram.value = "BGUMAp9Gq7iTEuizy4pqaxsTyUCBK68MDfK752saRPUY";
  }
  if (!accounts.logWrapper.value) {
    accounts.logWrapper.value = "noopb9bkMVfRPU8AsbpTUg8AQkHtKwMYZiFUjNRtMmV";
  }
  if (!accounts.compressionProgram.value) {
    accounts.compressionProgram.value = "cmtDvXumGCrqC1Age74AVPhSRVXJMd8PJS91L8KbNCK";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.cosigner.value) {
    accounts.cosigner.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.feeVault),
      getAccountMeta(accounts.feeVaultTa),
      getAccountMeta(accounts.treeAuthority),
      getAccountMeta(accounts.merkleTree),
      getAccountMeta(accounts.logWrapper),
      getAccountMeta(accounts.compressionProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.bubblegumProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.buyer),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.payerSource),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.ownerDestination),
      getAccountMeta(accounts.currency),
      getAccountMeta(accounts.takerBroker),
      getAccountMeta(accounts.takerBrokerCurrencyTa),
      getAccountMeta(accounts.makerBroker),
      getAccountMeta(accounts.makerBrokerCurrencyTa),
      getAccountMeta(accounts.rentDestination),
      getAccountMeta(accounts.rentPayer),
      getAccountMeta(accounts.cosigner)
    ],
    programAddress,
    data: getBuySplInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseBuySplInstruction(instruction) {
  if (instruction.accounts.length < 25) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  const getNextOptionalAccount = () => {
    const accountMeta = getNextAccount();
    return accountMeta.address === TENSOR_MARKETPLACE_PROGRAM_ADDRESS ? void 0 : accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      feeVault: getNextAccount(),
      feeVaultTa: getNextAccount(),
      treeAuthority: getNextAccount(),
      merkleTree: getNextAccount(),
      logWrapper: getNextAccount(),
      compressionProgram: getNextAccount(),
      systemProgram: getNextAccount(),
      bubblegumProgram: getNextAccount(),
      marketplaceProgram: getNextAccount(),
      tokenProgram: getNextAccount(),
      associatedTokenProgram: getNextAccount(),
      listState: getNextAccount(),
      buyer: getNextAccount(),
      payer: getNextAccount(),
      payerSource: getNextAccount(),
      owner: getNextAccount(),
      ownerDestination: getNextAccount(),
      currency: getNextAccount(),
      takerBroker: getNextOptionalAccount(),
      takerBrokerCurrencyTa: getNextOptionalAccount(),
      makerBroker: getNextOptionalAccount(),
      makerBrokerCurrencyTa: getNextOptionalAccount(),
      rentDestination: getNextAccount(),
      rentPayer: getNextAccount(),
      cosigner: getNextOptionalAccount()
    },
    data: getBuySplInstructionDataDecoder().decode(instruction.data)
  };
}
function getBuyT22InstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["maxAmount", getU64Encoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([81, 98, 227, 171, 201, 105, 180, 216])
    })
  );
}
function getBuyT22InstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["maxAmount", getU64Decoder()]
  ]);
}
function getBuyT22InstructionDataCodec() {
  return combineCodec(
    getBuyT22InstructionDataEncoder(),
    getBuyT22InstructionDataDecoder()
  );
}
async function getBuyT22InstructionAsync(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    feeVault: { value: input.feeVault ?? null, isWritable: true },
    buyer: { value: input.buyer ?? null, isWritable: false },
    buyerTa: { value: input.buyerTa ?? null, isWritable: true },
    listState: { value: input.listState ?? null, isWritable: true },
    listTa: { value: input.listTa ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    owner: { value: input.owner ?? null, isWritable: true },
    payer: { value: input.payer ?? null, isWritable: true },
    takerBroker: { value: input.takerBroker ?? null, isWritable: true },
    makerBroker: { value: input.makerBroker ?? null, isWritable: true },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    cosigner: { value: input.cosigner ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.buyer.value) {
    accounts.buyer.value = expectTransactionSigner(
      accounts.payer.value
    ).address;
  }
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.buyerTa.value) {
    accounts.buyerTa = {
      ...accounts.buyerTa,
      ...await resolveBuyerAta(resolverScope)
    };
  }
  if (!accounts.listState.value) {
    accounts.listState.value = await findListStatePda({
      mint: expectAddress(accounts.mint.value)
    });
  }
  if (!accounts.listTa.value) {
    accounts.listTa = {
      ...accounts.listTa,
      ...await resolveListAta(resolverScope)
    };
  }
  if (!accounts.rentDestination.value) {
    accounts.rentDestination.value = expectSome(accounts.owner.value);
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.cosigner.value) {
    accounts.cosigner.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  const remainingAccounts = (args.creators ?? []).map(
    (address3) => ({ address: address3, role: AccountRole.WRITABLE })
  );
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.feeVault),
      getAccountMeta(accounts.buyer),
      getAccountMeta(accounts.buyerTa),
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.listTa),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.takerBroker),
      getAccountMeta(accounts.makerBroker),
      getAccountMeta(accounts.rentDestination),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.cosigner),
      ...remainingAccounts
    ],
    programAddress,
    data: getBuyT22InstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function getBuyT22Instruction(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    feeVault: { value: input.feeVault ?? null, isWritable: true },
    buyer: { value: input.buyer ?? null, isWritable: false },
    buyerTa: { value: input.buyerTa ?? null, isWritable: true },
    listState: { value: input.listState ?? null, isWritable: true },
    listTa: { value: input.listTa ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    owner: { value: input.owner ?? null, isWritable: true },
    payer: { value: input.payer ?? null, isWritable: true },
    takerBroker: { value: input.takerBroker ?? null, isWritable: true },
    makerBroker: { value: input.makerBroker ?? null, isWritable: true },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    cosigner: { value: input.cosigner ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.buyer.value) {
    accounts.buyer.value = expectTransactionSigner(
      accounts.payer.value
    ).address;
  }
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.rentDestination.value) {
    accounts.rentDestination.value = expectSome(accounts.owner.value);
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.cosigner.value) {
    accounts.cosigner.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  const remainingAccounts = (args.creators ?? []).map(
    (address3) => ({ address: address3, role: AccountRole.WRITABLE })
  );
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.feeVault),
      getAccountMeta(accounts.buyer),
      getAccountMeta(accounts.buyerTa),
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.listTa),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.takerBroker),
      getAccountMeta(accounts.makerBroker),
      getAccountMeta(accounts.rentDestination),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.cosigner),
      ...remainingAccounts
    ],
    programAddress,
    data: getBuyT22InstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseBuyT22Instruction(instruction) {
  if (instruction.accounts.length < 16) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  const getNextOptionalAccount = () => {
    const accountMeta = getNextAccount();
    return accountMeta.address === TENSOR_MARKETPLACE_PROGRAM_ADDRESS ? void 0 : accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      feeVault: getNextAccount(),
      buyer: getNextAccount(),
      buyerTa: getNextAccount(),
      listState: getNextAccount(),
      listTa: getNextAccount(),
      mint: getNextAccount(),
      owner: getNextAccount(),
      payer: getNextAccount(),
      takerBroker: getNextOptionalAccount(),
      makerBroker: getNextOptionalAccount(),
      rentDestination: getNextAccount(),
      tokenProgram: getNextAccount(),
      associatedTokenProgram: getNextAccount(),
      marketplaceProgram: getNextAccount(),
      systemProgram: getNextAccount(),
      cosigner: getNextOptionalAccount()
    },
    data: getBuyT22InstructionDataDecoder().decode(instruction.data)
  };
}
function getBuyT22SplInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["maxAmount", getU64Encoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([102, 21, 163, 39, 94, 39, 122, 94])
    })
  );
}
function getBuyT22SplInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["maxAmount", getU64Decoder()]
  ]);
}
function getBuyT22SplInstructionDataCodec() {
  return combineCodec(
    getBuyT22SplInstructionDataEncoder(),
    getBuyT22SplInstructionDataDecoder()
  );
}
async function getBuyT22SplInstructionAsync(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    feeVault: { value: input.feeVault ?? null, isWritable: true },
    feeVaultCurrencyTa: {
      value: input.feeVaultCurrencyTa ?? null,
      isWritable: true
    },
    buyer: { value: input.buyer ?? null, isWritable: false },
    buyerTa: { value: input.buyerTa ?? null, isWritable: true },
    listTa: { value: input.listTa ?? null, isWritable: true },
    listState: { value: input.listState ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    currency: { value: input.currency ?? null, isWritable: false },
    owner: { value: input.owner ?? null, isWritable: true },
    ownerCurrencyTa: { value: input.ownerCurrencyTa ?? null, isWritable: true },
    payer: { value: input.payer ?? null, isWritable: true },
    payerCurrencyTa: { value: input.payerCurrencyTa ?? null, isWritable: true },
    takerBroker: { value: input.takerBroker ?? null, isWritable: true },
    takerBrokerCurrencyTa: {
      value: input.takerBrokerCurrencyTa ?? null,
      isWritable: true
    },
    makerBroker: { value: input.makerBroker ?? null, isWritable: true },
    makerBrokerCurrencyTa: {
      value: input.makerBrokerCurrencyTa ?? null,
      isWritable: true
    },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    currencyTokenProgram: {
      value: input.currencyTokenProgram ?? null,
      isWritable: false
    },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    cosigner: { value: input.cosigner ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.buyer.value) {
    accounts.buyer.value = expectTransactionSigner(
      accounts.payer.value
    ).address;
  }
  if (!accounts.listState.value) {
    accounts.listState.value = await findListStatePda({
      mint: expectAddress(accounts.mint.value)
    });
  }
  if (!accounts.rentDestination.value) {
    accounts.rentDestination.value = expectSome(accounts.owner.value);
  }
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const remainingAccounts = (args.creators ?? []).map(
    (address3) => ({ address: address3, role: AccountRole.WRITABLE })
  );
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.feeVault),
      getAccountMeta(accounts.feeVaultCurrencyTa),
      getAccountMeta(accounts.buyer),
      getAccountMeta(accounts.buyerTa),
      getAccountMeta(accounts.listTa),
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.currency),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.ownerCurrencyTa),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.payerCurrencyTa),
      getAccountMeta(accounts.takerBroker),
      getAccountMeta(accounts.takerBrokerCurrencyTa),
      getAccountMeta(accounts.makerBroker),
      getAccountMeta(accounts.makerBrokerCurrencyTa),
      getAccountMeta(accounts.rentDestination),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.currencyTokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.cosigner),
      ...remainingAccounts
    ],
    programAddress,
    data: getBuyT22SplInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function getBuyT22SplInstruction(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    feeVault: { value: input.feeVault ?? null, isWritable: true },
    feeVaultCurrencyTa: {
      value: input.feeVaultCurrencyTa ?? null,
      isWritable: true
    },
    buyer: { value: input.buyer ?? null, isWritable: false },
    buyerTa: { value: input.buyerTa ?? null, isWritable: true },
    listTa: { value: input.listTa ?? null, isWritable: true },
    listState: { value: input.listState ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    currency: { value: input.currency ?? null, isWritable: false },
    owner: { value: input.owner ?? null, isWritable: true },
    ownerCurrencyTa: { value: input.ownerCurrencyTa ?? null, isWritable: true },
    payer: { value: input.payer ?? null, isWritable: true },
    payerCurrencyTa: { value: input.payerCurrencyTa ?? null, isWritable: true },
    takerBroker: { value: input.takerBroker ?? null, isWritable: true },
    takerBrokerCurrencyTa: {
      value: input.takerBrokerCurrencyTa ?? null,
      isWritable: true
    },
    makerBroker: { value: input.makerBroker ?? null, isWritable: true },
    makerBrokerCurrencyTa: {
      value: input.makerBrokerCurrencyTa ?? null,
      isWritable: true
    },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    currencyTokenProgram: {
      value: input.currencyTokenProgram ?? null,
      isWritable: false
    },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    cosigner: { value: input.cosigner ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.buyer.value) {
    accounts.buyer.value = expectTransactionSigner(
      accounts.payer.value
    ).address;
  }
  if (!accounts.rentDestination.value) {
    accounts.rentDestination.value = expectSome(accounts.owner.value);
  }
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const remainingAccounts = (args.creators ?? []).map(
    (address3) => ({ address: address3, role: AccountRole.WRITABLE })
  );
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.feeVault),
      getAccountMeta(accounts.feeVaultCurrencyTa),
      getAccountMeta(accounts.buyer),
      getAccountMeta(accounts.buyerTa),
      getAccountMeta(accounts.listTa),
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.currency),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.ownerCurrencyTa),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.payerCurrencyTa),
      getAccountMeta(accounts.takerBroker),
      getAccountMeta(accounts.takerBrokerCurrencyTa),
      getAccountMeta(accounts.makerBroker),
      getAccountMeta(accounts.makerBrokerCurrencyTa),
      getAccountMeta(accounts.rentDestination),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.currencyTokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.cosigner),
      ...remainingAccounts
    ],
    programAddress,
    data: getBuyT22SplInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseBuyT22SplInstruction(instruction) {
  if (instruction.accounts.length < 23) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  const getNextOptionalAccount = () => {
    const accountMeta = getNextAccount();
    return accountMeta.address === TENSOR_MARKETPLACE_PROGRAM_ADDRESS ? void 0 : accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      feeVault: getNextAccount(),
      feeVaultCurrencyTa: getNextAccount(),
      buyer: getNextAccount(),
      buyerTa: getNextAccount(),
      listTa: getNextAccount(),
      listState: getNextAccount(),
      mint: getNextAccount(),
      currency: getNextAccount(),
      owner: getNextAccount(),
      ownerCurrencyTa: getNextAccount(),
      payer: getNextAccount(),
      payerCurrencyTa: getNextAccount(),
      takerBroker: getNextOptionalAccount(),
      takerBrokerCurrencyTa: getNextOptionalAccount(),
      makerBroker: getNextOptionalAccount(),
      makerBrokerCurrencyTa: getNextOptionalAccount(),
      rentDestination: getNextAccount(),
      tokenProgram: getNextAccount(),
      currencyTokenProgram: getNextAccount(),
      associatedTokenProgram: getNextAccount(),
      marketplaceProgram: getNextAccount(),
      systemProgram: getNextAccount(),
      cosigner: getNextOptionalAccount()
    },
    data: getBuyT22SplInstructionDataDecoder().decode(instruction.data)
  };
}
function getBuyWnsInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["maxAmount", getU64Encoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([168, 43, 179, 217, 44, 59, 35, 244])
    })
  );
}
function getBuyWnsInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["maxAmount", getU64Decoder()]
  ]);
}
function getBuyWnsInstructionDataCodec() {
  return combineCodec(
    getBuyWnsInstructionDataEncoder(),
    getBuyWnsInstructionDataDecoder()
  );
}
async function getBuyWnsInstructionAsync(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    feeVault: { value: input.feeVault ?? null, isWritable: true },
    buyer: { value: input.buyer ?? null, isWritable: false },
    buyerTa: { value: input.buyerTa ?? null, isWritable: true },
    listState: { value: input.listState ?? null, isWritable: true },
    listTa: { value: input.listTa ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    owner: { value: input.owner ?? null, isWritable: true },
    payer: { value: input.payer ?? null, isWritable: true },
    takerBroker: { value: input.takerBroker ?? null, isWritable: true },
    makerBroker: { value: input.makerBroker ?? null, isWritable: true },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    approve: { value: input.approve ?? null, isWritable: true },
    distribution: { value: input.distribution ?? null, isWritable: true },
    wnsProgram: { value: input.wnsProgram ?? null, isWritable: false },
    wnsDistributionProgram: {
      value: input.wnsDistributionProgram ?? null,
      isWritable: false
    },
    extraMetas: { value: input.extraMetas ?? null, isWritable: false },
    cosigner: { value: input.cosigner ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.buyer.value) {
    accounts.buyer.value = expectTransactionSigner(
      accounts.payer.value
    ).address;
  }
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.buyerTa.value) {
    accounts.buyerTa = {
      ...accounts.buyerTa,
      ...await resolveBuyerAta(resolverScope)
    };
  }
  if (!accounts.listState.value) {
    accounts.listState.value = await findListStatePda({
      mint: expectAddress(accounts.mint.value)
    });
  }
  if (!accounts.listTa.value) {
    accounts.listTa = {
      ...accounts.listTa,
      ...await resolveListAta(resolverScope)
    };
  }
  if (!accounts.rentDestination.value) {
    accounts.rentDestination.value = expectSome(accounts.owner.value);
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.approve.value) {
    accounts.approve = {
      ...accounts.approve,
      ...await resolveWnsApprovePda(resolverScope)
    };
  }
  if (!args.paymentMint) {
    args.paymentMint = "11111111111111111111111111111111";
  }
  if (!accounts.distribution.value) {
    accounts.distribution = {
      ...accounts.distribution,
      ...await resolveWnsDistributionPda(resolverScope)
    };
  }
  if (!accounts.wnsProgram.value) {
    accounts.wnsProgram.value = "wns1gDLt8fgLcGhWi5MqAqgXpwEP1JftKE9eZnXS1HM";
  }
  if (!accounts.wnsDistributionProgram.value) {
    accounts.wnsDistributionProgram.value = "diste3nXmK7ddDTs1zb6uday6j4etCa9RChD8fJ1xay";
  }
  if (!accounts.extraMetas.value) {
    accounts.extraMetas = {
      ...accounts.extraMetas,
      ...await resolveWnsExtraAccountMetasPda(resolverScope)
    };
  }
  if (!accounts.cosigner.value) {
    accounts.cosigner.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.feeVault),
      getAccountMeta(accounts.buyer),
      getAccountMeta(accounts.buyerTa),
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.listTa),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.takerBroker),
      getAccountMeta(accounts.makerBroker),
      getAccountMeta(accounts.rentDestination),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.approve),
      getAccountMeta(accounts.distribution),
      getAccountMeta(accounts.wnsProgram),
      getAccountMeta(accounts.wnsDistributionProgram),
      getAccountMeta(accounts.extraMetas),
      getAccountMeta(accounts.cosigner)
    ],
    programAddress,
    data: getBuyWnsInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function getBuyWnsInstruction(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    feeVault: { value: input.feeVault ?? null, isWritable: true },
    buyer: { value: input.buyer ?? null, isWritable: false },
    buyerTa: { value: input.buyerTa ?? null, isWritable: true },
    listState: { value: input.listState ?? null, isWritable: true },
    listTa: { value: input.listTa ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    owner: { value: input.owner ?? null, isWritable: true },
    payer: { value: input.payer ?? null, isWritable: true },
    takerBroker: { value: input.takerBroker ?? null, isWritable: true },
    makerBroker: { value: input.makerBroker ?? null, isWritable: true },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    approve: { value: input.approve ?? null, isWritable: true },
    distribution: { value: input.distribution ?? null, isWritable: true },
    wnsProgram: { value: input.wnsProgram ?? null, isWritable: false },
    wnsDistributionProgram: {
      value: input.wnsDistributionProgram ?? null,
      isWritable: false
    },
    extraMetas: { value: input.extraMetas ?? null, isWritable: false },
    cosigner: { value: input.cosigner ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.buyer.value) {
    accounts.buyer.value = expectTransactionSigner(
      accounts.payer.value
    ).address;
  }
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.rentDestination.value) {
    accounts.rentDestination.value = expectSome(accounts.owner.value);
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!args.paymentMint) {
    args.paymentMint = "11111111111111111111111111111111";
  }
  if (!accounts.wnsProgram.value) {
    accounts.wnsProgram.value = "wns1gDLt8fgLcGhWi5MqAqgXpwEP1JftKE9eZnXS1HM";
  }
  if (!accounts.wnsDistributionProgram.value) {
    accounts.wnsDistributionProgram.value = "diste3nXmK7ddDTs1zb6uday6j4etCa9RChD8fJ1xay";
  }
  if (!accounts.cosigner.value) {
    accounts.cosigner.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.feeVault),
      getAccountMeta(accounts.buyer),
      getAccountMeta(accounts.buyerTa),
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.listTa),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.takerBroker),
      getAccountMeta(accounts.makerBroker),
      getAccountMeta(accounts.rentDestination),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.approve),
      getAccountMeta(accounts.distribution),
      getAccountMeta(accounts.wnsProgram),
      getAccountMeta(accounts.wnsDistributionProgram),
      getAccountMeta(accounts.extraMetas),
      getAccountMeta(accounts.cosigner)
    ],
    programAddress,
    data: getBuyWnsInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseBuyWnsInstruction(instruction) {
  if (instruction.accounts.length < 21) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  const getNextOptionalAccount = () => {
    const accountMeta = getNextAccount();
    return accountMeta.address === TENSOR_MARKETPLACE_PROGRAM_ADDRESS ? void 0 : accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      feeVault: getNextAccount(),
      buyer: getNextAccount(),
      buyerTa: getNextAccount(),
      listState: getNextAccount(),
      listTa: getNextAccount(),
      mint: getNextAccount(),
      owner: getNextAccount(),
      payer: getNextAccount(),
      takerBroker: getNextOptionalAccount(),
      makerBroker: getNextOptionalAccount(),
      rentDestination: getNextAccount(),
      tokenProgram: getNextAccount(),
      associatedTokenProgram: getNextAccount(),
      marketplaceProgram: getNextAccount(),
      systemProgram: getNextAccount(),
      approve: getNextAccount(),
      distribution: getNextAccount(),
      wnsProgram: getNextAccount(),
      wnsDistributionProgram: getNextAccount(),
      extraMetas: getNextAccount(),
      cosigner: getNextOptionalAccount()
    },
    data: getBuyWnsInstructionDataDecoder().decode(instruction.data)
  };
}
function getBuyWnsSplInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["maxAmount", getU64Encoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([113, 137, 57, 23, 186, 196, 217, 210])
    })
  );
}
function getBuyWnsSplInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["maxAmount", getU64Decoder()]
  ]);
}
function getBuyWnsSplInstructionDataCodec() {
  return combineCodec(
    getBuyWnsSplInstructionDataEncoder(),
    getBuyWnsSplInstructionDataDecoder()
  );
}
function getBuyWnsSplInstruction(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    feeVault: { value: input.feeVault ?? null, isWritable: true },
    feeVaultCurrencyTa: {
      value: input.feeVaultCurrencyTa ?? null,
      isWritable: true
    },
    buyer: { value: input.buyer ?? null, isWritable: false },
    buyerTa: { value: input.buyerTa ?? null, isWritable: true },
    listTa: { value: input.listTa ?? null, isWritable: true },
    listState: { value: input.listState ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    currency: { value: input.currency ?? null, isWritable: false },
    owner: { value: input.owner ?? null, isWritable: true },
    ownerCurrencyTa: { value: input.ownerCurrencyTa ?? null, isWritable: true },
    payer: { value: input.payer ?? null, isWritable: true },
    payerCurrencyTa: { value: input.payerCurrencyTa ?? null, isWritable: true },
    takerBroker: { value: input.takerBroker ?? null, isWritable: true },
    takerBrokerCurrencyTa: {
      value: input.takerBrokerCurrencyTa ?? null,
      isWritable: true
    },
    makerBroker: { value: input.makerBroker ?? null, isWritable: true },
    makerBrokerCurrencyTa: {
      value: input.makerBrokerCurrencyTa ?? null,
      isWritable: true
    },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    currencyTokenProgram: {
      value: input.currencyTokenProgram ?? null,
      isWritable: false
    },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    approve: { value: input.approve ?? null, isWritable: true },
    distribution: { value: input.distribution ?? null, isWritable: true },
    distributionCurrencyTa: {
      value: input.distributionCurrencyTa ?? null,
      isWritable: true
    },
    wnsProgram: { value: input.wnsProgram ?? null, isWritable: false },
    wnsDistributionProgram: {
      value: input.wnsDistributionProgram ?? null,
      isWritable: false
    },
    extraMetas: { value: input.extraMetas ?? null, isWritable: false },
    cosigner: { value: input.cosigner ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.wnsProgram.value) {
    accounts.wnsProgram.value = "wns1gDLt8fgLcGhWi5MqAqgXpwEP1JftKE9eZnXS1HM";
  }
  if (!accounts.wnsDistributionProgram.value) {
    accounts.wnsDistributionProgram.value = "diste3nXmK7ddDTs1zb6uday6j4etCa9RChD8fJ1xay";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.feeVault),
      getAccountMeta(accounts.feeVaultCurrencyTa),
      getAccountMeta(accounts.buyer),
      getAccountMeta(accounts.buyerTa),
      getAccountMeta(accounts.listTa),
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.currency),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.ownerCurrencyTa),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.payerCurrencyTa),
      getAccountMeta(accounts.takerBroker),
      getAccountMeta(accounts.takerBrokerCurrencyTa),
      getAccountMeta(accounts.makerBroker),
      getAccountMeta(accounts.makerBrokerCurrencyTa),
      getAccountMeta(accounts.rentDestination),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.currencyTokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.approve),
      getAccountMeta(accounts.distribution),
      getAccountMeta(accounts.distributionCurrencyTa),
      getAccountMeta(accounts.wnsProgram),
      getAccountMeta(accounts.wnsDistributionProgram),
      getAccountMeta(accounts.extraMetas),
      getAccountMeta(accounts.cosigner)
    ],
    programAddress,
    data: getBuyWnsSplInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseBuyWnsSplInstruction(instruction) {
  if (instruction.accounts.length < 29) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  const getNextOptionalAccount = () => {
    const accountMeta = getNextAccount();
    return accountMeta.address === TENSOR_MARKETPLACE_PROGRAM_ADDRESS ? void 0 : accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      feeVault: getNextAccount(),
      feeVaultCurrencyTa: getNextAccount(),
      buyer: getNextAccount(),
      buyerTa: getNextAccount(),
      listTa: getNextAccount(),
      listState: getNextAccount(),
      mint: getNextAccount(),
      currency: getNextAccount(),
      owner: getNextAccount(),
      ownerCurrencyTa: getNextAccount(),
      payer: getNextAccount(),
      payerCurrencyTa: getNextAccount(),
      takerBroker: getNextOptionalAccount(),
      takerBrokerCurrencyTa: getNextOptionalAccount(),
      makerBroker: getNextOptionalAccount(),
      makerBrokerCurrencyTa: getNextOptionalAccount(),
      rentDestination: getNextAccount(),
      tokenProgram: getNextAccount(),
      currencyTokenProgram: getNextAccount(),
      associatedTokenProgram: getNextAccount(),
      marketplaceProgram: getNextAccount(),
      systemProgram: getNextAccount(),
      approve: getNextAccount(),
      distribution: getNextAccount(),
      distributionCurrencyTa: getNextAccount(),
      wnsProgram: getNextAccount(),
      wnsDistributionProgram: getNextAccount(),
      extraMetas: getNextAccount(),
      cosigner: getNextOptionalAccount()
    },
    data: getBuyWnsSplInstructionDataDecoder().decode(instruction.data)
  };
}
function getCancelBidInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([["discriminator", fixEncoderSize(getBytesEncoder(), 8)]]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([40, 243, 190, 217, 208, 253, 86, 206])
    })
  );
}
function getCancelBidInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)]
  ]);
}
function getCancelBidInstructionDataCodec() {
  return combineCodec(
    getCancelBidInstructionDataEncoder(),
    getCancelBidInstructionDataDecoder()
  );
}
async function getCancelBidInstructionAsync(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    bidState: { value: input.bidState ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: true },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!args.bidId) {
    args.bidId = "11111111111111111111111111111111";
  }
  if (!accounts.bidState.value) {
    accounts.bidState.value = await findBidStatePda({
      bidId: expectSome(args.bidId),
      owner: expectAddress(accounts.owner.value)
    });
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.rentDestination.value) {
    accounts.rentDestination.value = expectTransactionSigner(
      accounts.owner.value
    ).address;
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.bidState),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.rentDestination)
    ],
    programAddress,
    data: getCancelBidInstructionDataEncoder().encode({})
  };
  return instruction;
}
function getCancelBidInstruction(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    bidState: { value: input.bidState ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: true },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true }
  };
  const accounts = originalAccounts;
  ({ ...input });
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.rentDestination.value) {
    accounts.rentDestination.value = expectTransactionSigner(
      accounts.owner.value
    ).address;
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.bidState),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.rentDestination)
    ],
    programAddress,
    data: getCancelBidInstructionDataEncoder().encode({})
  };
  return instruction;
}
function parseCancelBidInstruction(instruction) {
  if (instruction.accounts.length < 5) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      bidState: getNextAccount(),
      owner: getNextAccount(),
      systemProgram: getNextAccount(),
      marketplaceProgram: getNextAccount(),
      rentDestination: getNextAccount()
    },
    data: getCancelBidInstructionDataDecoder().decode(instruction.data)
  };
}
function getCloseExpiredBidInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([["discriminator", fixEncoderSize(getBytesEncoder(), 8)]]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([83, 20, 105, 67, 248, 68, 104, 190])
    })
  );
}
function getCloseExpiredBidInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)]
  ]);
}
function getCloseExpiredBidInstructionDataCodec() {
  return combineCodec(
    getCloseExpiredBidInstructionDataEncoder(),
    getCloseExpiredBidInstructionDataDecoder()
  );
}
async function getCloseExpiredBidInstructionAsync(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    bidState: { value: input.bidState ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: true },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!args.bidId) {
    args.bidId = "11111111111111111111111111111111";
  }
  if (!accounts.bidState.value) {
    accounts.bidState.value = await findBidStatePda({
      bidId: expectSome(args.bidId),
      owner: expectAddress(accounts.owner.value)
    });
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.rentDestination.value) {
    accounts.rentDestination.value = expectSome(accounts.owner.value);
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.bidState),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.rentDestination)
    ],
    programAddress,
    data: getCloseExpiredBidInstructionDataEncoder().encode({})
  };
  return instruction;
}
function getCloseExpiredBidInstruction(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    bidState: { value: input.bidState ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: true },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true }
  };
  const accounts = originalAccounts;
  ({ ...input });
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.rentDestination.value) {
    accounts.rentDestination.value = expectSome(accounts.owner.value);
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.bidState),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.rentDestination)
    ],
    programAddress,
    data: getCloseExpiredBidInstructionDataEncoder().encode({})
  };
  return instruction;
}
function parseCloseExpiredBidInstruction(instruction) {
  if (instruction.accounts.length < 5) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      bidState: getNextAccount(),
      owner: getNextAccount(),
      systemProgram: getNextAccount(),
      marketplaceProgram: getNextAccount(),
      rentDestination: getNextAccount()
    },
    data: getCloseExpiredBidInstructionDataDecoder().decode(instruction.data)
  };
}
function getCloseExpiredListingCompressedInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["nonce", getU64Encoder()],
      ["index", getU32Encoder()],
      ["root", fixEncoderSize(getBytesEncoder(), 32)],
      ["dataHash", fixEncoderSize(getBytesEncoder(), 32)],
      ["creatorHash", fixEncoderSize(getBytesEncoder(), 32)]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([150, 70, 13, 135, 9, 204, 75, 4])
    })
  );
}
function getCloseExpiredListingCompressedInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["nonce", getU64Decoder()],
    ["index", getU32Decoder()],
    ["root", fixDecoderSize(getBytesDecoder(), 32)],
    ["dataHash", fixDecoderSize(getBytesDecoder(), 32)],
    ["creatorHash", fixDecoderSize(getBytesDecoder(), 32)]
  ]);
}
function getCloseExpiredListingCompressedInstructionDataCodec() {
  return combineCodec(
    getCloseExpiredListingCompressedInstructionDataEncoder(),
    getCloseExpiredListingCompressedInstructionDataDecoder()
  );
}
async function getCloseExpiredListingCompressedInstructionAsync(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    listState: { value: input.listState ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: false },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    treeAuthority: { value: input.treeAuthority ?? null, isWritable: false },
    merkleTree: { value: input.merkleTree ?? null, isWritable: true },
    logWrapper: { value: input.logWrapper ?? null, isWritable: false },
    compressionProgram: {
      value: input.compressionProgram ?? null,
      isWritable: false
    },
    bubblegumProgram: {
      value: input.bubblegumProgram ?? null,
      isWritable: false
    },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.bubblegumProgram.value) {
    accounts.bubblegumProgram.value = "BGUMAp9Gq7iTEuizy4pqaxsTyUCBK68MDfK752saRPUY";
  }
  if (!accounts.treeAuthority.value) {
    accounts.treeAuthority = {
      ...accounts.treeAuthority,
      ...await resolveTreeAuthorityPda(resolverScope)
    };
  }
  if (!accounts.logWrapper.value) {
    accounts.logWrapper.value = "noopb9bkMVfRPU8AsbpTUg8AQkHtKwMYZiFUjNRtMmV";
  }
  if (!accounts.compressionProgram.value) {
    accounts.compressionProgram.value = "cmtDvXumGCrqC1Age74AVPhSRVXJMd8PJS91L8KbNCK";
  }
  if (!accounts.rentDestination.value) {
    accounts.rentDestination.value = expectSome(accounts.owner.value);
  }
  if (!args.nonce) {
    args.nonce = expectSome(args.index);
  }
  if (!args.creators) {
    args.creators = [];
  }
  if (!args.proof) {
    args.proof = [];
  }
  if (!args.canopyDepth) {
    args.canopyDepth = 0;
  }
  const remainingAccounts = [
    ...resolveCreatorPath(resolverScope),
    ...resolveProofPath(resolverScope)
  ];
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.treeAuthority),
      getAccountMeta(accounts.merkleTree),
      getAccountMeta(accounts.logWrapper),
      getAccountMeta(accounts.compressionProgram),
      getAccountMeta(accounts.bubblegumProgram),
      getAccountMeta(accounts.rentDestination),
      ...remainingAccounts
    ],
    programAddress,
    data: getCloseExpiredListingCompressedInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function getCloseExpiredListingCompressedInstruction(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    listState: { value: input.listState ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: false },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    treeAuthority: { value: input.treeAuthority ?? null, isWritable: false },
    merkleTree: { value: input.merkleTree ?? null, isWritable: true },
    logWrapper: { value: input.logWrapper ?? null, isWritable: false },
    compressionProgram: {
      value: input.compressionProgram ?? null,
      isWritable: false
    },
    bubblegumProgram: {
      value: input.bubblegumProgram ?? null,
      isWritable: false
    },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.bubblegumProgram.value) {
    accounts.bubblegumProgram.value = "BGUMAp9Gq7iTEuizy4pqaxsTyUCBK68MDfK752saRPUY";
  }
  if (!accounts.logWrapper.value) {
    accounts.logWrapper.value = "noopb9bkMVfRPU8AsbpTUg8AQkHtKwMYZiFUjNRtMmV";
  }
  if (!accounts.compressionProgram.value) {
    accounts.compressionProgram.value = "cmtDvXumGCrqC1Age74AVPhSRVXJMd8PJS91L8KbNCK";
  }
  if (!accounts.rentDestination.value) {
    accounts.rentDestination.value = expectSome(accounts.owner.value);
  }
  if (!args.nonce) {
    args.nonce = expectSome(args.index);
  }
  if (!args.creators) {
    args.creators = [];
  }
  if (!args.proof) {
    args.proof = [];
  }
  if (!args.canopyDepth) {
    args.canopyDepth = 0;
  }
  const remainingAccounts = [
    ...resolveCreatorPath(resolverScope),
    ...resolveProofPath(resolverScope)
  ];
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.treeAuthority),
      getAccountMeta(accounts.merkleTree),
      getAccountMeta(accounts.logWrapper),
      getAccountMeta(accounts.compressionProgram),
      getAccountMeta(accounts.bubblegumProgram),
      getAccountMeta(accounts.rentDestination),
      ...remainingAccounts
    ],
    programAddress,
    data: getCloseExpiredListingCompressedInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseCloseExpiredListingCompressedInstruction(instruction) {
  if (instruction.accounts.length < 10) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      listState: getNextAccount(),
      owner: getNextAccount(),
      systemProgram: getNextAccount(),
      marketplaceProgram: getNextAccount(),
      treeAuthority: getNextAccount(),
      merkleTree: getNextAccount(),
      logWrapper: getNextAccount(),
      compressionProgram: getNextAccount(),
      bubblegumProgram: getNextAccount(),
      rentDestination: getNextAccount()
    },
    data: getCloseExpiredListingCompressedInstructionDataDecoder().decode(
      instruction.data
    )
  };
}
function getCloseExpiredListingCoreInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([["discriminator", fixEncoderSize(getBytesEncoder(), 8)]]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([89, 171, 78, 80, 74, 188, 63, 58])
    })
  );
}
function getCloseExpiredListingCoreInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)]
  ]);
}
function getCloseExpiredListingCoreInstructionDataCodec() {
  return combineCodec(
    getCloseExpiredListingCoreInstructionDataEncoder(),
    getCloseExpiredListingCoreInstructionDataDecoder()
  );
}
function getCloseExpiredListingCoreInstruction(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    listState: { value: input.listState ?? null, isWritable: true },
    asset: { value: input.asset ?? null, isWritable: true },
    collection: { value: input.collection ?? null, isWritable: false },
    owner: { value: input.owner ?? null, isWritable: false },
    mplCoreProgram: { value: input.mplCoreProgram ?? null, isWritable: false },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true }
  };
  const accounts = originalAccounts;
  if (!accounts.mplCoreProgram.value) {
    accounts.mplCoreProgram.value = "CoREENxT6tW1HoK8ypY1SxRMZTcVPm7R94rH4PZNhX7d";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.asset),
      getAccountMeta(accounts.collection),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.mplCoreProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.rentDestination)
    ],
    programAddress,
    data: getCloseExpiredListingCoreInstructionDataEncoder().encode({})
  };
  return instruction;
}
function parseCloseExpiredListingCoreInstruction(instruction) {
  if (instruction.accounts.length < 8) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  const getNextOptionalAccount = () => {
    const accountMeta = getNextAccount();
    return accountMeta.address === TENSOR_MARKETPLACE_PROGRAM_ADDRESS ? void 0 : accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      listState: getNextAccount(),
      asset: getNextAccount(),
      collection: getNextOptionalAccount(),
      owner: getNextAccount(),
      mplCoreProgram: getNextAccount(),
      systemProgram: getNextAccount(),
      marketplaceProgram: getNextAccount(),
      rentDestination: getNextAccount()
    },
    data: getCloseExpiredListingCoreInstructionDataDecoder().decode(
      instruction.data
    )
  };
}
function getCloseExpiredListingLegacyInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      [
        "authorizationData",
        getOptionEncoder(getAuthorizationDataLocalEncoder())
      ]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([56, 16, 96, 188, 55, 68, 250, 58]),
      authorizationData: value.authorizationData ?? none()
    })
  );
}
function getCloseExpiredListingLegacyInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["authorizationData", getOptionDecoder(getAuthorizationDataLocalDecoder())]
  ]);
}
function getCloseExpiredListingLegacyInstructionDataCodec() {
  return combineCodec(
    getCloseExpiredListingLegacyInstructionDataEncoder(),
    getCloseExpiredListingLegacyInstructionDataDecoder()
  );
}
async function getCloseExpiredListingLegacyInstructionAsync(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    owner: { value: input.owner ?? null, isWritable: false },
    ownerTa: { value: input.ownerTa ?? null, isWritable: true },
    listState: { value: input.listState ?? null, isWritable: true },
    listTa: { value: input.listTa ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    payer: { value: input.payer ?? null, isWritable: true },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    metadata: { value: input.metadata ?? null, isWritable: true },
    edition: { value: input.edition ?? null, isWritable: false },
    ownerTokenRecord: {
      value: input.ownerTokenRecord ?? null,
      isWritable: true
    },
    listTokenRecord: { value: input.listTokenRecord ?? null, isWritable: true },
    authorizationRules: {
      value: input.authorizationRules ?? null,
      isWritable: false
    },
    authorizationRulesProgram: {
      value: input.authorizationRulesProgram ?? null,
      isWritable: false
    },
    tokenMetadataProgram: {
      value: input.tokenMetadataProgram ?? null,
      isWritable: false
    },
    sysvarInstructions: {
      value: input.sysvarInstructions ?? null,
      isWritable: false
    }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.owner.value) {
    accounts.owner.value = expectTransactionSigner(
      accounts.payer.value
    ).address;
  }
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.ownerTa.value) {
    accounts.ownerTa = {
      ...accounts.ownerTa,
      ...await resolveOwnerAta(resolverScope)
    };
  }
  if (!accounts.listState.value) {
    accounts.listState.value = await findListStatePda({
      mint: expectAddress(accounts.mint.value)
    });
  }
  if (!accounts.listTa.value) {
    accounts.listTa = {
      ...accounts.listTa,
      ...await resolveListAta(resolverScope)
    };
  }
  if (!accounts.rentDestination.value) {
    accounts.rentDestination.value = expectSome(accounts.owner.value);
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.metadata.value) {
    accounts.metadata = {
      ...accounts.metadata,
      ...await resolveMetadata(resolverScope)
    };
  }
  if (!args.tokenStandard) {
    args.tokenStandard = TokenStandard.ProgrammableNonFungible;
  }
  if (!accounts.edition.value) {
    accounts.edition = {
      ...accounts.edition,
      ...await resolveEditionFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.ownerTokenRecord.value) {
    accounts.ownerTokenRecord = {
      ...accounts.ownerTokenRecord,
      ...await resolveOwnerTokenRecordFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.listTokenRecord.value) {
    accounts.listTokenRecord = {
      ...accounts.listTokenRecord,
      ...await resolveListTokenRecordFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.authorizationRulesProgram.value) {
    accounts.authorizationRulesProgram = {
      ...accounts.authorizationRulesProgram,
      ...resolveAuthorizationRulesProgramFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.tokenMetadataProgram.value) {
    accounts.tokenMetadataProgram = {
      ...accounts.tokenMetadataProgram,
      ...resolveTokenMetadataProgramFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.sysvarInstructions.value) {
    accounts.sysvarInstructions = {
      ...accounts.sysvarInstructions,
      ...resolveSysvarInstructionsFromTokenStandard(resolverScope)
    };
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.ownerTa),
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.listTa),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.rentDestination),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.metadata),
      getAccountMeta(accounts.edition),
      getAccountMeta(accounts.ownerTokenRecord),
      getAccountMeta(accounts.listTokenRecord),
      getAccountMeta(accounts.authorizationRules),
      getAccountMeta(accounts.authorizationRulesProgram),
      getAccountMeta(accounts.tokenMetadataProgram),
      getAccountMeta(accounts.sysvarInstructions)
    ],
    programAddress,
    data: getCloseExpiredListingLegacyInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function getCloseExpiredListingLegacyInstruction(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    owner: { value: input.owner ?? null, isWritable: false },
    ownerTa: { value: input.ownerTa ?? null, isWritable: true },
    listState: { value: input.listState ?? null, isWritable: true },
    listTa: { value: input.listTa ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    payer: { value: input.payer ?? null, isWritable: true },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    metadata: { value: input.metadata ?? null, isWritable: true },
    edition: { value: input.edition ?? null, isWritable: false },
    ownerTokenRecord: {
      value: input.ownerTokenRecord ?? null,
      isWritable: true
    },
    listTokenRecord: { value: input.listTokenRecord ?? null, isWritable: true },
    authorizationRules: {
      value: input.authorizationRules ?? null,
      isWritable: false
    },
    authorizationRulesProgram: {
      value: input.authorizationRulesProgram ?? null,
      isWritable: false
    },
    tokenMetadataProgram: {
      value: input.tokenMetadataProgram ?? null,
      isWritable: false
    },
    sysvarInstructions: {
      value: input.sysvarInstructions ?? null,
      isWritable: false
    }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.owner.value) {
    accounts.owner.value = expectTransactionSigner(
      accounts.payer.value
    ).address;
  }
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.rentDestination.value) {
    accounts.rentDestination.value = expectSome(accounts.owner.value);
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!args.tokenStandard) {
    args.tokenStandard = TokenStandard.ProgrammableNonFungible;
  }
  if (!accounts.authorizationRulesProgram.value) {
    accounts.authorizationRulesProgram = {
      ...accounts.authorizationRulesProgram,
      ...resolveAuthorizationRulesProgramFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.tokenMetadataProgram.value) {
    accounts.tokenMetadataProgram = {
      ...accounts.tokenMetadataProgram,
      ...resolveTokenMetadataProgramFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.sysvarInstructions.value) {
    accounts.sysvarInstructions = {
      ...accounts.sysvarInstructions,
      ...resolveSysvarInstructionsFromTokenStandard(resolverScope)
    };
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.ownerTa),
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.listTa),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.rentDestination),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.metadata),
      getAccountMeta(accounts.edition),
      getAccountMeta(accounts.ownerTokenRecord),
      getAccountMeta(accounts.listTokenRecord),
      getAccountMeta(accounts.authorizationRules),
      getAccountMeta(accounts.authorizationRulesProgram),
      getAccountMeta(accounts.tokenMetadataProgram),
      getAccountMeta(accounts.sysvarInstructions)
    ],
    programAddress,
    data: getCloseExpiredListingLegacyInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseCloseExpiredListingLegacyInstruction(instruction) {
  if (instruction.accounts.length < 19) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  const getNextOptionalAccount = () => {
    const accountMeta = getNextAccount();
    return accountMeta.address === TENSOR_MARKETPLACE_PROGRAM_ADDRESS ? void 0 : accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      owner: getNextAccount(),
      ownerTa: getNextAccount(),
      listState: getNextAccount(),
      listTa: getNextAccount(),
      mint: getNextAccount(),
      payer: getNextAccount(),
      rentDestination: getNextAccount(),
      tokenProgram: getNextAccount(),
      associatedTokenProgram: getNextAccount(),
      systemProgram: getNextAccount(),
      marketplaceProgram: getNextAccount(),
      metadata: getNextAccount(),
      edition: getNextAccount(),
      ownerTokenRecord: getNextOptionalAccount(),
      listTokenRecord: getNextOptionalAccount(),
      authorizationRules: getNextOptionalAccount(),
      authorizationRulesProgram: getNextOptionalAccount(),
      tokenMetadataProgram: getNextOptionalAccount(),
      sysvarInstructions: getNextOptionalAccount()
    },
    data: getCloseExpiredListingLegacyInstructionDataDecoder().decode(
      instruction.data
    )
  };
}
function getCloseExpiredListingT22InstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([["discriminator", fixEncoderSize(getBytesEncoder(), 8)]]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([69, 2, 190, 122, 144, 119, 122, 220])
    })
  );
}
function getCloseExpiredListingT22InstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)]
  ]);
}
function getCloseExpiredListingT22InstructionDataCodec() {
  return combineCodec(
    getCloseExpiredListingT22InstructionDataEncoder(),
    getCloseExpiredListingT22InstructionDataDecoder()
  );
}
async function getCloseExpiredListingT22InstructionAsync(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    owner: { value: input.owner ?? null, isWritable: false },
    ownerTa: { value: input.ownerTa ?? null, isWritable: true },
    listState: { value: input.listState ?? null, isWritable: true },
    listTa: { value: input.listTa ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true },
    payer: { value: input.payer ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    }
  };
  const accounts = originalAccounts;
  const resolverScope = { programAddress, accounts };
  if (!accounts.owner.value) {
    accounts.owner.value = expectTransactionSigner(
      accounts.payer.value
    ).address;
  }
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.ownerTa.value) {
    accounts.ownerTa = {
      ...accounts.ownerTa,
      ...await resolveOwnerAta(resolverScope)
    };
  }
  if (!accounts.listState.value) {
    accounts.listState.value = await findListStatePda({
      mint: expectAddress(accounts.mint.value)
    });
  }
  if (!accounts.listTa.value) {
    accounts.listTa = {
      ...accounts.listTa,
      ...await resolveListAta(resolverScope)
    };
  }
  if (!accounts.rentDestination.value) {
    accounts.rentDestination.value = expectSome(accounts.owner.value);
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.ownerTa),
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.listTa),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.rentDestination),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.marketplaceProgram)
    ],
    programAddress,
    data: getCloseExpiredListingT22InstructionDataEncoder().encode({})
  };
  return instruction;
}
function getCloseExpiredListingT22Instruction(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    owner: { value: input.owner ?? null, isWritable: false },
    ownerTa: { value: input.ownerTa ?? null, isWritable: true },
    listState: { value: input.listState ?? null, isWritable: true },
    listTa: { value: input.listTa ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true },
    payer: { value: input.payer ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    }
  };
  const accounts = originalAccounts;
  if (!accounts.owner.value) {
    accounts.owner.value = expectTransactionSigner(
      accounts.payer.value
    ).address;
  }
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.rentDestination.value) {
    accounts.rentDestination.value = expectSome(accounts.owner.value);
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.ownerTa),
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.listTa),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.rentDestination),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.marketplaceProgram)
    ],
    programAddress,
    data: getCloseExpiredListingT22InstructionDataEncoder().encode({})
  };
  return instruction;
}
function parseCloseExpiredListingT22Instruction(instruction) {
  if (instruction.accounts.length < 11) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      owner: getNextAccount(),
      ownerTa: getNextAccount(),
      listState: getNextAccount(),
      listTa: getNextAccount(),
      mint: getNextAccount(),
      rentDestination: getNextAccount(),
      payer: getNextAccount(),
      tokenProgram: getNextAccount(),
      associatedTokenProgram: getNextAccount(),
      systemProgram: getNextAccount(),
      marketplaceProgram: getNextAccount()
    },
    data: getCloseExpiredListingT22InstructionDataDecoder().decode(
      instruction.data
    )
  };
}
function getCloseExpiredListingWnsInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([["discriminator", fixEncoderSize(getBytesEncoder(), 8)]]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([222, 31, 183, 134, 230, 207, 7, 132])
    })
  );
}
function getCloseExpiredListingWnsInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)]
  ]);
}
function getCloseExpiredListingWnsInstructionDataCodec() {
  return combineCodec(
    getCloseExpiredListingWnsInstructionDataEncoder(),
    getCloseExpiredListingWnsInstructionDataDecoder()
  );
}
async function getCloseExpiredListingWnsInstructionAsync(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    owner: { value: input.owner ?? null, isWritable: false },
    ownerTa: { value: input.ownerTa ?? null, isWritable: true },
    listState: { value: input.listState ?? null, isWritable: true },
    listTa: { value: input.listTa ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true },
    payer: { value: input.payer ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    approve: { value: input.approve ?? null, isWritable: true },
    distribution: { value: input.distribution ?? null, isWritable: true },
    wnsProgram: { value: input.wnsProgram ?? null, isWritable: false },
    wnsDistributionProgram: {
      value: input.wnsDistributionProgram ?? null,
      isWritable: false
    },
    extraMetas: { value: input.extraMetas ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.owner.value) {
    accounts.owner.value = expectTransactionSigner(
      accounts.payer.value
    ).address;
  }
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.ownerTa.value) {
    accounts.ownerTa = {
      ...accounts.ownerTa,
      ...await resolveOwnerAta(resolverScope)
    };
  }
  if (!accounts.listState.value) {
    accounts.listState.value = await findListStatePda({
      mint: expectAddress(accounts.mint.value)
    });
  }
  if (!accounts.listTa.value) {
    accounts.listTa = {
      ...accounts.listTa,
      ...await resolveListAta(resolverScope)
    };
  }
  if (!accounts.rentDestination.value) {
    accounts.rentDestination.value = expectSome(accounts.owner.value);
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.approve.value) {
    accounts.approve = {
      ...accounts.approve,
      ...await resolveWnsApprovePda(resolverScope)
    };
  }
  if (!args.paymentMint) {
    args.paymentMint = "11111111111111111111111111111111";
  }
  if (!accounts.distribution.value) {
    accounts.distribution = {
      ...accounts.distribution,
      ...await resolveWnsDistributionPda(resolverScope)
    };
  }
  if (!accounts.wnsProgram.value) {
    accounts.wnsProgram.value = "wns1gDLt8fgLcGhWi5MqAqgXpwEP1JftKE9eZnXS1HM";
  }
  if (!accounts.wnsDistributionProgram.value) {
    accounts.wnsDistributionProgram.value = "diste3nXmK7ddDTs1zb6uday6j4etCa9RChD8fJ1xay";
  }
  if (!accounts.extraMetas.value) {
    accounts.extraMetas = {
      ...accounts.extraMetas,
      ...await resolveWnsExtraAccountMetasPda(resolverScope)
    };
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.ownerTa),
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.listTa),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.rentDestination),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.approve),
      getAccountMeta(accounts.distribution),
      getAccountMeta(accounts.wnsProgram),
      getAccountMeta(accounts.wnsDistributionProgram),
      getAccountMeta(accounts.extraMetas)
    ],
    programAddress,
    data: getCloseExpiredListingWnsInstructionDataEncoder().encode({})
  };
  return instruction;
}
function getCloseExpiredListingWnsInstruction(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    owner: { value: input.owner ?? null, isWritable: false },
    ownerTa: { value: input.ownerTa ?? null, isWritable: true },
    listState: { value: input.listState ?? null, isWritable: true },
    listTa: { value: input.listTa ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true },
    payer: { value: input.payer ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    approve: { value: input.approve ?? null, isWritable: true },
    distribution: { value: input.distribution ?? null, isWritable: true },
    wnsProgram: { value: input.wnsProgram ?? null, isWritable: false },
    wnsDistributionProgram: {
      value: input.wnsDistributionProgram ?? null,
      isWritable: false
    },
    extraMetas: { value: input.extraMetas ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  ({ ...input });
  if (!accounts.owner.value) {
    accounts.owner.value = expectTransactionSigner(
      accounts.payer.value
    ).address;
  }
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.rentDestination.value) {
    accounts.rentDestination.value = expectSome(accounts.owner.value);
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.wnsProgram.value) {
    accounts.wnsProgram.value = "wns1gDLt8fgLcGhWi5MqAqgXpwEP1JftKE9eZnXS1HM";
  }
  if (!accounts.wnsDistributionProgram.value) {
    accounts.wnsDistributionProgram.value = "diste3nXmK7ddDTs1zb6uday6j4etCa9RChD8fJ1xay";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.ownerTa),
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.listTa),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.rentDestination),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.approve),
      getAccountMeta(accounts.distribution),
      getAccountMeta(accounts.wnsProgram),
      getAccountMeta(accounts.wnsDistributionProgram),
      getAccountMeta(accounts.extraMetas)
    ],
    programAddress,
    data: getCloseExpiredListingWnsInstructionDataEncoder().encode({})
  };
  return instruction;
}
function parseCloseExpiredListingWnsInstruction(instruction) {
  if (instruction.accounts.length < 16) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      owner: getNextAccount(),
      ownerTa: getNextAccount(),
      listState: getNextAccount(),
      listTa: getNextAccount(),
      mint: getNextAccount(),
      rentDestination: getNextAccount(),
      payer: getNextAccount(),
      tokenProgram: getNextAccount(),
      associatedTokenProgram: getNextAccount(),
      systemProgram: getNextAccount(),
      marketplaceProgram: getNextAccount(),
      approve: getNextAccount(),
      distribution: getNextAccount(),
      wnsProgram: getNextAccount(),
      wnsDistributionProgram: getNextAccount(),
      extraMetas: getNextAccount()
    },
    data: getCloseExpiredListingWnsInstructionDataDecoder().decode(
      instruction.data
    )
  };
}
function getDelistCompressedInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["nonce", getU64Encoder()],
      ["index", getU32Encoder()],
      ["root", fixEncoderSize(getBytesEncoder(), 32)],
      ["dataHash", fixEncoderSize(getBytesEncoder(), 32)],
      ["creatorHash", fixEncoderSize(getBytesEncoder(), 32)]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([55, 136, 205, 107, 107, 173, 4, 31])
    })
  );
}
function getDelistCompressedInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["nonce", getU64Decoder()],
    ["index", getU32Decoder()],
    ["root", fixDecoderSize(getBytesDecoder(), 32)],
    ["dataHash", fixDecoderSize(getBytesDecoder(), 32)],
    ["creatorHash", fixDecoderSize(getBytesDecoder(), 32)]
  ]);
}
function getDelistCompressedInstructionDataCodec() {
  return combineCodec(
    getDelistCompressedInstructionDataEncoder(),
    getDelistCompressedInstructionDataDecoder()
  );
}
async function getDelistCompressedInstructionAsync(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    treeAuthority: { value: input.treeAuthority ?? null, isWritable: false },
    merkleTree: { value: input.merkleTree ?? null, isWritable: true },
    logWrapper: { value: input.logWrapper ?? null, isWritable: false },
    compressionProgram: {
      value: input.compressionProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    bubblegumProgram: {
      value: input.bubblegumProgram ?? null,
      isWritable: false
    },
    listState: { value: input.listState ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: false },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.bubblegumProgram.value) {
    accounts.bubblegumProgram.value = "BGUMAp9Gq7iTEuizy4pqaxsTyUCBK68MDfK752saRPUY";
  }
  if (!accounts.treeAuthority.value) {
    accounts.treeAuthority = {
      ...accounts.treeAuthority,
      ...await resolveTreeAuthorityPda(resolverScope)
    };
  }
  if (!accounts.logWrapper.value) {
    accounts.logWrapper.value = "noopb9bkMVfRPU8AsbpTUg8AQkHtKwMYZiFUjNRtMmV";
  }
  if (!accounts.compressionProgram.value) {
    accounts.compressionProgram.value = "cmtDvXumGCrqC1Age74AVPhSRVXJMd8PJS91L8KbNCK";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.rentDestination.value) {
    accounts.rentDestination.value = expectTransactionSigner(
      accounts.owner.value
    ).address;
  }
  if (!args.nonce) {
    args.nonce = expectSome(args.index);
  }
  if (!args.proof) {
    args.proof = [];
  }
  if (!args.canopyDepth) {
    args.canopyDepth = 0;
  }
  const remainingAccounts = resolveProofPath(resolverScope);
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.treeAuthority),
      getAccountMeta(accounts.merkleTree),
      getAccountMeta(accounts.logWrapper),
      getAccountMeta(accounts.compressionProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.bubblegumProgram),
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.rentDestination),
      ...remainingAccounts
    ],
    programAddress,
    data: getDelistCompressedInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function getDelistCompressedInstruction(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    treeAuthority: { value: input.treeAuthority ?? null, isWritable: false },
    merkleTree: { value: input.merkleTree ?? null, isWritable: true },
    logWrapper: { value: input.logWrapper ?? null, isWritable: false },
    compressionProgram: {
      value: input.compressionProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    bubblegumProgram: {
      value: input.bubblegumProgram ?? null,
      isWritable: false
    },
    listState: { value: input.listState ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: false },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.bubblegumProgram.value) {
    accounts.bubblegumProgram.value = "BGUMAp9Gq7iTEuizy4pqaxsTyUCBK68MDfK752saRPUY";
  }
  if (!accounts.logWrapper.value) {
    accounts.logWrapper.value = "noopb9bkMVfRPU8AsbpTUg8AQkHtKwMYZiFUjNRtMmV";
  }
  if (!accounts.compressionProgram.value) {
    accounts.compressionProgram.value = "cmtDvXumGCrqC1Age74AVPhSRVXJMd8PJS91L8KbNCK";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.rentDestination.value) {
    accounts.rentDestination.value = expectTransactionSigner(
      accounts.owner.value
    ).address;
  }
  if (!args.nonce) {
    args.nonce = expectSome(args.index);
  }
  if (!args.proof) {
    args.proof = [];
  }
  if (!args.canopyDepth) {
    args.canopyDepth = 0;
  }
  const remainingAccounts = resolveProofPath(resolverScope);
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.treeAuthority),
      getAccountMeta(accounts.merkleTree),
      getAccountMeta(accounts.logWrapper),
      getAccountMeta(accounts.compressionProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.bubblegumProgram),
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.rentDestination),
      ...remainingAccounts
    ],
    programAddress,
    data: getDelistCompressedInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseDelistCompressedInstruction(instruction) {
  if (instruction.accounts.length < 10) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      treeAuthority: getNextAccount(),
      merkleTree: getNextAccount(),
      logWrapper: getNextAccount(),
      compressionProgram: getNextAccount(),
      systemProgram: getNextAccount(),
      bubblegumProgram: getNextAccount(),
      listState: getNextAccount(),
      owner: getNextAccount(),
      marketplaceProgram: getNextAccount(),
      rentDestination: getNextAccount()
    },
    data: getDelistCompressedInstructionDataDecoder().decode(instruction.data)
  };
}
function getDelistCoreInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([["discriminator", fixEncoderSize(getBytesEncoder(), 8)]]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([56, 24, 231, 2, 227, 19, 14, 68])
    })
  );
}
function getDelistCoreInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)]
  ]);
}
function getDelistCoreInstructionDataCodec() {
  return combineCodec(
    getDelistCoreInstructionDataEncoder(),
    getDelistCoreInstructionDataDecoder()
  );
}
function getDelistCoreInstruction(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    asset: { value: input.asset ?? null, isWritable: true },
    collection: { value: input.collection ?? null, isWritable: false },
    owner: { value: input.owner ?? null, isWritable: true },
    listState: { value: input.listState ?? null, isWritable: true },
    mplCoreProgram: { value: input.mplCoreProgram ?? null, isWritable: false },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true }
  };
  const accounts = originalAccounts;
  if (!accounts.mplCoreProgram.value) {
    accounts.mplCoreProgram.value = "CoREENxT6tW1HoK8ypY1SxRMZTcVPm7R94rH4PZNhX7d";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.asset),
      getAccountMeta(accounts.collection),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.mplCoreProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.rentDestination)
    ],
    programAddress,
    data: getDelistCoreInstructionDataEncoder().encode({})
  };
  return instruction;
}
function parseDelistCoreInstruction(instruction) {
  if (instruction.accounts.length < 8) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  const getNextOptionalAccount = () => {
    const accountMeta = getNextAccount();
    return accountMeta.address === TENSOR_MARKETPLACE_PROGRAM_ADDRESS ? void 0 : accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      asset: getNextAccount(),
      collection: getNextOptionalAccount(),
      owner: getNextAccount(),
      listState: getNextAccount(),
      mplCoreProgram: getNextAccount(),
      marketplaceProgram: getNextAccount(),
      systemProgram: getNextAccount(),
      rentDestination: getNextAccount()
    },
    data: getDelistCoreInstructionDataDecoder().decode(instruction.data)
  };
}
function getDelistLegacyInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      [
        "authorizationData",
        getOptionEncoder(getAuthorizationDataLocalEncoder())
      ]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([88, 35, 231, 184, 110, 218, 149, 23]),
      authorizationData: value.authorizationData ?? none()
    })
  );
}
function getDelistLegacyInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["authorizationData", getOptionDecoder(getAuthorizationDataLocalDecoder())]
  ]);
}
function getDelistLegacyInstructionDataCodec() {
  return combineCodec(
    getDelistLegacyInstructionDataEncoder(),
    getDelistLegacyInstructionDataDecoder()
  );
}
async function getDelistLegacyInstructionAsync(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    owner: { value: input.owner ?? null, isWritable: true },
    ownerTa: { value: input.ownerTa ?? null, isWritable: true },
    listState: { value: input.listState ?? null, isWritable: true },
    listTa: { value: input.listTa ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true },
    payer: { value: input.payer ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    metadata: { value: input.metadata ?? null, isWritable: true },
    edition: { value: input.edition ?? null, isWritable: false },
    ownerTokenRecord: {
      value: input.ownerTokenRecord ?? null,
      isWritable: true
    },
    listTokenRecord: { value: input.listTokenRecord ?? null, isWritable: true },
    authorizationRules: {
      value: input.authorizationRules ?? null,
      isWritable: false
    },
    authorizationRulesProgram: {
      value: input.authorizationRulesProgram ?? null,
      isWritable: false
    },
    tokenMetadataProgram: {
      value: input.tokenMetadataProgram ?? null,
      isWritable: false
    },
    sysvarInstructions: {
      value: input.sysvarInstructions ?? null,
      isWritable: false
    }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.ownerTa.value) {
    accounts.ownerTa = {
      ...accounts.ownerTa,
      ...await resolveOwnerAta(resolverScope)
    };
  }
  if (!accounts.listState.value) {
    accounts.listState.value = await findListStatePda({
      mint: expectAddress(accounts.mint.value)
    });
  }
  if (!accounts.listTa.value) {
    accounts.listTa = {
      ...accounts.listTa,
      ...await resolveListAta(resolverScope)
    };
  }
  if (!accounts.rentDestination.value) {
    accounts.rentDestination.value = expectTransactionSigner(
      accounts.owner.value
    ).address;
  }
  if (!accounts.payer.value) {
    accounts.payer.value = expectSome(accounts.owner.value);
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.metadata.value) {
    accounts.metadata = {
      ...accounts.metadata,
      ...await resolveMetadata(resolverScope)
    };
  }
  if (!args.tokenStandard) {
    args.tokenStandard = TokenStandard.ProgrammableNonFungible;
  }
  if (!accounts.edition.value) {
    accounts.edition = {
      ...accounts.edition,
      ...await resolveEditionFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.ownerTokenRecord.value) {
    accounts.ownerTokenRecord = {
      ...accounts.ownerTokenRecord,
      ...await resolveOwnerTokenRecordFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.listTokenRecord.value) {
    accounts.listTokenRecord = {
      ...accounts.listTokenRecord,
      ...await resolveListTokenRecordFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.authorizationRulesProgram.value) {
    accounts.authorizationRulesProgram = {
      ...accounts.authorizationRulesProgram,
      ...resolveAuthorizationRulesProgramFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.tokenMetadataProgram.value) {
    accounts.tokenMetadataProgram = {
      ...accounts.tokenMetadataProgram,
      ...resolveTokenMetadataProgramFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.sysvarInstructions.value) {
    accounts.sysvarInstructions = {
      ...accounts.sysvarInstructions,
      ...resolveSysvarInstructionsFromTokenStandard(resolverScope)
    };
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.ownerTa),
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.listTa),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.rentDestination),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.metadata),
      getAccountMeta(accounts.edition),
      getAccountMeta(accounts.ownerTokenRecord),
      getAccountMeta(accounts.listTokenRecord),
      getAccountMeta(accounts.authorizationRules),
      getAccountMeta(accounts.authorizationRulesProgram),
      getAccountMeta(accounts.tokenMetadataProgram),
      getAccountMeta(accounts.sysvarInstructions)
    ],
    programAddress,
    data: getDelistLegacyInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function getDelistLegacyInstruction(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    owner: { value: input.owner ?? null, isWritable: true },
    ownerTa: { value: input.ownerTa ?? null, isWritable: true },
    listState: { value: input.listState ?? null, isWritable: true },
    listTa: { value: input.listTa ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true },
    payer: { value: input.payer ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    metadata: { value: input.metadata ?? null, isWritable: true },
    edition: { value: input.edition ?? null, isWritable: false },
    ownerTokenRecord: {
      value: input.ownerTokenRecord ?? null,
      isWritable: true
    },
    listTokenRecord: { value: input.listTokenRecord ?? null, isWritable: true },
    authorizationRules: {
      value: input.authorizationRules ?? null,
      isWritable: false
    },
    authorizationRulesProgram: {
      value: input.authorizationRulesProgram ?? null,
      isWritable: false
    },
    tokenMetadataProgram: {
      value: input.tokenMetadataProgram ?? null,
      isWritable: false
    },
    sysvarInstructions: {
      value: input.sysvarInstructions ?? null,
      isWritable: false
    }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.rentDestination.value) {
    accounts.rentDestination.value = expectTransactionSigner(
      accounts.owner.value
    ).address;
  }
  if (!accounts.payer.value) {
    accounts.payer.value = expectSome(accounts.owner.value);
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!args.tokenStandard) {
    args.tokenStandard = TokenStandard.ProgrammableNonFungible;
  }
  if (!accounts.authorizationRulesProgram.value) {
    accounts.authorizationRulesProgram = {
      ...accounts.authorizationRulesProgram,
      ...resolveAuthorizationRulesProgramFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.tokenMetadataProgram.value) {
    accounts.tokenMetadataProgram = {
      ...accounts.tokenMetadataProgram,
      ...resolveTokenMetadataProgramFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.sysvarInstructions.value) {
    accounts.sysvarInstructions = {
      ...accounts.sysvarInstructions,
      ...resolveSysvarInstructionsFromTokenStandard(resolverScope)
    };
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.ownerTa),
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.listTa),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.rentDestination),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.metadata),
      getAccountMeta(accounts.edition),
      getAccountMeta(accounts.ownerTokenRecord),
      getAccountMeta(accounts.listTokenRecord),
      getAccountMeta(accounts.authorizationRules),
      getAccountMeta(accounts.authorizationRulesProgram),
      getAccountMeta(accounts.tokenMetadataProgram),
      getAccountMeta(accounts.sysvarInstructions)
    ],
    programAddress,
    data: getDelistLegacyInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseDelistLegacyInstruction(instruction) {
  if (instruction.accounts.length < 19) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  const getNextOptionalAccount = () => {
    const accountMeta = getNextAccount();
    return accountMeta.address === TENSOR_MARKETPLACE_PROGRAM_ADDRESS ? void 0 : accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      owner: getNextAccount(),
      ownerTa: getNextAccount(),
      listState: getNextAccount(),
      listTa: getNextAccount(),
      mint: getNextAccount(),
      rentDestination: getNextAccount(),
      payer: getNextAccount(),
      tokenProgram: getNextAccount(),
      associatedTokenProgram: getNextAccount(),
      marketplaceProgram: getNextAccount(),
      systemProgram: getNextAccount(),
      metadata: getNextAccount(),
      edition: getNextAccount(),
      ownerTokenRecord: getNextOptionalAccount(),
      listTokenRecord: getNextOptionalAccount(),
      authorizationRules: getNextOptionalAccount(),
      authorizationRulesProgram: getNextOptionalAccount(),
      tokenMetadataProgram: getNextOptionalAccount(),
      sysvarInstructions: getNextOptionalAccount()
    },
    data: getDelistLegacyInstructionDataDecoder().decode(instruction.data)
  };
}
function getDelistT22InstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([["discriminator", fixEncoderSize(getBytesEncoder(), 8)]]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([216, 72, 73, 18, 204, 82, 123, 26])
    })
  );
}
function getDelistT22InstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)]
  ]);
}
function getDelistT22InstructionDataCodec() {
  return combineCodec(
    getDelistT22InstructionDataEncoder(),
    getDelistT22InstructionDataDecoder()
  );
}
async function getDelistT22InstructionAsync(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    owner: { value: input.owner ?? null, isWritable: true },
    ownerTa: { value: input.ownerTa ?? null, isWritable: true },
    listState: { value: input.listState ?? null, isWritable: true },
    listTa: { value: input.listTa ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true },
    payer: { value: input.payer ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const resolverScope = { programAddress, accounts };
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.ownerTa.value) {
    accounts.ownerTa = {
      ...accounts.ownerTa,
      ...await resolveOwnerAta(resolverScope)
    };
  }
  if (!accounts.listState.value) {
    accounts.listState.value = await findListStatePda({
      mint: expectAddress(accounts.mint.value)
    });
  }
  if (!accounts.listTa.value) {
    accounts.listTa = {
      ...accounts.listTa,
      ...await resolveListAta(resolverScope)
    };
  }
  if (!accounts.rentDestination.value) {
    accounts.rentDestination.value = expectTransactionSigner(
      accounts.owner.value
    ).address;
  }
  if (!accounts.payer.value) {
    accounts.payer.value = expectSome(accounts.owner.value);
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.ownerTa),
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.listTa),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.rentDestination),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.systemProgram)
    ],
    programAddress,
    data: getDelistT22InstructionDataEncoder().encode({})
  };
  return instruction;
}
function getDelistT22Instruction(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    owner: { value: input.owner ?? null, isWritable: true },
    ownerTa: { value: input.ownerTa ?? null, isWritable: true },
    listState: { value: input.listState ?? null, isWritable: true },
    listTa: { value: input.listTa ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true },
    payer: { value: input.payer ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.rentDestination.value) {
    accounts.rentDestination.value = expectTransactionSigner(
      accounts.owner.value
    ).address;
  }
  if (!accounts.payer.value) {
    accounts.payer.value = expectSome(accounts.owner.value);
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.ownerTa),
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.listTa),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.rentDestination),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.systemProgram)
    ],
    programAddress,
    data: getDelistT22InstructionDataEncoder().encode({})
  };
  return instruction;
}
function parseDelistT22Instruction(instruction) {
  if (instruction.accounts.length < 11) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      owner: getNextAccount(),
      ownerTa: getNextAccount(),
      listState: getNextAccount(),
      listTa: getNextAccount(),
      mint: getNextAccount(),
      rentDestination: getNextAccount(),
      payer: getNextAccount(),
      tokenProgram: getNextAccount(),
      associatedTokenProgram: getNextAccount(),
      marketplaceProgram: getNextAccount(),
      systemProgram: getNextAccount()
    },
    data: getDelistT22InstructionDataDecoder().decode(instruction.data)
  };
}
function getDelistWnsInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([["discriminator", fixEncoderSize(getBytesEncoder(), 8)]]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([172, 171, 57, 16, 74, 158, 32, 57])
    })
  );
}
function getDelistWnsInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)]
  ]);
}
function getDelistWnsInstructionDataCodec() {
  return combineCodec(
    getDelistWnsInstructionDataEncoder(),
    getDelistWnsInstructionDataDecoder()
  );
}
async function getDelistWnsInstructionAsync(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    owner: { value: input.owner ?? null, isWritable: false },
    ownerTa: { value: input.ownerTa ?? null, isWritable: true },
    listState: { value: input.listState ?? null, isWritable: true },
    listTa: { value: input.listTa ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true },
    payer: { value: input.payer ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    approve: { value: input.approve ?? null, isWritable: true },
    distribution: { value: input.distribution ?? null, isWritable: true },
    wnsProgram: { value: input.wnsProgram ?? null, isWritable: false },
    wnsDistributionProgram: {
      value: input.wnsDistributionProgram ?? null,
      isWritable: false
    },
    extraMetas: { value: input.extraMetas ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.ownerTa.value) {
    accounts.ownerTa = {
      ...accounts.ownerTa,
      ...await resolveOwnerAta(resolverScope)
    };
  }
  if (!accounts.listState.value) {
    accounts.listState.value = await findListStatePda({
      mint: expectAddress(accounts.mint.value)
    });
  }
  if (!accounts.listTa.value) {
    accounts.listTa = {
      ...accounts.listTa,
      ...await resolveListAta(resolverScope)
    };
  }
  if (!accounts.rentDestination.value) {
    accounts.rentDestination.value = expectSome(accounts.owner.value);
  }
  if (!accounts.payer.value) {
    accounts.payer.value = expectSome(accounts.owner.value);
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.approve.value) {
    accounts.approve = {
      ...accounts.approve,
      ...await resolveWnsApprovePda(resolverScope)
    };
  }
  if (!args.paymentMint) {
    args.paymentMint = "11111111111111111111111111111111";
  }
  if (!accounts.distribution.value) {
    accounts.distribution = {
      ...accounts.distribution,
      ...await resolveWnsDistributionPda(resolverScope)
    };
  }
  if (!accounts.wnsProgram.value) {
    accounts.wnsProgram.value = "wns1gDLt8fgLcGhWi5MqAqgXpwEP1JftKE9eZnXS1HM";
  }
  if (!accounts.wnsDistributionProgram.value) {
    accounts.wnsDistributionProgram.value = "diste3nXmK7ddDTs1zb6uday6j4etCa9RChD8fJ1xay";
  }
  if (!accounts.extraMetas.value) {
    accounts.extraMetas = {
      ...accounts.extraMetas,
      ...await resolveWnsExtraAccountMetasPda(resolverScope)
    };
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.ownerTa),
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.listTa),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.rentDestination),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.approve),
      getAccountMeta(accounts.distribution),
      getAccountMeta(accounts.wnsProgram),
      getAccountMeta(accounts.wnsDistributionProgram),
      getAccountMeta(accounts.extraMetas)
    ],
    programAddress,
    data: getDelistWnsInstructionDataEncoder().encode({})
  };
  return instruction;
}
function getDelistWnsInstruction(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    owner: { value: input.owner ?? null, isWritable: false },
    ownerTa: { value: input.ownerTa ?? null, isWritable: true },
    listState: { value: input.listState ?? null, isWritable: true },
    listTa: { value: input.listTa ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true },
    payer: { value: input.payer ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    approve: { value: input.approve ?? null, isWritable: true },
    distribution: { value: input.distribution ?? null, isWritable: true },
    wnsProgram: { value: input.wnsProgram ?? null, isWritable: false },
    wnsDistributionProgram: {
      value: input.wnsDistributionProgram ?? null,
      isWritable: false
    },
    extraMetas: { value: input.extraMetas ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  ({ ...input });
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.rentDestination.value) {
    accounts.rentDestination.value = expectSome(accounts.owner.value);
  }
  if (!accounts.payer.value) {
    accounts.payer.value = expectSome(accounts.owner.value);
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.wnsProgram.value) {
    accounts.wnsProgram.value = "wns1gDLt8fgLcGhWi5MqAqgXpwEP1JftKE9eZnXS1HM";
  }
  if (!accounts.wnsDistributionProgram.value) {
    accounts.wnsDistributionProgram.value = "diste3nXmK7ddDTs1zb6uday6j4etCa9RChD8fJ1xay";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.ownerTa),
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.listTa),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.rentDestination),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.approve),
      getAccountMeta(accounts.distribution),
      getAccountMeta(accounts.wnsProgram),
      getAccountMeta(accounts.wnsDistributionProgram),
      getAccountMeta(accounts.extraMetas)
    ],
    programAddress,
    data: getDelistWnsInstructionDataEncoder().encode({})
  };
  return instruction;
}
function parseDelistWnsInstruction(instruction) {
  if (instruction.accounts.length < 16) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      owner: getNextAccount(),
      ownerTa: getNextAccount(),
      listState: getNextAccount(),
      listTa: getNextAccount(),
      mint: getNextAccount(),
      rentDestination: getNextAccount(),
      payer: getNextAccount(),
      tokenProgram: getNextAccount(),
      associatedTokenProgram: getNextAccount(),
      marketplaceProgram: getNextAccount(),
      systemProgram: getNextAccount(),
      approve: getNextAccount(),
      distribution: getNextAccount(),
      wnsProgram: getNextAccount(),
      wnsDistributionProgram: getNextAccount(),
      extraMetas: getNextAccount()
    },
    data: getDelistWnsInstructionDataDecoder().decode(instruction.data)
  };
}
function getEditInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["amount", getU64Encoder()],
      ["expireInSec", getOptionEncoder(getU64Encoder())],
      ["currency", getOptionEncoder(getAddressEncoder())],
      ["privateTaker", getOptionEncoder(getAddressEncoder())],
      ["makerBroker", getOptionEncoder(getAddressEncoder())]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([15, 183, 33, 86, 87, 28, 151, 145]),
      expireInSec: value.expireInSec ?? none(),
      currency: value.currency ?? none(),
      privateTaker: value.privateTaker ?? none(),
      makerBroker: value.makerBroker ?? none()
    })
  );
}
function getEditInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["amount", getU64Decoder()],
    ["expireInSec", getOptionDecoder(getU64Decoder())],
    ["currency", getOptionDecoder(getAddressDecoder())],
    ["privateTaker", getOptionDecoder(getAddressDecoder())],
    ["makerBroker", getOptionDecoder(getAddressDecoder())]
  ]);
}
function getEditInstructionDataCodec() {
  return combineCodec(
    getEditInstructionDataEncoder(),
    getEditInstructionDataDecoder()
  );
}
function getEditInstruction(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    listState: { value: input.listState ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: false },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.marketplaceProgram)
    ],
    programAddress,
    data: getEditInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseEditInstruction(instruction) {
  if (instruction.accounts.length < 3) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      listState: getNextAccount(),
      owner: getNextAccount(),
      marketplaceProgram: getNextAccount()
    },
    data: getEditInstructionDataDecoder().decode(instruction.data)
  };
}
function getListCompressedInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["nonce", getU64Encoder()],
      ["index", getU32Encoder()],
      ["root", fixEncoderSize(getBytesEncoder(), 32)],
      ["dataHash", fixEncoderSize(getBytesEncoder(), 32)],
      ["creatorHash", fixEncoderSize(getBytesEncoder(), 32)],
      ["amount", getU64Encoder()],
      ["expireInSec", getOptionEncoder(getU64Encoder())],
      ["currency", getOptionEncoder(getAddressEncoder())],
      ["privateTaker", getOptionEncoder(getAddressEncoder())],
      ["makerBroker", getOptionEncoder(getAddressEncoder())]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([54, 174, 193, 67, 17, 41, 132, 38]),
      expireInSec: value.expireInSec ?? none(),
      currency: value.currency ?? none(),
      privateTaker: value.privateTaker ?? none(),
      makerBroker: value.makerBroker ?? none()
    })
  );
}
function getListCompressedInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["nonce", getU64Decoder()],
    ["index", getU32Decoder()],
    ["root", fixDecoderSize(getBytesDecoder(), 32)],
    ["dataHash", fixDecoderSize(getBytesDecoder(), 32)],
    ["creatorHash", fixDecoderSize(getBytesDecoder(), 32)],
    ["amount", getU64Decoder()],
    ["expireInSec", getOptionDecoder(getU64Decoder())],
    ["currency", getOptionDecoder(getAddressDecoder())],
    ["privateTaker", getOptionDecoder(getAddressDecoder())],
    ["makerBroker", getOptionDecoder(getAddressDecoder())]
  ]);
}
function getListCompressedInstructionDataCodec() {
  return combineCodec(
    getListCompressedInstructionDataEncoder(),
    getListCompressedInstructionDataDecoder()
  );
}
async function getListCompressedInstructionAsync(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    treeAuthority: { value: input.treeAuthority ?? null, isWritable: false },
    owner: { value: input.owner ?? null, isWritable: false },
    delegate: { value: input.delegate ?? null, isWritable: false },
    merkleTree: { value: input.merkleTree ?? null, isWritable: true },
    logWrapper: { value: input.logWrapper ?? null, isWritable: false },
    compressionProgram: {
      value: input.compressionProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    bubblegumProgram: {
      value: input.bubblegumProgram ?? null,
      isWritable: false
    },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    listState: { value: input.listState ?? null, isWritable: true },
    rentPayer: { value: input.rentPayer ?? null, isWritable: true },
    cosigner: { value: input.cosigner ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.bubblegumProgram.value) {
    accounts.bubblegumProgram.value = "BGUMAp9Gq7iTEuizy4pqaxsTyUCBK68MDfK752saRPUY";
  }
  if (!accounts.treeAuthority.value) {
    accounts.treeAuthority = {
      ...accounts.treeAuthority,
      ...await resolveTreeAuthorityPda(resolverScope)
    };
  }
  if (!accounts.delegate.value) {
    accounts.delegate.value = expectSome(accounts.owner.value);
  }
  if (!accounts.logWrapper.value) {
    accounts.logWrapper.value = "noopb9bkMVfRPU8AsbpTUg8AQkHtKwMYZiFUjNRtMmV";
  }
  if (!accounts.compressionProgram.value) {
    accounts.compressionProgram.value = "cmtDvXumGCrqC1Age74AVPhSRVXJMd8PJS91L8KbNCK";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.rentPayer.value) {
    accounts.rentPayer = {
      ...accounts.rentPayer,
      ...resolveRemainingSignerWithOwnerOrDelegate(resolverScope)
    };
  }
  if (!accounts.cosigner.value) {
    accounts.cosigner.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!args.nonce) {
    args.nonce = expectSome(args.index);
  }
  if (!args.proof) {
    args.proof = [];
  }
  if (!args.canopyDepth) {
    args.canopyDepth = 0;
  }
  const remainingAccounts = resolveProofPath(resolverScope);
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.treeAuthority),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.delegate),
      getAccountMeta(accounts.merkleTree),
      getAccountMeta(accounts.logWrapper),
      getAccountMeta(accounts.compressionProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.bubblegumProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.rentPayer),
      getAccountMeta(accounts.cosigner),
      ...remainingAccounts
    ],
    programAddress,
    data: getListCompressedInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function getListCompressedInstruction(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    treeAuthority: { value: input.treeAuthority ?? null, isWritable: false },
    owner: { value: input.owner ?? null, isWritable: false },
    delegate: { value: input.delegate ?? null, isWritable: false },
    merkleTree: { value: input.merkleTree ?? null, isWritable: true },
    logWrapper: { value: input.logWrapper ?? null, isWritable: false },
    compressionProgram: {
      value: input.compressionProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    bubblegumProgram: {
      value: input.bubblegumProgram ?? null,
      isWritable: false
    },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    listState: { value: input.listState ?? null, isWritable: true },
    rentPayer: { value: input.rentPayer ?? null, isWritable: true },
    cosigner: { value: input.cosigner ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.bubblegumProgram.value) {
    accounts.bubblegumProgram.value = "BGUMAp9Gq7iTEuizy4pqaxsTyUCBK68MDfK752saRPUY";
  }
  if (!accounts.delegate.value) {
    accounts.delegate.value = expectSome(accounts.owner.value);
  }
  if (!accounts.logWrapper.value) {
    accounts.logWrapper.value = "noopb9bkMVfRPU8AsbpTUg8AQkHtKwMYZiFUjNRtMmV";
  }
  if (!accounts.compressionProgram.value) {
    accounts.compressionProgram.value = "cmtDvXumGCrqC1Age74AVPhSRVXJMd8PJS91L8KbNCK";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.rentPayer.value) {
    accounts.rentPayer = {
      ...accounts.rentPayer,
      ...resolveRemainingSignerWithOwnerOrDelegate(resolverScope)
    };
  }
  if (!accounts.cosigner.value) {
    accounts.cosigner.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!args.nonce) {
    args.nonce = expectSome(args.index);
  }
  if (!args.proof) {
    args.proof = [];
  }
  if (!args.canopyDepth) {
    args.canopyDepth = 0;
  }
  const remainingAccounts = resolveProofPath(resolverScope);
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.treeAuthority),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.delegate),
      getAccountMeta(accounts.merkleTree),
      getAccountMeta(accounts.logWrapper),
      getAccountMeta(accounts.compressionProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.bubblegumProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.rentPayer),
      getAccountMeta(accounts.cosigner),
      ...remainingAccounts
    ],
    programAddress,
    data: getListCompressedInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseListCompressedInstruction(instruction) {
  if (instruction.accounts.length < 12) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  const getNextOptionalAccount = () => {
    const accountMeta = getNextAccount();
    return accountMeta.address === TENSOR_MARKETPLACE_PROGRAM_ADDRESS ? void 0 : accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      treeAuthority: getNextAccount(),
      owner: getNextAccount(),
      delegate: getNextAccount(),
      merkleTree: getNextAccount(),
      logWrapper: getNextAccount(),
      compressionProgram: getNextAccount(),
      systemProgram: getNextAccount(),
      bubblegumProgram: getNextAccount(),
      marketplaceProgram: getNextAccount(),
      listState: getNextAccount(),
      rentPayer: getNextAccount(),
      cosigner: getNextOptionalAccount()
    },
    data: getListCompressedInstructionDataDecoder().decode(instruction.data)
  };
}
function getListCoreInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["amount", getU64Encoder()],
      ["expireInSec", getOptionEncoder(getU64Encoder())],
      ["currency", getOptionEncoder(getAddressEncoder())],
      ["privateTaker", getOptionEncoder(getAddressEncoder())],
      ["makerBroker", getOptionEncoder(getAddressEncoder())]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([173, 76, 167, 125, 118, 71, 1, 153]),
      expireInSec: value.expireInSec ?? none(),
      currency: value.currency ?? none(),
      privateTaker: value.privateTaker ?? none(),
      makerBroker: value.makerBroker ?? none()
    })
  );
}
function getListCoreInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["amount", getU64Decoder()],
    ["expireInSec", getOptionDecoder(getU64Decoder())],
    ["currency", getOptionDecoder(getAddressDecoder())],
    ["privateTaker", getOptionDecoder(getAddressDecoder())],
    ["makerBroker", getOptionDecoder(getAddressDecoder())]
  ]);
}
function getListCoreInstructionDataCodec() {
  return combineCodec(
    getListCoreInstructionDataEncoder(),
    getListCoreInstructionDataDecoder()
  );
}
function getListCoreInstruction(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    asset: { value: input.asset ?? null, isWritable: true },
    collection: { value: input.collection ?? null, isWritable: false },
    listState: { value: input.listState ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: false },
    mplCoreProgram: { value: input.mplCoreProgram ?? null, isWritable: false },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    payer: { value: input.payer ?? null, isWritable: true },
    cosigner: { value: input.cosigner ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.mplCoreProgram.value) {
    accounts.mplCoreProgram.value = "CoREENxT6tW1HoK8ypY1SxRMZTcVPm7R94rH4PZNhX7d";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.asset),
      getAccountMeta(accounts.collection),
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.mplCoreProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.cosigner)
    ],
    programAddress,
    data: getListCoreInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseListCoreInstruction(instruction) {
  if (instruction.accounts.length < 9) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  const getNextOptionalAccount = () => {
    const accountMeta = getNextAccount();
    return accountMeta.address === TENSOR_MARKETPLACE_PROGRAM_ADDRESS ? void 0 : accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      asset: getNextAccount(),
      collection: getNextOptionalAccount(),
      listState: getNextAccount(),
      owner: getNextAccount(),
      mplCoreProgram: getNextAccount(),
      marketplaceProgram: getNextAccount(),
      systemProgram: getNextAccount(),
      payer: getNextAccount(),
      cosigner: getNextOptionalAccount()
    },
    data: getListCoreInstructionDataDecoder().decode(instruction.data)
  };
}
function getListLegacyInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["amount", getU64Encoder()],
      ["expireInSec", getOptionEncoder(getU64Encoder())],
      ["currency", getOptionEncoder(getAddressEncoder())],
      ["privateTaker", getOptionEncoder(getAddressEncoder())],
      ["makerBroker", getOptionEncoder(getAddressEncoder())],
      [
        "authorizationData",
        getOptionEncoder(getAuthorizationDataLocalEncoder())
      ]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([6, 110, 255, 18, 16, 36, 8, 30]),
      expireInSec: value.expireInSec ?? none(),
      currency: value.currency ?? none(),
      privateTaker: value.privateTaker ?? none(),
      makerBroker: value.makerBroker ?? none(),
      authorizationData: value.authorizationData ?? none()
    })
  );
}
function getListLegacyInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["amount", getU64Decoder()],
    ["expireInSec", getOptionDecoder(getU64Decoder())],
    ["currency", getOptionDecoder(getAddressDecoder())],
    ["privateTaker", getOptionDecoder(getAddressDecoder())],
    ["makerBroker", getOptionDecoder(getAddressDecoder())],
    ["authorizationData", getOptionDecoder(getAuthorizationDataLocalDecoder())]
  ]);
}
function getListLegacyInstructionDataCodec() {
  return combineCodec(
    getListLegacyInstructionDataEncoder(),
    getListLegacyInstructionDataDecoder()
  );
}
async function getListLegacyInstructionAsync(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    owner: { value: input.owner ?? null, isWritable: false },
    ownerTa: { value: input.ownerTa ?? null, isWritable: true },
    listState: { value: input.listState ?? null, isWritable: true },
    listTa: { value: input.listTa ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    payer: { value: input.payer ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    metadata: { value: input.metadata ?? null, isWritable: true },
    edition: { value: input.edition ?? null, isWritable: false },
    ownerTokenRecord: {
      value: input.ownerTokenRecord ?? null,
      isWritable: true
    },
    listTokenRecord: { value: input.listTokenRecord ?? null, isWritable: true },
    authorizationRules: {
      value: input.authorizationRules ?? null,
      isWritable: false
    },
    authorizationRulesProgram: {
      value: input.authorizationRulesProgram ?? null,
      isWritable: false
    },
    tokenMetadataProgram: {
      value: input.tokenMetadataProgram ?? null,
      isWritable: false
    },
    sysvarInstructions: {
      value: input.sysvarInstructions ?? null,
      isWritable: false
    },
    cosigner: { value: input.cosigner ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.ownerTa.value) {
    accounts.ownerTa = {
      ...accounts.ownerTa,
      ...await resolveOwnerAta(resolverScope)
    };
  }
  if (!accounts.listState.value) {
    accounts.listState.value = await findListStatePda({
      mint: expectAddress(accounts.mint.value)
    });
  }
  if (!accounts.listTa.value) {
    accounts.listTa = {
      ...accounts.listTa,
      ...await resolveListAta(resolverScope)
    };
  }
  if (!accounts.payer.value) {
    accounts.payer.value = expectSome(accounts.owner.value);
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.metadata.value) {
    accounts.metadata = {
      ...accounts.metadata,
      ...await resolveMetadata(resolverScope)
    };
  }
  if (!args.tokenStandard) {
    args.tokenStandard = TokenStandard.ProgrammableNonFungible;
  }
  if (!accounts.edition.value) {
    accounts.edition = {
      ...accounts.edition,
      ...await resolveEditionFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.ownerTokenRecord.value) {
    accounts.ownerTokenRecord = {
      ...accounts.ownerTokenRecord,
      ...await resolveOwnerTokenRecordFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.listTokenRecord.value) {
    accounts.listTokenRecord = {
      ...accounts.listTokenRecord,
      ...await resolveListTokenRecordFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.authorizationRulesProgram.value) {
    accounts.authorizationRulesProgram = {
      ...accounts.authorizationRulesProgram,
      ...resolveAuthorizationRulesProgramFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.tokenMetadataProgram.value) {
    accounts.tokenMetadataProgram = {
      ...accounts.tokenMetadataProgram,
      ...resolveTokenMetadataProgramFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.sysvarInstructions.value) {
    accounts.sysvarInstructions = {
      ...accounts.sysvarInstructions,
      ...resolveSysvarInstructionsFromTokenStandard(resolverScope)
    };
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.ownerTa),
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.listTa),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.metadata),
      getAccountMeta(accounts.edition),
      getAccountMeta(accounts.ownerTokenRecord),
      getAccountMeta(accounts.listTokenRecord),
      getAccountMeta(accounts.authorizationRules),
      getAccountMeta(accounts.authorizationRulesProgram),
      getAccountMeta(accounts.tokenMetadataProgram),
      getAccountMeta(accounts.sysvarInstructions),
      getAccountMeta(accounts.cosigner)
    ],
    programAddress,
    data: getListLegacyInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function getListLegacyInstruction(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    owner: { value: input.owner ?? null, isWritable: false },
    ownerTa: { value: input.ownerTa ?? null, isWritable: true },
    listState: { value: input.listState ?? null, isWritable: true },
    listTa: { value: input.listTa ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    payer: { value: input.payer ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    metadata: { value: input.metadata ?? null, isWritable: true },
    edition: { value: input.edition ?? null, isWritable: false },
    ownerTokenRecord: {
      value: input.ownerTokenRecord ?? null,
      isWritable: true
    },
    listTokenRecord: { value: input.listTokenRecord ?? null, isWritable: true },
    authorizationRules: {
      value: input.authorizationRules ?? null,
      isWritable: false
    },
    authorizationRulesProgram: {
      value: input.authorizationRulesProgram ?? null,
      isWritable: false
    },
    tokenMetadataProgram: {
      value: input.tokenMetadataProgram ?? null,
      isWritable: false
    },
    sysvarInstructions: {
      value: input.sysvarInstructions ?? null,
      isWritable: false
    },
    cosigner: { value: input.cosigner ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.payer.value) {
    accounts.payer.value = expectSome(accounts.owner.value);
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!args.tokenStandard) {
    args.tokenStandard = TokenStandard.ProgrammableNonFungible;
  }
  if (!accounts.authorizationRulesProgram.value) {
    accounts.authorizationRulesProgram = {
      ...accounts.authorizationRulesProgram,
      ...resolveAuthorizationRulesProgramFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.tokenMetadataProgram.value) {
    accounts.tokenMetadataProgram = {
      ...accounts.tokenMetadataProgram,
      ...resolveTokenMetadataProgramFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.sysvarInstructions.value) {
    accounts.sysvarInstructions = {
      ...accounts.sysvarInstructions,
      ...resolveSysvarInstructionsFromTokenStandard(resolverScope)
    };
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.ownerTa),
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.listTa),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.metadata),
      getAccountMeta(accounts.edition),
      getAccountMeta(accounts.ownerTokenRecord),
      getAccountMeta(accounts.listTokenRecord),
      getAccountMeta(accounts.authorizationRules),
      getAccountMeta(accounts.authorizationRulesProgram),
      getAccountMeta(accounts.tokenMetadataProgram),
      getAccountMeta(accounts.sysvarInstructions),
      getAccountMeta(accounts.cosigner)
    ],
    programAddress,
    data: getListLegacyInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseListLegacyInstruction(instruction) {
  if (instruction.accounts.length < 19) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  const getNextOptionalAccount = () => {
    const accountMeta = getNextAccount();
    return accountMeta.address === TENSOR_MARKETPLACE_PROGRAM_ADDRESS ? void 0 : accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      owner: getNextAccount(),
      ownerTa: getNextAccount(),
      listState: getNextAccount(),
      listTa: getNextAccount(),
      mint: getNextAccount(),
      payer: getNextAccount(),
      tokenProgram: getNextAccount(),
      associatedTokenProgram: getNextAccount(),
      marketplaceProgram: getNextAccount(),
      systemProgram: getNextAccount(),
      metadata: getNextAccount(),
      edition: getNextAccount(),
      ownerTokenRecord: getNextOptionalAccount(),
      listTokenRecord: getNextOptionalAccount(),
      authorizationRules: getNextOptionalAccount(),
      authorizationRulesProgram: getNextOptionalAccount(),
      tokenMetadataProgram: getNextOptionalAccount(),
      sysvarInstructions: getNextOptionalAccount(),
      cosigner: getNextOptionalAccount()
    },
    data: getListLegacyInstructionDataDecoder().decode(instruction.data)
  };
}
function getListT22InstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["amount", getU64Encoder()],
      ["expireInSec", getOptionEncoder(getU64Encoder())],
      ["currency", getOptionEncoder(getAddressEncoder())],
      ["privateTaker", getOptionEncoder(getAddressEncoder())],
      ["makerBroker", getOptionEncoder(getAddressEncoder())]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([9, 117, 93, 230, 221, 4, 199, 212]),
      expireInSec: value.expireInSec ?? none(),
      currency: value.currency ?? none(),
      privateTaker: value.privateTaker ?? none(),
      makerBroker: value.makerBroker ?? none()
    })
  );
}
function getListT22InstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["amount", getU64Decoder()],
    ["expireInSec", getOptionDecoder(getU64Decoder())],
    ["currency", getOptionDecoder(getAddressDecoder())],
    ["privateTaker", getOptionDecoder(getAddressDecoder())],
    ["makerBroker", getOptionDecoder(getAddressDecoder())]
  ]);
}
function getListT22InstructionDataCodec() {
  return combineCodec(
    getListT22InstructionDataEncoder(),
    getListT22InstructionDataDecoder()
  );
}
async function getListT22InstructionAsync(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    owner: { value: input.owner ?? null, isWritable: false },
    ownerTa: { value: input.ownerTa ?? null, isWritable: true },
    listState: { value: input.listState ?? null, isWritable: true },
    listTa: { value: input.listTa ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    payer: { value: input.payer ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    cosigner: { value: input.cosigner ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.ownerTa.value) {
    accounts.ownerTa = {
      ...accounts.ownerTa,
      ...await resolveOwnerAta(resolverScope)
    };
  }
  if (!accounts.listState.value) {
    accounts.listState.value = await findListStatePda({
      mint: expectAddress(accounts.mint.value)
    });
  }
  if (!accounts.listTa.value) {
    accounts.listTa = {
      ...accounts.listTa,
      ...await resolveListAta(resolverScope)
    };
  }
  if (!accounts.payer.value) {
    accounts.payer.value = expectSome(accounts.owner.value);
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.cosigner.value) {
    accounts.cosigner.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.ownerTa),
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.listTa),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.cosigner)
    ],
    programAddress,
    data: getListT22InstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function getListT22Instruction(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    owner: { value: input.owner ?? null, isWritable: false },
    ownerTa: { value: input.ownerTa ?? null, isWritable: true },
    listState: { value: input.listState ?? null, isWritable: true },
    listTa: { value: input.listTa ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    payer: { value: input.payer ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    cosigner: { value: input.cosigner ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.payer.value) {
    accounts.payer.value = expectSome(accounts.owner.value);
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.cosigner.value) {
    accounts.cosigner.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.ownerTa),
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.listTa),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.cosigner)
    ],
    programAddress,
    data: getListT22InstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseListT22Instruction(instruction) {
  if (instruction.accounts.length < 11) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  const getNextOptionalAccount = () => {
    const accountMeta = getNextAccount();
    return accountMeta.address === TENSOR_MARKETPLACE_PROGRAM_ADDRESS ? void 0 : accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      owner: getNextAccount(),
      ownerTa: getNextAccount(),
      listState: getNextAccount(),
      listTa: getNextAccount(),
      mint: getNextAccount(),
      payer: getNextAccount(),
      tokenProgram: getNextAccount(),
      associatedTokenProgram: getNextAccount(),
      marketplaceProgram: getNextAccount(),
      systemProgram: getNextAccount(),
      cosigner: getNextOptionalAccount()
    },
    data: getListT22InstructionDataDecoder().decode(instruction.data)
  };
}
function getListWnsInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["amount", getU64Encoder()],
      ["expireInSec", getOptionEncoder(getU64Encoder())],
      ["currency", getOptionEncoder(getAddressEncoder())],
      ["privateTaker", getOptionEncoder(getAddressEncoder())],
      ["makerBroker", getOptionEncoder(getAddressEncoder())]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([23, 202, 102, 138, 255, 190, 39, 196]),
      expireInSec: value.expireInSec ?? none(),
      currency: value.currency ?? none(),
      privateTaker: value.privateTaker ?? none(),
      makerBroker: value.makerBroker ?? none()
    })
  );
}
function getListWnsInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["amount", getU64Decoder()],
    ["expireInSec", getOptionDecoder(getU64Decoder())],
    ["currency", getOptionDecoder(getAddressDecoder())],
    ["privateTaker", getOptionDecoder(getAddressDecoder())],
    ["makerBroker", getOptionDecoder(getAddressDecoder())]
  ]);
}
function getListWnsInstructionDataCodec() {
  return combineCodec(
    getListWnsInstructionDataEncoder(),
    getListWnsInstructionDataDecoder()
  );
}
async function getListWnsInstructionAsync(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    owner: { value: input.owner ?? null, isWritable: false },
    ownerTa: { value: input.ownerTa ?? null, isWritable: true },
    listState: { value: input.listState ?? null, isWritable: true },
    listTa: { value: input.listTa ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    payer: { value: input.payer ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    approve: { value: input.approve ?? null, isWritable: true },
    distribution: { value: input.distribution ?? null, isWritable: true },
    wnsProgram: { value: input.wnsProgram ?? null, isWritable: false },
    wnsDistributionProgram: {
      value: input.wnsDistributionProgram ?? null,
      isWritable: false
    },
    extraMetas: { value: input.extraMetas ?? null, isWritable: false },
    cosigner: { value: input.cosigner ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.ownerTa.value) {
    accounts.ownerTa = {
      ...accounts.ownerTa,
      ...await resolveOwnerAta(resolverScope)
    };
  }
  if (!accounts.listState.value) {
    accounts.listState.value = await findListStatePda({
      mint: expectAddress(accounts.mint.value)
    });
  }
  if (!accounts.listTa.value) {
    accounts.listTa = {
      ...accounts.listTa,
      ...await resolveListAta(resolverScope)
    };
  }
  if (!accounts.payer.value) {
    accounts.payer.value = expectSome(accounts.owner.value);
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.approve.value) {
    accounts.approve = {
      ...accounts.approve,
      ...await resolveWnsApprovePda(resolverScope)
    };
  }
  if (!args.paymentMint) {
    args.paymentMint = "11111111111111111111111111111111";
  }
  if (!accounts.distribution.value) {
    accounts.distribution = {
      ...accounts.distribution,
      ...await resolveWnsDistributionPda(resolverScope)
    };
  }
  if (!accounts.wnsProgram.value) {
    accounts.wnsProgram.value = "wns1gDLt8fgLcGhWi5MqAqgXpwEP1JftKE9eZnXS1HM";
  }
  if (!accounts.wnsDistributionProgram.value) {
    accounts.wnsDistributionProgram.value = "diste3nXmK7ddDTs1zb6uday6j4etCa9RChD8fJ1xay";
  }
  if (!accounts.extraMetas.value) {
    accounts.extraMetas = {
      ...accounts.extraMetas,
      ...await resolveWnsExtraAccountMetasPda(resolverScope)
    };
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.ownerTa),
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.listTa),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.approve),
      getAccountMeta(accounts.distribution),
      getAccountMeta(accounts.wnsProgram),
      getAccountMeta(accounts.wnsDistributionProgram),
      getAccountMeta(accounts.extraMetas),
      getAccountMeta(accounts.cosigner)
    ],
    programAddress,
    data: getListWnsInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function getListWnsInstruction(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    owner: { value: input.owner ?? null, isWritable: false },
    ownerTa: { value: input.ownerTa ?? null, isWritable: true },
    listState: { value: input.listState ?? null, isWritable: true },
    listTa: { value: input.listTa ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    payer: { value: input.payer ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    approve: { value: input.approve ?? null, isWritable: true },
    distribution: { value: input.distribution ?? null, isWritable: true },
    wnsProgram: { value: input.wnsProgram ?? null, isWritable: false },
    wnsDistributionProgram: {
      value: input.wnsDistributionProgram ?? null,
      isWritable: false
    },
    extraMetas: { value: input.extraMetas ?? null, isWritable: false },
    cosigner: { value: input.cosigner ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.payer.value) {
    accounts.payer.value = expectSome(accounts.owner.value);
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!args.paymentMint) {
    args.paymentMint = "11111111111111111111111111111111";
  }
  if (!accounts.wnsProgram.value) {
    accounts.wnsProgram.value = "wns1gDLt8fgLcGhWi5MqAqgXpwEP1JftKE9eZnXS1HM";
  }
  if (!accounts.wnsDistributionProgram.value) {
    accounts.wnsDistributionProgram.value = "diste3nXmK7ddDTs1zb6uday6j4etCa9RChD8fJ1xay";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.ownerTa),
      getAccountMeta(accounts.listState),
      getAccountMeta(accounts.listTa),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.approve),
      getAccountMeta(accounts.distribution),
      getAccountMeta(accounts.wnsProgram),
      getAccountMeta(accounts.wnsDistributionProgram),
      getAccountMeta(accounts.extraMetas),
      getAccountMeta(accounts.cosigner)
    ],
    programAddress,
    data: getListWnsInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseListWnsInstruction(instruction) {
  if (instruction.accounts.length < 16) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  const getNextOptionalAccount = () => {
    const accountMeta = getNextAccount();
    return accountMeta.address === TENSOR_MARKETPLACE_PROGRAM_ADDRESS ? void 0 : accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      owner: getNextAccount(),
      ownerTa: getNextAccount(),
      listState: getNextAccount(),
      listTa: getNextAccount(),
      mint: getNextAccount(),
      payer: getNextAccount(),
      tokenProgram: getNextAccount(),
      associatedTokenProgram: getNextAccount(),
      marketplaceProgram: getNextAccount(),
      systemProgram: getNextAccount(),
      approve: getNextAccount(),
      distribution: getNextAccount(),
      wnsProgram: getNextAccount(),
      wnsDistributionProgram: getNextAccount(),
      extraMetas: getNextAccount(),
      cosigner: getNextOptionalAccount()
    },
    data: getListWnsInstructionDataDecoder().decode(instruction.data)
  };
}
function getTakeBidCompressedFullMetaInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["nonce", getU64Encoder()],
      ["index", getU32Encoder()],
      ["root", fixEncoderSize(getBytesEncoder(), 32)],
      ["name", addEncoderSizePrefix(getUtf8Encoder(), getU32Encoder())],
      ["symbol", addEncoderSizePrefix(getUtf8Encoder(), getU32Encoder())],
      ["uri", addEncoderSizePrefix(getUtf8Encoder(), getU32Encoder())],
      ["sellerFeeBasisPoints", getU16Encoder()],
      ["primarySaleHappened", getBooleanEncoder()],
      ["isMutable", getBooleanEncoder()],
      ["editionNonce", getOptionEncoder(getU8Encoder())],
      ["tokenStandard", getOptionEncoder(getTTokenStandardEncoder())],
      ["collection", getOptionEncoder(getTCollectionEncoder())],
      ["uses", getOptionEncoder(getTUsesEncoder())],
      ["tokenProgramVersion", getTTokenProgramVersionEncoder()],
      [
        "creatorShares",
        addEncoderSizePrefix(getBytesEncoder(), getU32Encoder())
      ],
      ["creatorVerified", getArrayEncoder(getBooleanEncoder())],
      ["minAmount", getU64Encoder()],
      ["optionalRoyaltyPct", getOptionEncoder(getU16Encoder())]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([242, 194, 203, 225, 234, 53, 10, 96]),
      optionalRoyaltyPct: value.optionalRoyaltyPct ?? 100
    })
  );
}
function getTakeBidCompressedFullMetaInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["nonce", getU64Decoder()],
    ["index", getU32Decoder()],
    ["root", fixDecoderSize(getBytesDecoder(), 32)],
    ["name", addDecoderSizePrefix(getUtf8Decoder(), getU32Decoder())],
    ["symbol", addDecoderSizePrefix(getUtf8Decoder(), getU32Decoder())],
    ["uri", addDecoderSizePrefix(getUtf8Decoder(), getU32Decoder())],
    ["sellerFeeBasisPoints", getU16Decoder()],
    ["primarySaleHappened", getBooleanDecoder()],
    ["isMutable", getBooleanDecoder()],
    ["editionNonce", getOptionDecoder(getU8Decoder())],
    ["tokenStandard", getOptionDecoder(getTTokenStandardDecoder())],
    ["collection", getOptionDecoder(getTCollectionDecoder())],
    ["uses", getOptionDecoder(getTUsesDecoder())],
    ["tokenProgramVersion", getTTokenProgramVersionDecoder()],
    ["creatorShares", addDecoderSizePrefix(getBytesDecoder(), getU32Decoder())],
    ["creatorVerified", getArrayDecoder(getBooleanDecoder())],
    ["minAmount", getU64Decoder()],
    ["optionalRoyaltyPct", getOptionDecoder(getU16Decoder())]
  ]);
}
function getTakeBidCompressedFullMetaInstructionDataCodec() {
  return combineCodec(
    getTakeBidCompressedFullMetaInstructionDataEncoder(),
    getTakeBidCompressedFullMetaInstructionDataDecoder()
  );
}
async function getTakeBidCompressedFullMetaInstructionAsync(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    feeVault: { value: input.feeVault ?? null, isWritable: true },
    treeAuthority: { value: input.treeAuthority ?? null, isWritable: false },
    seller: { value: input.seller ?? null, isWritable: true },
    delegate: { value: input.delegate ?? null, isWritable: false },
    merkleTree: { value: input.merkleTree ?? null, isWritable: true },
    logWrapper: { value: input.logWrapper ?? null, isWritable: false },
    compressionProgram: {
      value: input.compressionProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    bubblegumProgram: {
      value: input.bubblegumProgram ?? null,
      isWritable: false
    },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    tensorswapProgram: {
      value: input.tensorswapProgram ?? null,
      isWritable: false
    },
    bidState: { value: input.bidState ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: true },
    takerBroker: { value: input.takerBroker ?? null, isWritable: true },
    makerBroker: { value: input.makerBroker ?? null, isWritable: true },
    marginAccount: { value: input.marginAccount ?? null, isWritable: true },
    whitelist: { value: input.whitelist ?? null, isWritable: false },
    cosigner: { value: input.cosigner ?? null, isWritable: false },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.feeVault.value) {
    accounts.feeVault = {
      ...accounts.feeVault,
      ...await resolveFeeVaultPdaFromBidState(resolverScope)
    };
  }
  if (!accounts.bubblegumProgram.value) {
    accounts.bubblegumProgram.value = "BGUMAp9Gq7iTEuizy4pqaxsTyUCBK68MDfK752saRPUY";
  }
  if (!accounts.treeAuthority.value) {
    accounts.treeAuthority = {
      ...accounts.treeAuthority,
      ...await resolveTreeAuthorityPda(resolverScope)
    };
  }
  if (!accounts.delegate.value) {
    accounts.delegate.value = expectSome(accounts.seller.value);
  }
  if (!accounts.logWrapper.value) {
    accounts.logWrapper.value = "noopb9bkMVfRPU8AsbpTUg8AQkHtKwMYZiFUjNRtMmV";
  }
  if (!accounts.compressionProgram.value) {
    accounts.compressionProgram.value = "cmtDvXumGCrqC1Age74AVPhSRVXJMd8PJS91L8KbNCK";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.tensorswapProgram.value) {
    accounts.tensorswapProgram.value = "TSWAPaqyCSx2KABk68Shruf4rp7CxcNi8hAsbdwmHbN";
  }
  if (!accounts.marginAccount.value) {
    accounts.marginAccount.value = expectSome(accounts.tensorswapProgram.value);
  }
  if (!accounts.rentDestination.value) {
    accounts.rentDestination.value = expectSome(accounts.owner.value);
  }
  if (!args.nonce) {
    args.nonce = expectSome(args.index);
  }
  if (!args.creators) {
    args.creators = [];
  }
  if (!args.proof) {
    args.proof = [];
  }
  if (!args.canopyDepth) {
    args.canopyDepth = 0;
  }
  const remainingAccounts = [
    ...resolveCreatorPath(resolverScope),
    ...resolveProofPath(resolverScope)
  ];
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.feeVault),
      getAccountMeta(accounts.treeAuthority),
      getAccountMeta(accounts.seller),
      getAccountMeta(accounts.delegate),
      getAccountMeta(accounts.merkleTree),
      getAccountMeta(accounts.logWrapper),
      getAccountMeta(accounts.compressionProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.bubblegumProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.tensorswapProgram),
      getAccountMeta(accounts.bidState),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.takerBroker),
      getAccountMeta(accounts.makerBroker),
      getAccountMeta(accounts.marginAccount),
      getAccountMeta(accounts.whitelist),
      getAccountMeta(accounts.cosigner),
      getAccountMeta(accounts.rentDestination),
      ...remainingAccounts
    ],
    programAddress,
    data: getTakeBidCompressedFullMetaInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function getTakeBidCompressedFullMetaInstruction(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    feeVault: { value: input.feeVault ?? null, isWritable: true },
    treeAuthority: { value: input.treeAuthority ?? null, isWritable: false },
    seller: { value: input.seller ?? null, isWritable: true },
    delegate: { value: input.delegate ?? null, isWritable: false },
    merkleTree: { value: input.merkleTree ?? null, isWritable: true },
    logWrapper: { value: input.logWrapper ?? null, isWritable: false },
    compressionProgram: {
      value: input.compressionProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    bubblegumProgram: {
      value: input.bubblegumProgram ?? null,
      isWritable: false
    },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    tensorswapProgram: {
      value: input.tensorswapProgram ?? null,
      isWritable: false
    },
    bidState: { value: input.bidState ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: true },
    takerBroker: { value: input.takerBroker ?? null, isWritable: true },
    makerBroker: { value: input.makerBroker ?? null, isWritable: true },
    marginAccount: { value: input.marginAccount ?? null, isWritable: true },
    whitelist: { value: input.whitelist ?? null, isWritable: false },
    cosigner: { value: input.cosigner ?? null, isWritable: false },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.bubblegumProgram.value) {
    accounts.bubblegumProgram.value = "BGUMAp9Gq7iTEuizy4pqaxsTyUCBK68MDfK752saRPUY";
  }
  if (!accounts.delegate.value) {
    accounts.delegate.value = expectSome(accounts.seller.value);
  }
  if (!accounts.logWrapper.value) {
    accounts.logWrapper.value = "noopb9bkMVfRPU8AsbpTUg8AQkHtKwMYZiFUjNRtMmV";
  }
  if (!accounts.compressionProgram.value) {
    accounts.compressionProgram.value = "cmtDvXumGCrqC1Age74AVPhSRVXJMd8PJS91L8KbNCK";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.tensorswapProgram.value) {
    accounts.tensorswapProgram.value = "TSWAPaqyCSx2KABk68Shruf4rp7CxcNi8hAsbdwmHbN";
  }
  if (!accounts.marginAccount.value) {
    accounts.marginAccount.value = expectSome(accounts.tensorswapProgram.value);
  }
  if (!accounts.rentDestination.value) {
    accounts.rentDestination.value = expectSome(accounts.owner.value);
  }
  if (!args.nonce) {
    args.nonce = expectSome(args.index);
  }
  if (!args.creators) {
    args.creators = [];
  }
  if (!args.proof) {
    args.proof = [];
  }
  if (!args.canopyDepth) {
    args.canopyDepth = 0;
  }
  const remainingAccounts = [
    ...resolveCreatorPath(resolverScope),
    ...resolveProofPath(resolverScope)
  ];
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.feeVault),
      getAccountMeta(accounts.treeAuthority),
      getAccountMeta(accounts.seller),
      getAccountMeta(accounts.delegate),
      getAccountMeta(accounts.merkleTree),
      getAccountMeta(accounts.logWrapper),
      getAccountMeta(accounts.compressionProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.bubblegumProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.tensorswapProgram),
      getAccountMeta(accounts.bidState),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.takerBroker),
      getAccountMeta(accounts.makerBroker),
      getAccountMeta(accounts.marginAccount),
      getAccountMeta(accounts.whitelist),
      getAccountMeta(accounts.cosigner),
      getAccountMeta(accounts.rentDestination),
      ...remainingAccounts
    ],
    programAddress,
    data: getTakeBidCompressedFullMetaInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseTakeBidCompressedFullMetaInstruction(instruction) {
  if (instruction.accounts.length < 19) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  const getNextOptionalAccount = () => {
    const accountMeta = getNextAccount();
    return accountMeta.address === TENSOR_MARKETPLACE_PROGRAM_ADDRESS ? void 0 : accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      feeVault: getNextAccount(),
      treeAuthority: getNextAccount(),
      seller: getNextAccount(),
      delegate: getNextAccount(),
      merkleTree: getNextAccount(),
      logWrapper: getNextAccount(),
      compressionProgram: getNextAccount(),
      systemProgram: getNextAccount(),
      bubblegumProgram: getNextAccount(),
      marketplaceProgram: getNextAccount(),
      tensorswapProgram: getNextAccount(),
      bidState: getNextAccount(),
      owner: getNextAccount(),
      takerBroker: getNextOptionalAccount(),
      makerBroker: getNextOptionalAccount(),
      marginAccount: getNextAccount(),
      whitelist: getNextAccount(),
      cosigner: getNextOptionalAccount(),
      rentDestination: getNextAccount()
    },
    data: getTakeBidCompressedFullMetaInstructionDataDecoder().decode(
      instruction.data
    )
  };
}
function getTakeBidCompressedMetaHashInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["nonce", getU64Encoder()],
      ["index", getU32Encoder()],
      ["root", fixEncoderSize(getBytesEncoder(), 32)],
      ["metaHash", fixEncoderSize(getBytesEncoder(), 32)],
      [
        "creatorShares",
        addEncoderSizePrefix(getBytesEncoder(), getU32Encoder())
      ],
      ["creatorVerified", getArrayEncoder(getBooleanEncoder())],
      ["sellerFeeBasisPoints", getU16Encoder()],
      ["minAmount", getU64Encoder()],
      ["optionalRoyaltyPct", getOptionEncoder(getU16Encoder())]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([85, 227, 202, 70, 45, 215, 10, 193]),
      optionalRoyaltyPct: value.optionalRoyaltyPct ?? 100
    })
  );
}
function getTakeBidCompressedMetaHashInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["nonce", getU64Decoder()],
    ["index", getU32Decoder()],
    ["root", fixDecoderSize(getBytesDecoder(), 32)],
    ["metaHash", fixDecoderSize(getBytesDecoder(), 32)],
    ["creatorShares", addDecoderSizePrefix(getBytesDecoder(), getU32Decoder())],
    ["creatorVerified", getArrayDecoder(getBooleanDecoder())],
    ["sellerFeeBasisPoints", getU16Decoder()],
    ["minAmount", getU64Decoder()],
    ["optionalRoyaltyPct", getOptionDecoder(getU16Decoder())]
  ]);
}
function getTakeBidCompressedMetaHashInstructionDataCodec() {
  return combineCodec(
    getTakeBidCompressedMetaHashInstructionDataEncoder(),
    getTakeBidCompressedMetaHashInstructionDataDecoder()
  );
}
async function getTakeBidCompressedMetaHashInstructionAsync(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    feeVault: { value: input.feeVault ?? null, isWritable: true },
    treeAuthority: { value: input.treeAuthority ?? null, isWritable: false },
    seller: { value: input.seller ?? null, isWritable: true },
    delegate: { value: input.delegate ?? null, isWritable: false },
    merkleTree: { value: input.merkleTree ?? null, isWritable: true },
    logWrapper: { value: input.logWrapper ?? null, isWritable: false },
    compressionProgram: {
      value: input.compressionProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    bubblegumProgram: {
      value: input.bubblegumProgram ?? null,
      isWritable: false
    },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    tensorswapProgram: {
      value: input.tensorswapProgram ?? null,
      isWritable: false
    },
    bidState: { value: input.bidState ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: true },
    takerBroker: { value: input.takerBroker ?? null, isWritable: true },
    makerBroker: { value: input.makerBroker ?? null, isWritable: true },
    marginAccount: { value: input.marginAccount ?? null, isWritable: true },
    whitelist: { value: input.whitelist ?? null, isWritable: false },
    cosigner: { value: input.cosigner ?? null, isWritable: false },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.feeVault.value) {
    accounts.feeVault = {
      ...accounts.feeVault,
      ...await resolveFeeVaultPdaFromBidState(resolverScope)
    };
  }
  if (!accounts.bubblegumProgram.value) {
    accounts.bubblegumProgram.value = "BGUMAp9Gq7iTEuizy4pqaxsTyUCBK68MDfK752saRPUY";
  }
  if (!accounts.treeAuthority.value) {
    accounts.treeAuthority = {
      ...accounts.treeAuthority,
      ...await resolveTreeAuthorityPda(resolverScope)
    };
  }
  if (!accounts.delegate.value) {
    accounts.delegate.value = expectSome(accounts.seller.value);
  }
  if (!accounts.logWrapper.value) {
    accounts.logWrapper.value = "noopb9bkMVfRPU8AsbpTUg8AQkHtKwMYZiFUjNRtMmV";
  }
  if (!accounts.compressionProgram.value) {
    accounts.compressionProgram.value = "cmtDvXumGCrqC1Age74AVPhSRVXJMd8PJS91L8KbNCK";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.tensorswapProgram.value) {
    accounts.tensorswapProgram.value = "TSWAPaqyCSx2KABk68Shruf4rp7CxcNi8hAsbdwmHbN";
  }
  if (!accounts.marginAccount.value) {
    accounts.marginAccount.value = expectSome(accounts.tensorswapProgram.value);
  }
  if (!accounts.rentDestination.value) {
    accounts.rentDestination.value = expectSome(accounts.owner.value);
  }
  if (!args.nonce) {
    args.nonce = expectSome(args.index);
  }
  if (!args.creators) {
    args.creators = [];
  }
  if (!args.proof) {
    args.proof = [];
  }
  if (!args.canopyDepth) {
    args.canopyDepth = 0;
  }
  const remainingAccounts = [
    ...resolveCreatorPath(resolverScope),
    ...resolveProofPath(resolverScope)
  ];
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.feeVault),
      getAccountMeta(accounts.treeAuthority),
      getAccountMeta(accounts.seller),
      getAccountMeta(accounts.delegate),
      getAccountMeta(accounts.merkleTree),
      getAccountMeta(accounts.logWrapper),
      getAccountMeta(accounts.compressionProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.bubblegumProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.tensorswapProgram),
      getAccountMeta(accounts.bidState),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.takerBroker),
      getAccountMeta(accounts.makerBroker),
      getAccountMeta(accounts.marginAccount),
      getAccountMeta(accounts.whitelist),
      getAccountMeta(accounts.cosigner),
      getAccountMeta(accounts.rentDestination),
      ...remainingAccounts
    ],
    programAddress,
    data: getTakeBidCompressedMetaHashInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function getTakeBidCompressedMetaHashInstruction(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    feeVault: { value: input.feeVault ?? null, isWritable: true },
    treeAuthority: { value: input.treeAuthority ?? null, isWritable: false },
    seller: { value: input.seller ?? null, isWritable: true },
    delegate: { value: input.delegate ?? null, isWritable: false },
    merkleTree: { value: input.merkleTree ?? null, isWritable: true },
    logWrapper: { value: input.logWrapper ?? null, isWritable: false },
    compressionProgram: {
      value: input.compressionProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    bubblegumProgram: {
      value: input.bubblegumProgram ?? null,
      isWritable: false
    },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    tensorswapProgram: {
      value: input.tensorswapProgram ?? null,
      isWritable: false
    },
    bidState: { value: input.bidState ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: true },
    takerBroker: { value: input.takerBroker ?? null, isWritable: true },
    makerBroker: { value: input.makerBroker ?? null, isWritable: true },
    marginAccount: { value: input.marginAccount ?? null, isWritable: true },
    whitelist: { value: input.whitelist ?? null, isWritable: false },
    cosigner: { value: input.cosigner ?? null, isWritable: false },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.bubblegumProgram.value) {
    accounts.bubblegumProgram.value = "BGUMAp9Gq7iTEuizy4pqaxsTyUCBK68MDfK752saRPUY";
  }
  if (!accounts.delegate.value) {
    accounts.delegate.value = expectSome(accounts.seller.value);
  }
  if (!accounts.logWrapper.value) {
    accounts.logWrapper.value = "noopb9bkMVfRPU8AsbpTUg8AQkHtKwMYZiFUjNRtMmV";
  }
  if (!accounts.compressionProgram.value) {
    accounts.compressionProgram.value = "cmtDvXumGCrqC1Age74AVPhSRVXJMd8PJS91L8KbNCK";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.tensorswapProgram.value) {
    accounts.tensorswapProgram.value = "TSWAPaqyCSx2KABk68Shruf4rp7CxcNi8hAsbdwmHbN";
  }
  if (!accounts.marginAccount.value) {
    accounts.marginAccount.value = expectSome(accounts.tensorswapProgram.value);
  }
  if (!accounts.rentDestination.value) {
    accounts.rentDestination.value = expectSome(accounts.owner.value);
  }
  if (!args.nonce) {
    args.nonce = expectSome(args.index);
  }
  if (!args.creators) {
    args.creators = [];
  }
  if (!args.proof) {
    args.proof = [];
  }
  if (!args.canopyDepth) {
    args.canopyDepth = 0;
  }
  const remainingAccounts = [
    ...resolveCreatorPath(resolverScope),
    ...resolveProofPath(resolverScope)
  ];
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.feeVault),
      getAccountMeta(accounts.treeAuthority),
      getAccountMeta(accounts.seller),
      getAccountMeta(accounts.delegate),
      getAccountMeta(accounts.merkleTree),
      getAccountMeta(accounts.logWrapper),
      getAccountMeta(accounts.compressionProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.bubblegumProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.tensorswapProgram),
      getAccountMeta(accounts.bidState),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.takerBroker),
      getAccountMeta(accounts.makerBroker),
      getAccountMeta(accounts.marginAccount),
      getAccountMeta(accounts.whitelist),
      getAccountMeta(accounts.cosigner),
      getAccountMeta(accounts.rentDestination),
      ...remainingAccounts
    ],
    programAddress,
    data: getTakeBidCompressedMetaHashInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseTakeBidCompressedMetaHashInstruction(instruction) {
  if (instruction.accounts.length < 19) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  const getNextOptionalAccount = () => {
    const accountMeta = getNextAccount();
    return accountMeta.address === TENSOR_MARKETPLACE_PROGRAM_ADDRESS ? void 0 : accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      feeVault: getNextAccount(),
      treeAuthority: getNextAccount(),
      seller: getNextAccount(),
      delegate: getNextAccount(),
      merkleTree: getNextAccount(),
      logWrapper: getNextAccount(),
      compressionProgram: getNextAccount(),
      systemProgram: getNextAccount(),
      bubblegumProgram: getNextAccount(),
      marketplaceProgram: getNextAccount(),
      tensorswapProgram: getNextAccount(),
      bidState: getNextAccount(),
      owner: getNextAccount(),
      takerBroker: getNextOptionalAccount(),
      makerBroker: getNextOptionalAccount(),
      marginAccount: getNextAccount(),
      whitelist: getNextAccount(),
      cosigner: getNextOptionalAccount(),
      rentDestination: getNextAccount()
    },
    data: getTakeBidCompressedMetaHashInstructionDataDecoder().decode(
      instruction.data
    )
  };
}
function getTakeBidCoreInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["minAmount", getU64Encoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([250, 41, 248, 20, 61, 161, 27, 141])
    })
  );
}
function getTakeBidCoreInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["minAmount", getU64Decoder()]
  ]);
}
function getTakeBidCoreInstructionDataCodec() {
  return combineCodec(
    getTakeBidCoreInstructionDataEncoder(),
    getTakeBidCoreInstructionDataDecoder()
  );
}
function getTakeBidCoreInstruction(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    feeVault: { value: input.feeVault ?? null, isWritable: true },
    seller: { value: input.seller ?? null, isWritable: true },
    bidState: { value: input.bidState ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: true },
    takerBroker: { value: input.takerBroker ?? null, isWritable: true },
    makerBroker: { value: input.makerBroker ?? null, isWritable: true },
    marginAccount: { value: input.marginAccount ?? null, isWritable: true },
    whitelist: { value: input.whitelist ?? null, isWritable: false },
    asset: { value: input.asset ?? null, isWritable: true },
    collection: { value: input.collection ?? null, isWritable: false },
    mplCoreProgram: { value: input.mplCoreProgram ?? null, isWritable: false },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    escrowProgram: { value: input.escrowProgram ?? null, isWritable: false },
    cosigner: { value: input.cosigner ?? null, isWritable: false },
    mintProof: { value: input.mintProof ?? null, isWritable: false },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.mplCoreProgram.value) {
    accounts.mplCoreProgram.value = "CoREENxT6tW1HoK8ypY1SxRMZTcVPm7R94rH4PZNhX7d";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.escrowProgram.value) {
    accounts.escrowProgram.value = "TSWAPaqyCSx2KABk68Shruf4rp7CxcNi8hAsbdwmHbN";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.feeVault),
      getAccountMeta(accounts.seller),
      getAccountMeta(accounts.bidState),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.takerBroker),
      getAccountMeta(accounts.makerBroker),
      getAccountMeta(accounts.marginAccount),
      getAccountMeta(accounts.whitelist),
      getAccountMeta(accounts.asset),
      getAccountMeta(accounts.collection),
      getAccountMeta(accounts.mplCoreProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.escrowProgram),
      getAccountMeta(accounts.cosigner),
      getAccountMeta(accounts.mintProof),
      getAccountMeta(accounts.rentDestination)
    ],
    programAddress,
    data: getTakeBidCoreInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseTakeBidCoreInstruction(instruction) {
  if (instruction.accounts.length < 17) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  const getNextOptionalAccount = () => {
    const accountMeta = getNextAccount();
    return accountMeta.address === TENSOR_MARKETPLACE_PROGRAM_ADDRESS ? void 0 : accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      feeVault: getNextAccount(),
      seller: getNextAccount(),
      bidState: getNextAccount(),
      owner: getNextAccount(),
      takerBroker: getNextOptionalAccount(),
      makerBroker: getNextOptionalAccount(),
      marginAccount: getNextAccount(),
      whitelist: getNextAccount(),
      asset: getNextAccount(),
      collection: getNextOptionalAccount(),
      mplCoreProgram: getNextAccount(),
      systemProgram: getNextAccount(),
      marketplaceProgram: getNextAccount(),
      escrowProgram: getNextAccount(),
      cosigner: getNextOptionalAccount(),
      mintProof: getNextAccount(),
      rentDestination: getNextAccount()
    },
    data: getTakeBidCoreInstructionDataDecoder().decode(instruction.data)
  };
}
function getTakeBidLegacyInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["minAmount", getU64Encoder()],
      ["optionalRoyaltyPct", getOptionEncoder(getU16Encoder())],
      ["rulesAccPresent", getBooleanEncoder()],
      [
        "authorizationData",
        getOptionEncoder(getAuthorizationDataLocalEncoder())
      ]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([188, 35, 116, 108, 0, 233, 237, 201]),
      optionalRoyaltyPct: value.optionalRoyaltyPct ?? none(),
      rulesAccPresent: value.rulesAccPresent ?? false,
      authorizationData: value.authorizationData ?? none()
    })
  );
}
function getTakeBidLegacyInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["minAmount", getU64Decoder()],
    ["optionalRoyaltyPct", getOptionDecoder(getU16Decoder())],
    ["rulesAccPresent", getBooleanDecoder()],
    ["authorizationData", getOptionDecoder(getAuthorizationDataLocalDecoder())]
  ]);
}
function getTakeBidLegacyInstructionDataCodec() {
  return combineCodec(
    getTakeBidLegacyInstructionDataEncoder(),
    getTakeBidLegacyInstructionDataDecoder()
  );
}
async function getTakeBidLegacyInstructionAsync(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    feeVault: { value: input.feeVault ?? null, isWritable: true },
    seller: { value: input.seller ?? null, isWritable: true },
    bidState: { value: input.bidState ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: true },
    takerBroker: { value: input.takerBroker ?? null, isWritable: true },
    makerBroker: { value: input.makerBroker ?? null, isWritable: true },
    sharedEscrow: { value: input.sharedEscrow ?? null, isWritable: true },
    whitelist: { value: input.whitelist ?? null, isWritable: false },
    sellerTa: { value: input.sellerTa ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    metadata: { value: input.metadata ?? null, isWritable: true },
    ownerTa: { value: input.ownerTa ?? null, isWritable: true },
    edition: { value: input.edition ?? null, isWritable: false },
    sellerTokenRecord: {
      value: input.sellerTokenRecord ?? null,
      isWritable: true
    },
    ownerTokenRecord: {
      value: input.ownerTokenRecord ?? null,
      isWritable: true
    },
    tokenMetadataProgram: {
      value: input.tokenMetadataProgram ?? null,
      isWritable: false
    },
    sysvarInstructions: {
      value: input.sysvarInstructions ?? null,
      isWritable: false
    },
    authorizationRulesProgram: {
      value: input.authorizationRulesProgram ?? null,
      isWritable: false
    },
    bidTa: { value: input.bidTa ?? null, isWritable: true },
    bidTokenRecord: { value: input.bidTokenRecord ?? null, isWritable: true },
    authorizationRules: {
      value: input.authorizationRules ?? null,
      isWritable: false
    },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    escrowProgram: { value: input.escrowProgram ?? null, isWritable: false },
    cosigner: { value: input.cosigner ?? null, isWritable: false },
    mintProof: { value: input.mintProof ?? null, isWritable: false },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.bidState.value) {
    accounts.bidState.value = await findBidStatePda({
      bidId: expectAddress(accounts.mint.value),
      owner: expectAddress(accounts.owner.value)
    });
  }
  if (!accounts.feeVault.value) {
    accounts.feeVault = {
      ...accounts.feeVault,
      ...await resolveFeeVaultPdaFromBidState(resolverScope)
    };
  }
  if (!accounts.sharedEscrow.value) {
    accounts.sharedEscrow.value = expectSome(accounts.owner.value);
  }
  if (!accounts.whitelist.value) {
    accounts.whitelist.value = "11111111111111111111111111111111";
  }
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.sellerTa.value) {
    accounts.sellerTa = {
      ...accounts.sellerTa,
      ...await resolveSellerAta(resolverScope)
    };
  }
  if (!accounts.metadata.value) {
    accounts.metadata = {
      ...accounts.metadata,
      ...await resolveMetadata(resolverScope)
    };
  }
  if (!accounts.ownerTa.value) {
    accounts.ownerTa = {
      ...accounts.ownerTa,
      ...await resolveOwnerAta(resolverScope)
    };
  }
  if (!args.tokenStandard) {
    args.tokenStandard = TokenStandard.ProgrammableNonFungible;
  }
  if (!accounts.edition.value) {
    accounts.edition = {
      ...accounts.edition,
      ...await resolveEditionFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.sellerTokenRecord.value) {
    accounts.sellerTokenRecord = {
      ...accounts.sellerTokenRecord,
      ...await resolveSellerTokenRecordFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.ownerTokenRecord.value) {
    accounts.ownerTokenRecord = {
      ...accounts.ownerTokenRecord,
      ...await resolveOwnerTokenRecordFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.tokenMetadataProgram.value) {
    accounts.tokenMetadataProgram = {
      ...accounts.tokenMetadataProgram,
      ...resolveTokenMetadataProgramFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.sysvarInstructions.value) {
    accounts.sysvarInstructions = {
      ...accounts.sysvarInstructions,
      ...resolveSysvarInstructionsFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.authorizationRulesProgram.value) {
    accounts.authorizationRulesProgram = {
      ...accounts.authorizationRulesProgram,
      ...resolveAuthorizationRulesProgramFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.bidTa.value) {
    accounts.bidTa = {
      ...accounts.bidTa,
      ...await resolveBidTa(resolverScope)
    };
  }
  if (!accounts.bidTokenRecord.value) {
    accounts.bidTokenRecord = {
      ...accounts.bidTokenRecord,
      ...await resolveBidTokenRecordFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.escrowProgram.value) {
    accounts.escrowProgram.value = "TSWAPaqyCSx2KABk68Shruf4rp7CxcNi8hAsbdwmHbN";
  }
  if (!accounts.rentDestination.value) {
    accounts.rentDestination.value = expectSome(accounts.owner.value);
  }
  const remainingAccounts = (args.creators ?? []).map(
    (address3) => ({ address: address3, role: AccountRole.WRITABLE })
  );
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.feeVault),
      getAccountMeta(accounts.seller),
      getAccountMeta(accounts.bidState),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.takerBroker),
      getAccountMeta(accounts.makerBroker),
      getAccountMeta(accounts.sharedEscrow),
      getAccountMeta(accounts.whitelist),
      getAccountMeta(accounts.sellerTa),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.metadata),
      getAccountMeta(accounts.ownerTa),
      getAccountMeta(accounts.edition),
      getAccountMeta(accounts.sellerTokenRecord),
      getAccountMeta(accounts.ownerTokenRecord),
      getAccountMeta(accounts.tokenMetadataProgram),
      getAccountMeta(accounts.sysvarInstructions),
      getAccountMeta(accounts.authorizationRulesProgram),
      getAccountMeta(accounts.bidTa),
      getAccountMeta(accounts.bidTokenRecord),
      getAccountMeta(accounts.authorizationRules),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.escrowProgram),
      getAccountMeta(accounts.cosigner),
      getAccountMeta(accounts.mintProof),
      getAccountMeta(accounts.rentDestination),
      ...remainingAccounts
    ],
    programAddress,
    data: getTakeBidLegacyInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function getTakeBidLegacyInstruction(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    feeVault: { value: input.feeVault ?? null, isWritable: true },
    seller: { value: input.seller ?? null, isWritable: true },
    bidState: { value: input.bidState ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: true },
    takerBroker: { value: input.takerBroker ?? null, isWritable: true },
    makerBroker: { value: input.makerBroker ?? null, isWritable: true },
    sharedEscrow: { value: input.sharedEscrow ?? null, isWritable: true },
    whitelist: { value: input.whitelist ?? null, isWritable: false },
    sellerTa: { value: input.sellerTa ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    metadata: { value: input.metadata ?? null, isWritable: true },
    ownerTa: { value: input.ownerTa ?? null, isWritable: true },
    edition: { value: input.edition ?? null, isWritable: false },
    sellerTokenRecord: {
      value: input.sellerTokenRecord ?? null,
      isWritable: true
    },
    ownerTokenRecord: {
      value: input.ownerTokenRecord ?? null,
      isWritable: true
    },
    tokenMetadataProgram: {
      value: input.tokenMetadataProgram ?? null,
      isWritable: false
    },
    sysvarInstructions: {
      value: input.sysvarInstructions ?? null,
      isWritable: false
    },
    authorizationRulesProgram: {
      value: input.authorizationRulesProgram ?? null,
      isWritable: false
    },
    bidTa: { value: input.bidTa ?? null, isWritable: true },
    bidTokenRecord: { value: input.bidTokenRecord ?? null, isWritable: true },
    authorizationRules: {
      value: input.authorizationRules ?? null,
      isWritable: false
    },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    escrowProgram: { value: input.escrowProgram ?? null, isWritable: false },
    cosigner: { value: input.cosigner ?? null, isWritable: false },
    mintProof: { value: input.mintProof ?? null, isWritable: false },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.sharedEscrow.value) {
    accounts.sharedEscrow.value = expectSome(accounts.owner.value);
  }
  if (!accounts.whitelist.value) {
    accounts.whitelist.value = "11111111111111111111111111111111";
  }
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!args.tokenStandard) {
    args.tokenStandard = TokenStandard.ProgrammableNonFungible;
  }
  if (!accounts.tokenMetadataProgram.value) {
    accounts.tokenMetadataProgram = {
      ...accounts.tokenMetadataProgram,
      ...resolveTokenMetadataProgramFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.sysvarInstructions.value) {
    accounts.sysvarInstructions = {
      ...accounts.sysvarInstructions,
      ...resolveSysvarInstructionsFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.authorizationRulesProgram.value) {
    accounts.authorizationRulesProgram = {
      ...accounts.authorizationRulesProgram,
      ...resolveAuthorizationRulesProgramFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.escrowProgram.value) {
    accounts.escrowProgram.value = "TSWAPaqyCSx2KABk68Shruf4rp7CxcNi8hAsbdwmHbN";
  }
  if (!accounts.rentDestination.value) {
    accounts.rentDestination.value = expectSome(accounts.owner.value);
  }
  const remainingAccounts = (args.creators ?? []).map(
    (address3) => ({ address: address3, role: AccountRole.WRITABLE })
  );
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.feeVault),
      getAccountMeta(accounts.seller),
      getAccountMeta(accounts.bidState),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.takerBroker),
      getAccountMeta(accounts.makerBroker),
      getAccountMeta(accounts.sharedEscrow),
      getAccountMeta(accounts.whitelist),
      getAccountMeta(accounts.sellerTa),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.metadata),
      getAccountMeta(accounts.ownerTa),
      getAccountMeta(accounts.edition),
      getAccountMeta(accounts.sellerTokenRecord),
      getAccountMeta(accounts.ownerTokenRecord),
      getAccountMeta(accounts.tokenMetadataProgram),
      getAccountMeta(accounts.sysvarInstructions),
      getAccountMeta(accounts.authorizationRulesProgram),
      getAccountMeta(accounts.bidTa),
      getAccountMeta(accounts.bidTokenRecord),
      getAccountMeta(accounts.authorizationRules),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.escrowProgram),
      getAccountMeta(accounts.cosigner),
      getAccountMeta(accounts.mintProof),
      getAccountMeta(accounts.rentDestination),
      ...remainingAccounts
    ],
    programAddress,
    data: getTakeBidLegacyInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseTakeBidLegacyInstruction(instruction) {
  if (instruction.accounts.length < 29) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  const getNextOptionalAccount = () => {
    const accountMeta = getNextAccount();
    return accountMeta.address === TENSOR_MARKETPLACE_PROGRAM_ADDRESS ? void 0 : accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      feeVault: getNextAccount(),
      seller: getNextAccount(),
      bidState: getNextAccount(),
      owner: getNextAccount(),
      takerBroker: getNextOptionalAccount(),
      makerBroker: getNextOptionalAccount(),
      sharedEscrow: getNextAccount(),
      whitelist: getNextOptionalAccount(),
      sellerTa: getNextAccount(),
      mint: getNextAccount(),
      metadata: getNextAccount(),
      ownerTa: getNextAccount(),
      edition: getNextAccount(),
      sellerTokenRecord: getNextOptionalAccount(),
      ownerTokenRecord: getNextOptionalAccount(),
      tokenMetadataProgram: getNextOptionalAccount(),
      sysvarInstructions: getNextOptionalAccount(),
      authorizationRulesProgram: getNextOptionalAccount(),
      bidTa: getNextAccount(),
      bidTokenRecord: getNextOptionalAccount(),
      authorizationRules: getNextOptionalAccount(),
      tokenProgram: getNextAccount(),
      associatedTokenProgram: getNextAccount(),
      systemProgram: getNextAccount(),
      marketplaceProgram: getNextAccount(),
      escrowProgram: getNextAccount(),
      cosigner: getNextOptionalAccount(),
      mintProof: getNextOptionalAccount(),
      rentDestination: getNextAccount()
    },
    data: getTakeBidLegacyInstructionDataDecoder().decode(instruction.data)
  };
}
function getTakeBidT22InstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["minAmount", getU64Encoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([18, 250, 113, 242, 31, 244, 19, 150])
    })
  );
}
function getTakeBidT22InstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["minAmount", getU64Decoder()]
  ]);
}
function getTakeBidT22InstructionDataCodec() {
  return combineCodec(
    getTakeBidT22InstructionDataEncoder(),
    getTakeBidT22InstructionDataDecoder()
  );
}
function getTakeBidT22Instruction(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    feeVault: { value: input.feeVault ?? null, isWritable: true },
    seller: { value: input.seller ?? null, isWritable: true },
    bidState: { value: input.bidState ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: true },
    takerBroker: { value: input.takerBroker ?? null, isWritable: true },
    makerBroker: { value: input.makerBroker ?? null, isWritable: true },
    marginAccount: { value: input.marginAccount ?? null, isWritable: true },
    whitelist: { value: input.whitelist ?? null, isWritable: false },
    sellerTa: { value: input.sellerTa ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    ownerTa: { value: input.ownerTa ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    escrowProgram: { value: input.escrowProgram ?? null, isWritable: false },
    cosigner: { value: input.cosigner ?? null, isWritable: false },
    mintProof: { value: input.mintProof ?? null, isWritable: false },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.escrowProgram.value) {
    accounts.escrowProgram.value = "TSWAPaqyCSx2KABk68Shruf4rp7CxcNi8hAsbdwmHbN";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.feeVault),
      getAccountMeta(accounts.seller),
      getAccountMeta(accounts.bidState),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.takerBroker),
      getAccountMeta(accounts.makerBroker),
      getAccountMeta(accounts.marginAccount),
      getAccountMeta(accounts.whitelist),
      getAccountMeta(accounts.sellerTa),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.ownerTa),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.escrowProgram),
      getAccountMeta(accounts.cosigner),
      getAccountMeta(accounts.mintProof),
      getAccountMeta(accounts.rentDestination)
    ],
    programAddress,
    data: getTakeBidT22InstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseTakeBidT22Instruction(instruction) {
  if (instruction.accounts.length < 19) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  const getNextOptionalAccount = () => {
    const accountMeta = getNextAccount();
    return accountMeta.address === TENSOR_MARKETPLACE_PROGRAM_ADDRESS ? void 0 : accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      feeVault: getNextAccount(),
      seller: getNextAccount(),
      bidState: getNextAccount(),
      owner: getNextAccount(),
      takerBroker: getNextOptionalAccount(),
      makerBroker: getNextOptionalAccount(),
      marginAccount: getNextAccount(),
      whitelist: getNextAccount(),
      sellerTa: getNextAccount(),
      mint: getNextAccount(),
      ownerTa: getNextAccount(),
      tokenProgram: getNextAccount(),
      associatedTokenProgram: getNextAccount(),
      systemProgram: getNextAccount(),
      marketplaceProgram: getNextAccount(),
      escrowProgram: getNextAccount(),
      cosigner: getNextOptionalAccount(),
      mintProof: getNextAccount(),
      rentDestination: getNextAccount()
    },
    data: getTakeBidT22InstructionDataDecoder().decode(instruction.data)
  };
}
function getTakeBidWnsInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["minAmount", getU64Encoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([88, 5, 122, 88, 250, 139, 35, 216])
    })
  );
}
function getTakeBidWnsInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["minAmount", getU64Decoder()]
  ]);
}
function getTakeBidWnsInstructionDataCodec() {
  return combineCodec(
    getTakeBidWnsInstructionDataEncoder(),
    getTakeBidWnsInstructionDataDecoder()
  );
}
function getTakeBidWnsInstruction(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    feeVault: { value: input.feeVault ?? null, isWritable: true },
    seller: { value: input.seller ?? null, isWritable: true },
    bidState: { value: input.bidState ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: true },
    takerBroker: { value: input.takerBroker ?? null, isWritable: true },
    makerBroker: { value: input.makerBroker ?? null, isWritable: true },
    marginAccount: { value: input.marginAccount ?? null, isWritable: true },
    whitelist: { value: input.whitelist ?? null, isWritable: false },
    sellerTa: { value: input.sellerTa ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    ownerTa: { value: input.ownerTa ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    marketplaceProgram: {
      value: input.marketplaceProgram ?? null,
      isWritable: false
    },
    escrowProgram: { value: input.escrowProgram ?? null, isWritable: false },
    cosigner: { value: input.cosigner ?? null, isWritable: false },
    mintProof: { value: input.mintProof ?? null, isWritable: false },
    rentDestination: { value: input.rentDestination ?? null, isWritable: true },
    approveAccount: { value: input.approveAccount ?? null, isWritable: true },
    distribution: { value: input.distribution ?? null, isWritable: true },
    wnsProgram: { value: input.wnsProgram ?? null, isWritable: false },
    distributionProgram: {
      value: input.distributionProgram ?? null,
      isWritable: false
    },
    extraMetas: { value: input.extraMetas ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.marketplaceProgram.value) {
    accounts.marketplaceProgram.value = "TCMPhJdwDryooaGtiocG1u3xcYbRpiJzb283XfCZsDp";
  }
  if (!accounts.escrowProgram.value) {
    accounts.escrowProgram.value = "TSWAPaqyCSx2KABk68Shruf4rp7CxcNi8hAsbdwmHbN";
  }
  if (!accounts.wnsProgram.value) {
    accounts.wnsProgram.value = "wns1gDLt8fgLcGhWi5MqAqgXpwEP1JftKE9eZnXS1HM";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.feeVault),
      getAccountMeta(accounts.seller),
      getAccountMeta(accounts.bidState),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.takerBroker),
      getAccountMeta(accounts.makerBroker),
      getAccountMeta(accounts.marginAccount),
      getAccountMeta(accounts.whitelist),
      getAccountMeta(accounts.sellerTa),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.ownerTa),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.marketplaceProgram),
      getAccountMeta(accounts.escrowProgram),
      getAccountMeta(accounts.cosigner),
      getAccountMeta(accounts.mintProof),
      getAccountMeta(accounts.rentDestination),
      getAccountMeta(accounts.approveAccount),
      getAccountMeta(accounts.distribution),
      getAccountMeta(accounts.wnsProgram),
      getAccountMeta(accounts.distributionProgram),
      getAccountMeta(accounts.extraMetas)
    ],
    programAddress,
    data: getTakeBidWnsInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseTakeBidWnsInstruction(instruction) {
  if (instruction.accounts.length < 24) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  const getNextOptionalAccount = () => {
    const accountMeta = getNextAccount();
    return accountMeta.address === TENSOR_MARKETPLACE_PROGRAM_ADDRESS ? void 0 : accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      feeVault: getNextAccount(),
      seller: getNextAccount(),
      bidState: getNextAccount(),
      owner: getNextAccount(),
      takerBroker: getNextOptionalAccount(),
      makerBroker: getNextOptionalAccount(),
      marginAccount: getNextAccount(),
      whitelist: getNextAccount(),
      sellerTa: getNextAccount(),
      mint: getNextAccount(),
      ownerTa: getNextAccount(),
      tokenProgram: getNextAccount(),
      associatedTokenProgram: getNextAccount(),
      systemProgram: getNextAccount(),
      marketplaceProgram: getNextAccount(),
      escrowProgram: getNextAccount(),
      cosigner: getNextOptionalAccount(),
      mintProof: getNextAccount(),
      rentDestination: getNextAccount(),
      approveAccount: getNextAccount(),
      distribution: getNextAccount(),
      wnsProgram: getNextAccount(),
      distributionProgram: getNextAccount(),
      extraMetas: getNextAccount()
    },
    data: getTakeBidWnsInstructionDataDecoder().decode(instruction.data)
  };
}
function getTcompNoopInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["event", getTcompEventEncoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([106, 162, 10, 226, 132, 68, 223, 21])
    })
  );
}
function getTcompNoopInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["event", getTcompEventDecoder()]
  ]);
}
function getTcompNoopInstructionDataCodec() {
  return combineCodec(
    getTcompNoopInstructionDataEncoder(),
    getTcompNoopInstructionDataDecoder()
  );
}
function getTcompNoopInstruction(input) {
  const programAddress = TENSOR_MARKETPLACE_PROGRAM_ADDRESS;
  const originalAccounts = {
    tcompSigner: { value: input.tcompSigner ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [getAccountMeta(accounts.tcompSigner)],
    programAddress,
    data: getTcompNoopInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseTcompNoopInstruction(instruction) {
  if (instruction.accounts.length < 1) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      tcompSigner: getNextAccount()
    },
    data: getTcompNoopInstructionDataDecoder().decode(instruction.data)
  };
}

export { Field, TENSOR_MARKETPLACE_ERROR__ARITHMETIC_ERROR, TENSOR_MARKETPLACE_ERROR__ASSET_ID_MISMATCH, TENSOR_MARKETPLACE_ERROR__BAD_BID_FIELD, TENSOR_MARKETPLACE_ERROR__BAD_COSIGNER, TENSOR_MARKETPLACE_ERROR__BAD_LIST_STATE, TENSOR_MARKETPLACE_ERROR__BAD_MARGIN, TENSOR_MARKETPLACE_ERROR__BAD_MINT_PROOF, TENSOR_MARKETPLACE_ERROR__BAD_OWNER, TENSOR_MARKETPLACE_ERROR__BAD_QUANTITY, TENSOR_MARKETPLACE_ERROR__BAD_RENT_DEST, TENSOR_MARKETPLACE_ERROR__BAD_ROYALTIES_PCT, TENSOR_MARKETPLACE_ERROR__BAD_WHITELIST, TENSOR_MARKETPLACE_ERROR__BID_BALANCE_NOT_EMPTIED, TENSOR_MARKETPLACE_ERROR__BID_EXPIRED, TENSOR_MARKETPLACE_ERROR__BID_FULLY_FILLED, TENSOR_MARKETPLACE_ERROR__BID_NOT_YET_EXPIRED, TENSOR_MARKETPLACE_ERROR__BROKER_MISMATCH, TENSOR_MARKETPLACE_ERROR__CANNOT_MODIFY_TARGET, TENSOR_MARKETPLACE_ERROR__CREATOR_MISMATCH, TENSOR_MARKETPLACE_ERROR__CURRENCY_MISMATCH, TENSOR_MARKETPLACE_ERROR__CURRENCY_NOT_YET_ENABLED, TENSOR_MARKETPLACE_ERROR__CURRENCY_NOT_YET_WHITELISTED, TENSOR_MARKETPLACE_ERROR__EXPIRY_TOO_LARGE, TENSOR_MARKETPLACE_ERROR__FORBIDDEN_COLLECTION, TENSOR_MARKETPLACE_ERROR__INSUFFICIENT_BALANCE, TENSOR_MARKETPLACE_ERROR__INSUFFICIENT_REMAINING_ACCOUNTS, TENSOR_MARKETPLACE_ERROR__INVALID_FEE_ACCOUNT, TENSOR_MARKETPLACE_ERROR__INVALID_TOKEN_ACCOUNT, TENSOR_MARKETPLACE_ERROR__LISTING_EXPIRED, TENSOR_MARKETPLACE_ERROR__LISTING_NOT_YET_EXPIRED, TENSOR_MARKETPLACE_ERROR__MAKER_BROKER_NOT_YET_ENABLED, TENSOR_MARKETPLACE_ERROR__MAKER_BROKER_NOT_YET_WHITELISTED, TENSOR_MARKETPLACE_ERROR__MISSING_BROKER, TENSOR_MARKETPLACE_ERROR__MISSING_BROKER_TOKEN_ACCOUNT, TENSOR_MARKETPLACE_ERROR__MISSING_COLLECTION, TENSOR_MARKETPLACE_ERROR__MISSING_FVC, TENSOR_MARKETPLACE_ERROR__OPTIONAL_ROYALTIES_NOT_YET_ENABLED, TENSOR_MARKETPLACE_ERROR__PRICE_MISMATCH, TENSOR_MARKETPLACE_ERROR__TAKER_NOT_ALLOWED, TENSOR_MARKETPLACE_ERROR__TARGET_ID_MUST_EQUAL_BID_ID, TENSOR_MARKETPLACE_ERROR__WRONG_BID_FIELD_ID, TENSOR_MARKETPLACE_ERROR__WRONG_IX_FOR_BID_TARGET, TENSOR_MARKETPLACE_ERROR__WRONG_STATE_VERSION, TENSOR_MARKETPLACE_ERROR__WRONG_TARGET_ID, TENSOR_MARKETPLACE_ERROR__WRONG_TOKEN_RECORD_DERIVATION, TENSOR_MARKETPLACE_PROGRAM_ADDRESS, TTokenProgramVersion, TTokenStandard, TUseMethod, Target, TensorMarketplaceAccount, TensorMarketplaceInstruction, decodeBidState, decodeBidTa, decodeListState, fetchAllBidState, fetchAllBidTa, fetchAllListState, fetchAllMaybeBidState, fetchAllMaybeBidTa, fetchAllMaybeListState, fetchBidState, fetchBidStateFromSeeds, fetchBidTa, fetchBidTaFromSeeds, fetchListState, fetchListStateFromSeeds, fetchMaybeBidState, fetchMaybeBidStateFromSeeds, fetchMaybeBidTa, fetchMaybeBidTaFromSeeds, fetchMaybeListState, fetchMaybeListStateFromSeeds, findBidStatePda, findBidTaPda, findFeeVaultPda, findListStatePda, findTreeAuthorityPda, getAuthorizationDataLocalCodec, getAuthorizationDataLocalDecoder, getAuthorizationDataLocalEncoder, getBidInstruction, getBidInstructionAsync, getBidInstructionDataCodec, getBidInstructionDataDecoder, getBidInstructionDataEncoder, getBidStateCodec, getBidStateDecoder, getBidStateEncoder, getBidTaCodec, getBidTaDecoder, getBidTaEncoder, getBidTaSize, getBuyCompressedInstruction, getBuyCompressedInstructionAsync, getBuyCompressedInstructionDataCodec, getBuyCompressedInstructionDataDecoder, getBuyCompressedInstructionDataEncoder, getBuyCoreInstruction, getBuyCoreInstructionDataCodec, getBuyCoreInstructionDataDecoder, getBuyCoreInstructionDataEncoder, getBuyCoreSplInstruction, getBuyCoreSplInstructionDataCodec, getBuyCoreSplInstructionDataDecoder, getBuyCoreSplInstructionDataEncoder, getBuyLegacyInstruction, getBuyLegacyInstructionAsync, getBuyLegacyInstructionDataCodec, getBuyLegacyInstructionDataDecoder, getBuyLegacyInstructionDataEncoder, getBuyLegacySplInstruction, getBuyLegacySplInstructionAsync, getBuyLegacySplInstructionDataCodec, getBuyLegacySplInstructionDataDecoder, getBuyLegacySplInstructionDataEncoder, getBuySplInstruction, getBuySplInstructionAsync, getBuySplInstructionDataCodec, getBuySplInstructionDataDecoder, getBuySplInstructionDataEncoder, getBuyT22Instruction, getBuyT22InstructionAsync, getBuyT22InstructionDataCodec, getBuyT22InstructionDataDecoder, getBuyT22InstructionDataEncoder, getBuyT22SplInstruction, getBuyT22SplInstructionAsync, getBuyT22SplInstructionDataCodec, getBuyT22SplInstructionDataDecoder, getBuyT22SplInstructionDataEncoder, getBuyWnsInstruction, getBuyWnsInstructionAsync, getBuyWnsInstructionDataCodec, getBuyWnsInstructionDataDecoder, getBuyWnsInstructionDataEncoder, getBuyWnsSplInstruction, getBuyWnsSplInstructionDataCodec, getBuyWnsSplInstructionDataDecoder, getBuyWnsSplInstructionDataEncoder, getCancelBidInstruction, getCancelBidInstructionAsync, getCancelBidInstructionDataCodec, getCancelBidInstructionDataDecoder, getCancelBidInstructionDataEncoder, getCloseExpiredBidInstruction, getCloseExpiredBidInstructionAsync, getCloseExpiredBidInstructionDataCodec, getCloseExpiredBidInstructionDataDecoder, getCloseExpiredBidInstructionDataEncoder, getCloseExpiredListingCompressedInstruction, getCloseExpiredListingCompressedInstructionAsync, getCloseExpiredListingCompressedInstructionDataCodec, getCloseExpiredListingCompressedInstructionDataDecoder, getCloseExpiredListingCompressedInstructionDataEncoder, getCloseExpiredListingCoreInstruction, getCloseExpiredListingCoreInstructionDataCodec, getCloseExpiredListingCoreInstructionDataDecoder, getCloseExpiredListingCoreInstructionDataEncoder, getCloseExpiredListingLegacyInstruction, getCloseExpiredListingLegacyInstructionAsync, getCloseExpiredListingLegacyInstructionDataCodec, getCloseExpiredListingLegacyInstructionDataDecoder, getCloseExpiredListingLegacyInstructionDataEncoder, getCloseExpiredListingT22Instruction, getCloseExpiredListingT22InstructionAsync, getCloseExpiredListingT22InstructionDataCodec, getCloseExpiredListingT22InstructionDataDecoder, getCloseExpiredListingT22InstructionDataEncoder, getCloseExpiredListingWnsInstruction, getCloseExpiredListingWnsInstructionAsync, getCloseExpiredListingWnsInstructionDataCodec, getCloseExpiredListingWnsInstructionDataDecoder, getCloseExpiredListingWnsInstructionDataEncoder, getDelistCompressedInstruction, getDelistCompressedInstructionAsync, getDelistCompressedInstructionDataCodec, getDelistCompressedInstructionDataDecoder, getDelistCompressedInstructionDataEncoder, getDelistCoreInstruction, getDelistCoreInstructionDataCodec, getDelistCoreInstructionDataDecoder, getDelistCoreInstructionDataEncoder, getDelistLegacyInstruction, getDelistLegacyInstructionAsync, getDelistLegacyInstructionDataCodec, getDelistLegacyInstructionDataDecoder, getDelistLegacyInstructionDataEncoder, getDelistT22Instruction, getDelistT22InstructionAsync, getDelistT22InstructionDataCodec, getDelistT22InstructionDataDecoder, getDelistT22InstructionDataEncoder, getDelistWnsInstruction, getDelistWnsInstructionAsync, getDelistWnsInstructionDataCodec, getDelistWnsInstructionDataDecoder, getDelistWnsInstructionDataEncoder, getEditInstruction, getEditInstructionDataCodec, getEditInstructionDataDecoder, getEditInstructionDataEncoder, getFieldCodec, getFieldDecoder, getFieldEncoder, getListCompressedInstruction, getListCompressedInstructionAsync, getListCompressedInstructionDataCodec, getListCompressedInstructionDataDecoder, getListCompressedInstructionDataEncoder, getListCoreInstruction, getListCoreInstructionDataCodec, getListCoreInstructionDataDecoder, getListCoreInstructionDataEncoder, getListLegacyInstruction, getListLegacyInstructionAsync, getListLegacyInstructionDataCodec, getListLegacyInstructionDataDecoder, getListLegacyInstructionDataEncoder, getListStateCodec, getListStateDecoder, getListStateEncoder, getListT22Instruction, getListT22InstructionAsync, getListT22InstructionDataCodec, getListT22InstructionDataDecoder, getListT22InstructionDataEncoder, getListWnsInstruction, getListWnsInstructionAsync, getListWnsInstructionDataCodec, getListWnsInstructionDataDecoder, getListWnsInstructionDataEncoder, getMakeEventCodec, getMakeEventDecoder, getMakeEventEncoder, getNullableAddressCodec, getNullableAddressDecoder, getNullableAddressEncoder, getPayloadTypeLocalCodec, getPayloadTypeLocalDecoder, getPayloadTypeLocalEncoder, getProofInfoLocalCodec, getProofInfoLocalDecoder, getProofInfoLocalEncoder, getSeedsVecLocalCodec, getSeedsVecLocalDecoder, getSeedsVecLocalEncoder, getTCollectionCodec, getTCollectionDecoder, getTCollectionEncoder, getTTokenProgramVersionCodec, getTTokenProgramVersionDecoder, getTTokenProgramVersionEncoder, getTTokenStandardCodec, getTTokenStandardDecoder, getTTokenStandardEncoder, getTUseMethodCodec, getTUseMethodDecoder, getTUseMethodEncoder, getTUsesCodec, getTUsesDecoder, getTUsesEncoder, getTaggedPayloadCodec, getTaggedPayloadDecoder, getTaggedPayloadEncoder, getTakeBidCompressedFullMetaInstruction, getTakeBidCompressedFullMetaInstructionAsync, getTakeBidCompressedFullMetaInstructionDataCodec, getTakeBidCompressedFullMetaInstructionDataDecoder, getTakeBidCompressedFullMetaInstructionDataEncoder, getTakeBidCompressedMetaHashInstruction, getTakeBidCompressedMetaHashInstructionAsync, getTakeBidCompressedMetaHashInstructionDataCodec, getTakeBidCompressedMetaHashInstructionDataDecoder, getTakeBidCompressedMetaHashInstructionDataEncoder, getTakeBidCoreInstruction, getTakeBidCoreInstructionDataCodec, getTakeBidCoreInstructionDataDecoder, getTakeBidCoreInstructionDataEncoder, getTakeBidLegacyInstruction, getTakeBidLegacyInstructionAsync, getTakeBidLegacyInstructionDataCodec, getTakeBidLegacyInstructionDataDecoder, getTakeBidLegacyInstructionDataEncoder, getTakeBidT22Instruction, getTakeBidT22InstructionDataCodec, getTakeBidT22InstructionDataDecoder, getTakeBidT22InstructionDataEncoder, getTakeBidWnsInstruction, getTakeBidWnsInstructionDataCodec, getTakeBidWnsInstructionDataDecoder, getTakeBidWnsInstructionDataEncoder, getTakeEventCodec, getTakeEventDecoder, getTakeEventEncoder, getTargetCodec, getTargetDecoder, getTargetEncoder, getTcompEventCodec, getTcompEventDecoder, getTcompEventEncoder, getTcompNoopInstruction, getTcompNoopInstructionDataCodec, getTcompNoopInstructionDataDecoder, getTcompNoopInstructionDataEncoder, getTensorMarketplaceErrorMessage, identifyTensorMarketplaceAccount, identifyTensorMarketplaceInstruction, isPayloadTypeLocal, isTcompEvent, parseBidInstruction, parseBuyCompressedInstruction, parseBuyCoreInstruction, parseBuyCoreSplInstruction, parseBuyLegacyInstruction, parseBuyLegacySplInstruction, parseBuySplInstruction, parseBuyT22Instruction, parseBuyT22SplInstruction, parseBuyWnsInstruction, parseBuyWnsSplInstruction, parseCancelBidInstruction, parseCloseExpiredBidInstruction, parseCloseExpiredListingCompressedInstruction, parseCloseExpiredListingCoreInstruction, parseCloseExpiredListingLegacyInstruction, parseCloseExpiredListingT22Instruction, parseCloseExpiredListingWnsInstruction, parseDelistCompressedInstruction, parseDelistCoreInstruction, parseDelistLegacyInstruction, parseDelistT22Instruction, parseDelistWnsInstruction, parseEditInstruction, parseListCompressedInstruction, parseListCoreInstruction, parseListLegacyInstruction, parseListT22Instruction, parseListWnsInstruction, parseTakeBidCompressedFullMetaInstruction, parseTakeBidCompressedMetaHashInstruction, parseTakeBidCoreInstruction, parseTakeBidLegacyInstruction, parseTakeBidT22Instruction, parseTakeBidWnsInstruction, parseTcompNoopInstruction, payloadTypeLocal, resolveBidIdOnCreate, resolveBidTa, resolveCreatorPath, resolveFeeVaultPdaFromBidState, resolveFeeVaultPdaFromListState, resolveProofPath, resolveRemainingSignerWithOwnerOrDelegate, resolveRemainingSignerWithSellerOrDelegate, resolveTreeAuthorityPda, tcompEvent };
//# sourceMappingURL=index.mjs.map
//# sourceMappingURL=index.mjs.map